Package: Category
Title: Category Analysis
Version: 2.79.0
Authors@R: c(
    person("Robert", "Gentleman", role="aut"),
    person("Seth", "Falcon", role="ctb"),
    person("Deepayan", "Sarkar", role="ctb"),
    person("Robert", "Castelo", role="ctb"),
    person("Bioconductor Package Maintainer",
        email = "maintainer@bioconductor.org", role="cre"))
Description: A collection of tools for performing category (gene set
    enrichment) analysis.
License: Artistic-2.0
Depends: methods, stats4, BiocGenerics, AnnotationDbi, Biobase, Matrix
Imports: utils, stats, graph, RBGL, GSEABase, genefilter, annotate, DBI
Suggests: EBarrays, ALL, Rgraphviz, RColorBrewer, xtable (>= 1.4-6),
        hgu95av2.db, KEGGREST, karyoploteR, geneplotter, limma,
        lattice, RUnit, org.Sc.sgd.db, GOstats, GO.db
LazyLoad: Yes
Collate: AllClasses.R AllGenerics.R categoryToEntrezBuilder-methods.R
        categoryName-methods.R hyperg-methods.R hyperGTest-methods.R
        linearMTest-methods.R DatPkg-accessors.R ChrBandTree.R
        tree_visitor.R cb_test.R initialize-methods.R
        HyperGParams-accessors.R LinearMParams-accessors.R
        HyperGResult-accessors.R LinearMResult-accessors.R
        ChrMapHyperGResult-accessors.R ChrMapLinearMResult-accessors.R
        show-methods.R universeBuilder-methods.R utils.R MAPcode.R
        catcode.R cateGOryMatrix.R gseaperm.R summary-methods.R mouse.R
biocViews: Annotation, GO, Pathways, GeneSetEnrichment
git_url: https://git.bioconductor.org/packages/Category
git_branch: devel
git_last_commit: dadddc8
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-06-01
NeedsCompilation: no
Packaged: 2026-06-01 19:54:24 UTC; biocbuild
Author: Robert Gentleman [aut],
  Seth Falcon [ctb],
  Deepayan Sarkar [ctb],
  Robert Castelo [ctb],
  Bioconductor Package Maintainer [cre]
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>
Built: R 4.6.0; ; 2026-06-02 03:44:37 UTC; unix

import(methods)
importFrom(utils, find, tail)
importFrom(stats, chisq.test, coef, formula, lm, phyper, pnorm, runif,
           setNames)
importFrom(stats4, summary)
importFrom(Matrix, Matrix)

importFrom(BiocGenerics,
           annotation,
           "annotation<-",
           organism)

importClassesFrom(graph,
                  graph)

importClassesFrom(GSEABase,
                  OBOCollection)

importMethodsFrom(AnnotationDbi,
                  as.list,
                  colnames,
                  dbmeta,
                  eapply,
                  exists,
                  get,
                  ls,
                  mget,
                  ncol,
                  nrow,
                  revmap,
                  sample,
                  keys,
                  select,
                  dbconn,
                  columns)

importMethodsFrom(Biobase,
                  annotation,
                  "annotation<-",
                  description)

importMethodsFrom(GSEABase,
                  geneIds,
                  "geneIds<-",
                  ontology,
                  geneIdType,
                  incidence,
                  GeneSetCollection,
                  GeneSet)

importMethodsFrom(annotate,
                  organism)

importMethodsFrom(genefilter,
                  rowttests)

importMethodsFrom(graph,
                  edges,
                  leaves,
                  nodeData,
                  "nodeData<-",
                  "nodeDataDefaults<-",
                  nodes,
                  numNodes,
                  subGraph)

importMethodsFrom(DBI, dbGetQuery, dbListFields)

importMethodsFrom(Matrix,
                  as.vector,
                  "%*%",
                  dimnames,
                  "[<-")

importFrom(AnnotationDbi,
           toSQLStringSet)

importFrom(Biobase,
           contents,
           lcPrefix,
           lcPrefixC,
           listLen,
           reverseSplit,
           subListExtract)

importFrom(GSEABase,
           EntrezIdentifier)

importFrom(RBGL,
           dijkstra.sp,
           tsort)

importFrom(annotate,
           allValidKeys,
           aqListGOIDs,
           filterGOByOntology,
           getAnnMap)

importFrom(graph,
           ftM2graphNEL,
           reverseEdgeDirections)

export(
       ".doHyperGTest",
       "applyByCategory",
       "cateGOry",
       "cb_children",
       "cb_contingency",
       "cb_parse_band_Hs",
       "cb_parse_band_Mm",
       "cb_sigBands",
       "cb_test",
       "ChrBandTreeFromGraph",
       "DatPkgFactory",
       "exampleLevels",
       "findAMstats",
       "geneIds",
       "geneIds<-",
       "getPathNames",
       "gseattperm",
       "hyperg",
       "makeChrBandGraph",
       "makeChrBandInciMat",
       "makeChrBandGSC",
       "makeEBcontr",
       "NewChrBandTree",
       "MAPAmat",
       "ontology",
       "probes2MAP",
       "probes2Path",
       "ttperm",
       "OBOCollectionDatPkg",
       "OBOHyperGParams",
       "GSEAGOHyperGParams",
       "GSEAKEGGHyperGParams")

exportClasses("DatPkg",
              "AffyDatPkg",
              "YeastDatPkg",
              "Org.XX.egDatPkg",
              "OBOCollectionDatPkg",
              "HyperGParams",
              "GOHyperGParams",
              "KEGGHyperGParams",
              "PFAMHyperGParams",
              "ChrMapHyperGParams",

              "HyperGResultBase",
              "HyperGResult",
              "ChrMapHyperGResult",
              "ChrBandTree",
              "KEGGHyperGResult",
              "PFAMHyperGResult",

              "LinearMParams",
              "ChrMapLinearMParams",

              "LinearMResultBase",
              "LinearMResult",
              "ChrMapLinearMResult")

exportMethods("hyperGTest",
              "linearMTest",

              "universeBuilder",
              "categoryToEntrezBuilder",

              "categoryName",
              "makeValidParams",

              "organism",

              ## accessors for HyperGResult and LinearMResult
              "chrGraph",
              "pvalues",
              "effectSize",
              "geneCounts",
              "universeCounts",
              "universeMappedCount",
              "geneMappedCount",
              "annotation",
              "annotation<-",
              "DatPkgFactory",
              "geneIds",
              "geneIds<-",
              "geneIdsByCategory",
              "geneIdUniverse",
              "testName",
              "pvalueCutoff",
              "pvalueCutoff<-",
              "testDirection",
              "testDirection<-",
              "description",
              "universeGeneIds",
              "ontology",
              "ontology<-",
              "conditional",
              "conditional<-",
              "oddsRatios",
              "expectedCounts",
              "htmlReport",
              "sigCategories",

              ## accessors for DatPkg
              "ID2GO",
              "ID2EntrezID",
              "GO2AllProbes",
              "ID2KEGG",
              "KEGG2AllProbes",
              
              "initialize",
              "show",
              "summary",

              ## accessors for ChrBandTree
              "allGeneIds",
              "childrenOf",
              "level2nodes",
              "lgeneIds",
              "parentOf",
              "treeLevels")

# category
> Categorical transformation for data science

[![PyPI version](https://img.shields.io/pypi/v/tai-chi-engine)](https://pypi.org/project/category)
![Python version](https://img.shields.io/pypi/pyversions/category)
![License](https://img.shields.io/github/license/raynardj/category)
![PyPI Downloads](https://img.shields.io/pypi/dm/category)

## Installation
pip install works for this library.

```shell
pip install category
```

## Single Category
```python
>>> from category import Category
>>> book = Category(['a', 'b', 'c', 'Category_d', 'e', 'f', 'g', 'h', 'i', 'j'], pad_mst = False)
>>> book.i2c[2]
'c'

>>> book.c2i[['Category_d','f']]
array([3, 5])
```

You can set ```pad_mst``` to ```True``` to handle the missing token
```python
>>> from category import Category
>>> book = Category(['a', 'b', 'c', 'Category_d', 'e', 'f', 'g', 'h', 'i', 'j'], pad_mst = True)
>>> book.i2c[2] # the 1st token is the missing token, not 'a' any more
'b'
>>> book.c2i[['Stranger','Category_d','Unknown','f']]
array([0, 4, 0, 6])
```

library(CompStatViz)

cache <- function(name, expr) {
  cachefile <- paste("tmp-", name, ".RData", sep="")
  if(file.exists(cachefile)) {
    load(cachefile)
  } else {
    assign(name, expr)
    save(list=name, file=cachefile)
  }
  get(name)
}


CLL.exprs <- cache("CLL.exprs", gcrma(CLLbatch))
sampleNames(CLL.exprs) <- gsub("\\.CEL", "", sampleNames(CLL.exprs))
data(disease)
pD <- new("AnnotatedDataFrame")
pData(pD) <- disease
varLabels(pD) <- list(SampleID = "Sample ID", Disease = "Stable/Progressive")
phenoData(CLL.exprs) <- pD
f1 <- pOverA(.25, log2(100))
f2 <- function(x)(IQR(x)>0.5)
ff <- filterfun(f1,f2)
selected <- genefilter(CLL.exprs, ff)
sum(selected)
goodCell <- !is.na(CLL.exprs$Disease)

CLL.sub <- CLL.exprs[selected,goodCell]


rwtt <- rowttests(CLL.sub, CLL.sub$Disease)

library("Category")

fL <- findLargest(geneNames(CLL.sub), abs(rwtt$statistic), "hgu95av2")

fL2 <- probes2MAP(fL, "hgu95av2")
length(fL2)
inBoth <- fL %in% names(fL2)
fL <- fL[inBoth]


eS <- CLL.sub[match(fL, geneNames(CLL.sub)),]

tobs = rowttests(eS, "Disease")


z1 <- MAPAmat("hgu95av2")

z2 <- t(z1)

whichCol <- match(names(fL), dimnames(z2)[[2]], 0)

z2ER <- z2[, whichCol]

##now we have found which elements map to the genes we have
rs <- rowSums(z2ER)
##drop categories with less than 3
z2ER2 <- z2ER[rs>3,]
rs2 <- rs[rs>3]
nCats <- length(rs2)


m2 <- match(dimnames(z2ER2)[[2]], names(fL))

tA <- z2ER2 %*% (tobs$statistic[m2])
dim(tA) <- NULL
tA <- tA/sqrt(rs2)
names(tA) <- row.names(z2ER2)

qqnorm(tA)

require("RUnit", quietly=TRUE) || stop("RUnit package not found")
require("Category")


TEST_DATA_DIR <- "data"
runitPat <- ".*_test\\.R$"
runitDirs <- c(".")
suite <- defineTestSuite(name="Category Test Suite",
                         dirs=runitDirs,
                         testFileRegexp=runitPat,
                         rngKind="default",
                         rngNormalKind="default")
result <- runTestSuite(suite)
printTextProtocol(result, showDetails=FALSE)

### R code from vignette source 'ChromBand.Rnw'

###################################################
### code chunk number 1: setup1
###################################################
library("Category")
library("ALL")
library("hgu95av2.db")
library("annotate")
library("genefilter")
##library("SNPchip")
library("karyoploteR")
library("geneplotter")
library("limma")
library("lattice")
library("graph")


###################################################
### code chunk number 2: chr12ideogram
###################################################
build <- "hg19" ## or hg18
kp <- plotKaryotype(genome=build, chromosomes="chr12", main="Human chromosome 12")
## cyt2 <- getCytoband(build=build)
## cyt2$gieStain <- "foo"
## c12p12_idx <- intersect(grep("^q21", cyt2$name),
##                         which(cyt2$chrom == "12"))
## cyt2[c12p12_idx, "gieStain"] <- rep(c("gpos50", "gpos75"),
##                                     length=length(c12p12_idx))
##
##
## plotCytoband2(chromosome="12", build=build, cytoband=cyt2, outer=FALSE,  ## From SNPchip
##     cex.axis=0.6, main="Human chromosome 12")
## plotCytoband2(chromosome="12", build=build, cytoband=cyt2, outer=FALSE,
##               cex.axis=0.6,
##               main="Human chromosome 12")


###################################################
### code chunk number 3: bcrAblOrNegSubset
###################################################
data(ALL, package="ALL")

subsetType <- "BCR/ABL"
Bcell <- grep("^B", as.character(ALL$BT))
bcrAblOrNegIdx <- which(as.character(ALL$mol.biol) %in% c("NEG", subsetType))

bcrAblOrNeg <- ALL[, intersect(Bcell, bcrAblOrNegIdx)]
bcrAblOrNeg$mol.biol <- factor(bcrAblOrNeg$mol.biol)


###################################################
### code chunk number 4: annMaps
###################################################
annType <- c("db", "env")
entrezMap <- getAnnMap("ENTREZID", annotation(bcrAblOrNeg),
                       type=annType, load=TRUE)
symbolMap <- getAnnMap("SYMBOL", annotation(bcrAblOrNeg),
                       type=annType, load=TRUE)
bandMap <- getAnnMap("MAP", annotation(bcrAblOrNeg),
                     type=annType, load=TRUE)


###################################################
### code chunk number 5: nsFiltering
###################################################
filterAns <- nsFilter(bcrAblOrNeg,
                      require.entrez = TRUE, 
                      remove.dupEntrez = TRUE, 
                      var.func = IQR, var.cutoff = 0.5)
nsFiltered <- filterAns$eset


###################################################
### code chunk number 6: ChromBand.Rnw:208-213
###################################################
hasSYM <- sapply(mget(featureNames(nsFiltered), symbolMap, ifnotfound=NA),
                 function(x) length(x) > 0 && !is.na(x[1]))
hasMAP <- sapply(mget(featureNames(nsFiltered), bandMap, ifnotfound=NA),
                 function(x) length(x) > 0 && !is.na(x[1]))
nsFiltered <- nsFiltered[hasSYM & hasMAP, ]


###################################################
### code chunk number 7: defineGeneUniverse
###################################################
affyUniverse <- featureNames(nsFiltered)
entrezUniverse <- unlist(mget(affyUniverse, entrezMap))
names(affyUniverse) <- entrezUniverse
if (any(duplicated(entrezUniverse)))
    stop("error in gene universe: can't have duplicate Entrez Gene Ids")


###################################################
### code chunk number 8: parametric1
###################################################
design <- model.matrix(~ 0 + nsFiltered$mol.biol)
colnames(design) <- c("BCR/ABL", "NEG")
contr <- c(1, -1) ## NOTE: we thus have BCR/ABL w.r.t NEG
fm1 <- lmFit(nsFiltered, design)
fm2 <- contrasts.fit(fm1, contr)
fm3 <- eBayes(fm2)
ttestLimma <- topTable(fm3, number = nrow(fm3), adjust.method = "none")
ttestLimma <- ttestLimma[featureNames(nsFiltered), ]

tstats <- ttestLimma$t
names(tstats) <- entrezUniverse[rownames(ttestLimma)]
##


###################################################
### code chunk number 9: selectedSubset
###################################################
ttestCutoff <- 0.01
smPV  <- ttestLimma$P.Value < ttestCutoff
pvalFiltered <- nsFiltered[smPV, ]
selectedEntrezIds <- unlist(mget(featureNames(pvalFiltered), entrezMap))
##


###################################################
### code chunk number 10: ChromBand.Rnw:418-434
###################################################

chrSortOrder <- function(df) {
    chrs <- sub("([^pq]+).*$", "\\1", rownames(df))
    xyIdx <- chrs %in% c("X", "Y")
    xydf <- NULL
    if (any(xyIdx)) {
        chrs <- chrs[!xyIdx]
        xydf <- df[xyIdx, ]
        df <- df[!xyIdx, ]
    }
    ord <- order(as.integer(chrs), rownames(df))
    df <- df[ord, ]
    if (!is.null(xydf))
      df <- rbind(df, xydf)
    df
}


###################################################
### code chunk number 11: ChromBand.Rnw:439-454
###################################################

gseaTstatStripplot <- function(bands, g, ..., include.all = FALSE)
{
    chroms <- c(1:22, "X", "Y")
    chromArms <- c(paste(chroms, "p", sep=""), paste(chroms, "q", sep=""))
    egid <- lapply(nodeData(g, bands), "[[", "geneIds")
    if (include.all) {
        egid$All <- 
            unique(unlist(lapply(nodeData(g)[chromArms], "[[", "geneIds")))
    }
    tdf <- do.call(make.groups, lapply(egid, function(x) tstats[x]))
    stripplot(which ~ data, tdf, jitter = TRUE, ...)
}




###################################################
### code chunk number 12: ChromBand.Rnw:459-506
###################################################

esetBWPlot <- function(tmpSet, ..., layout=c(1, nrow(emat)))
{
    emat <- exprs(tmpSet)
    pd <- pData(tmpSet)
    probes <- rownames(emat)
    syms <- 
        sapply(mget(probes, hgu95av2SYMBOL, ifnotfound=NA),
               function(x) if (all(is.na(x))) "NA" else as.character(x)[1])
    selectedAffy <- 
        probes %in% affyUniverse[selectedEntrezIds]
    symsSelected <- syms[selectedAffy]
    symsWithStatus <- 
        paste(syms, 
              ifelse(selectedAffy, "*", ""), 
              sep = "")
    pdat <- 
        cbind(exprs=as.vector(emat),
              genes=factor(probes, levels = probes, labels = syms),
              pd[rep(seq_len(nrow(pd)), each=nrow(emat)), ])
    pdat <- transform(pdat, genes = reorder(genes, exprs))
    panels.to.shade <- levels(pdat$genes) %in% symsSelected
    bwplot(mol.biol ~ exprs | genes, data=pdat, 
           layout = layout,
           auto.key=TRUE,
           scales=list(x=list(log=2L)),
           xlab="Log2 Expression",
           panels.to.shade = panels.to.shade,
           panel = function(..., panels.to.shade) {
               if (panels.to.shade[packet.number()]) 
                   panel.fill(col = "lightgrey")
               panel.bwplot(...)
           },
           strip=FALSE,
           strip.left=TRUE, ...)
}

g1 <- makeChrBandGraph(annotation(nsFiltered), univ=entrezUniverse)
ct <- ChrBandTreeFromGraph(g1)

subsetByBand <- function(eset, ct, band) {
    egIDs <- unlist(nodeData(ct@toChildGraph, n=band, 
                             attr="geneIds"), use.names=FALSE)
    wantedProbes <- affyUniverse[as.character(egIDs)]
    eset[intersect(wantedProbes, featureNames(eset)), ]
}



###################################################
### code chunk number 13: basicParams
###################################################
params <- new("ChrMapHyperGParams",
              conditional=FALSE,
              testDirection="over",
              universeGeneIds=entrezUniverse,
              geneIds=selectedEntrezIds,
              annotation="hgu95av2",
              pvalueCutoff=0.05)

paramsCond <- params
paramsCond@conditional <- TRUE


###################################################
### code chunk number 14: basicTest
###################################################
hgans <- hyperGTest(params)
hgansCond <- hyperGTest(paramsCond)


###################################################
### code chunk number 15: result1
###################################################
sumUn <- summary(hgans, categorySize=1)
chrSortOrder(sumUn)

sumCond <- summary(hgansCond, categorySize=1)
chrSortOrder(sumCond)


###################################################
### code chunk number 16: ChromBand.Rnw:597-612
###################################################
gseaTstatStripplot(c("12q21.1", "12q21", "12q2", "12q"),
                   include.all = TRUE, 
                   g = g1,
                   xlab = "Per-gene t-statistics", 
                   panel = function(...) {
                       require(grid, quietly = TRUE)
                       grid.rect(y = unit(2, "native"), 
                                 height = unit(1, "native"),
                                 gp = 
                                 gpar(fill = "lightgrey", 
                                      col = "transparent"))
                       panel.grid(v = -1, h = 0)
                       panel.stripplot(...)
                       panel.average(..., fun = mean, lwd = 3)
                   })


###################################################
### code chunk number 17: gseaStripplot
###################################################
plot(trellis.last.object())


###################################################
### code chunk number 18: LMtestSetup
###################################################
params <- new("ChrMapLinearMParams",
              conditional = FALSE,
              testDirection = "up",
              universeGeneIds = entrezUniverse,
              geneStats = tstats,
              annotation = "hgu95av2",
              pvalueCutoff = 0.01, 
              minSize = 4L)
params@graph <- makeChrBandGraph(params@annotation, params@universeGeneIds)
params@gsc <- makeChrBandGSC(params@graph)
paramsCond <- params
paramsCond@conditional <- TRUE


###################################################
### code chunk number 19: LMtest
###################################################

lmans <- linearMTest(params)
lmansCond <- linearMTest(paramsCond)

chrSortOrder(summary(lmans))
chrSortOrder(summary(lmansCond))

##


###################################################
### code chunk number 20: ChromBand.Rnw:716-719
###################################################
tmpSet <- subsetByBand(nsFiltered, ct, "1p36.2")
esetBWPlot(tmpSet, ylab="1p36.2", layout = c(2, 8),
           par.strip.text = list(cex = 0.8))


###################################################
### code chunk number 21: dotplot1p362
###################################################
plot(trellis.last.object())

### Accessor methods for LinearMResultBase class

## generic "annotation" defined in Biobase

setMethod("annotation", signature(object="LinearMResultBase"),
          function(object) object@annotation)

setMethod("universeCounts", signature(r="LinearMResultBase"),
          function(r) {
              univ <- geneIdUniverse(r)
              ans <- listLen(univ)
              names(ans) <- names(univ)
              ans
          })

setMethod("universeMappedCount", signature(r="LinearMResultBase"),
           function(r) length(unique(unlist(geneIdUniverse(r)))))

setMethod("geneMappedCount", signature(r="LinearMResultBase"),
           function(r) length(geneIds(r)))

setMethod("geneIds", signature(object="LinearMResultBase"),
          function(object, ...) object@geneIds)

setMethod("geneIdsByCategory", signature(r="LinearMResultBase"),
          function(r, catids=NULL) {
              ans <- geneIdUniverse(r)
              if (!missing(catids) && !is.null(catids))
                ans <- ans[catids]
              lapply(ans, intersect, geneIds(r))
          })

setMethod("sigCategories", signature(r="LinearMResultBase"),
          function(r, p) {
              if (missing(p))
                  p <- pvalueCutoff(r)
              pv <- pvalues(r)
              names(pv[pv < p])
          })

setMethod("testName", signature(r="LinearMResultBase"),
          function(r) r@testName)

setMethod("pvalueCutoff", signature(r="LinearMResultBase"),
          function(r) r@pvalueCutoff)

setMethod("testDirection", signature(r="LinearMResultBase"),
          function(r) r@testDirection)

setMethod("description",
          signature(object="LinearMResultBase"),
          function(object) {
              cond <- "Conditional"
              if (!conditional(object))
                cond <- ""
              desc <- paste("Gene to %s ", cond, " test for %s-regulation", sep = "")
              desc <- sprintf(desc, paste(testName(object), collapse=" "),
                              testDirection(object))
              desc
          })

setMethod("conditional", "LinearMResultBase",
          function(r) {
              if (!("conditional" %in% slotNames(r)))
                  FALSE
              else
                  r@conditional
          })


setMethod("geneIdUniverse", signature(r="LinearMResultBase"),
          function(r, cond=TRUE) {
            im <- addHierarchyToIncidenceMatrix(incidence(r@gsc), r@graph)
            split(colnames(im)[col(im)[as.logical(im)]],
                  factor(rownames(im)[row(im)[as.logical(im)]], nodes(r@graph)))
          })

### Accessor methods for LinearMResult class

setMethod("pvalues", signature(r="LinearMResult"),
          function(r) r@pvalues)

setMethod("effectSize", signature(r="LinearMResult"),
          function(r) r@effectSize)

### Accessor methods for HyperGResultBase class

setMethod("annotation", signature(object="HyperGResultBase"),
          function(object) object@annotation)

setMethod("geneCounts", signature(r="HyperGResultBase"),
          function(r) {
              sapply(geneIdUniverse(r), function(x) {
                  sum(geneIds(r) %in% x)
              })
          })

setMethod("universeCounts", signature(r="HyperGResultBase"),
          function(r) {
              univ <- geneIdUniverse(r)
              ans <- listLen(univ)
              names(ans) <- names(univ)
              ans
          })

setMethod("universeMappedCount", signature(r="HyperGResultBase"),
           function(r) length(unique(unlist(geneIdUniverse(r)))))

setMethod("geneMappedCount", signature(r="HyperGResultBase"),
           function(r) length(geneIds(r)))

setMethod("geneIds", signature(object="HyperGResultBase"),
          function(object, ...) object@geneIds)

setMethod("geneIdsByCategory", signature(r="HyperGResultBase"),
          function(r, catids=NULL) {
              ans <- geneIdUniverse(r)
              if (!missing(catids) && !is.null(catids))
                ans <- ans[catids]
              lapply(ans, intersect, geneIds(r))
          })

setMethod("sigCategories", signature(r="HyperGResultBase"),
          function(r, p) {
              if (missing(p))
                p <- pvalueCutoff(r)
              pv <- pvalues(r)
              names(pv[pv < p])
          })

setMethod("testName", signature(r="HyperGResultBase"),
          function(r) r@testName)

setMethod("pvalueCutoff", signature(r="HyperGResultBase"),
          function(r) r@pvalueCutoff)

setMethod("testDirection", signature(r="HyperGResultBase"),
          function(r) r@testDirection)

setMethod("description",
          signature(object="HyperGResultBase"),
          function(object) {
              cond <- "Conditional"
              if (!conditional(object))
                cond <- ""
              desc <- paste("Gene to %s", cond, "test for %s-representation")
              desc <- sprintf(desc, paste(testName(object), collapse=" "),
                              testDirection(object))
              desc
          })

setMethod("conditional", "HyperGResultBase",
          function(r) {
              if (!("conditional" %in% slotNames(r)))
                FALSE
              else
                r@conditional
          })



### Accessor methods for HyperGResult class

setMethod("pvalues", signature(r="HyperGResult"),
          function(r) r@pvalues)

setMethod("oddsRatios", signature(r="HyperGResult"),
          function(r) r@oddsRatios)

setMethod("expectedCounts", signature(r="HyperGResult"),
          function(r) r@expectedCounts)

## generic "annotation" defined in Biobase

setMethod("geneIdUniverse", signature(r="HyperGResult"),
          function(r, cond=TRUE) r@catToGeneId)

getWantedResults <-
    function(result, pvalue, categorySize=NULL)
{
    ## Returns a logical vector with TRUE indicating selected
    ## results from those tested in the specified result instance.
    pvals <- pvalues(result)
    wanted <- is.finite(pvals) & pvals < pvalue
    if (!is.null(categorySize)) {
        ucounts <- universeCounts(result)
        hasMinSize <- ucounts >= categorySize
        wanted <- wanted & hasMinSize
    }
    wanted
}

setMethod("summary", signature(object="HyperGResultBase"),
          function(object, pvalue=pvalueCutoff(object), categorySize=NULL)
          {
              ## Filter based on p-value and category size
              wanted <- getWantedResults(object, pvalue, categorySize)
              pvals <- pvalues(object)
              ucounts <- universeCounts(object)
              if (!any(wanted)) {
                  warning("No results met the specified criteria.  ",
                          "Returning 0-row data.frame", call.=FALSE)
                  catIds <- character(0)
                  pvals <- odds <- ecounts <- numeric(0)
                  counts <- ucounts <- integer(0)
              } else {
                  pvals <- pvals[wanted]
                  ucounts <- ucounts[wanted]
                  catIds <- names(pvals)
                  odds <- oddsRatios(object)[wanted]
                  ecounts <- expectedCounts(object)[wanted]
                  counts <- geneCounts(object)[wanted]
              }
              df <- data.frame(ID=catIds, Pvalue=pvals, OddsRatio=odds,
                               ExpCount=ecounts, Count=counts, Size=ucounts,
                               stringsAsFactors=FALSE, row.names=NULL)
              names(df)[1] <- paste(paste(testName(object), collapse=""),
                                    "ID", sep="")
              df
          })

setMethod("summary", signature(object="KEGGHyperGResult"),
          function(object, pvalue=pvalueCutoff(object),
                   categorySize=NULL, htmlLinks=FALSE){
              KEGG_URL <- "http://www.genome.jp/dbget-bin/www_bget?path:%s%s"
              ## annOrg <- get(paste(annotation(object), "ORGANISM", sep=""))
              annOrg <- organism(object)
              orgSpecifier <- switch(annOrg,
                                     "Homo sapiens"="hsa",
                                     "Mus musculus"="mmu",
                                     "Rattus norvegicus"="rnu",
                                     ## will need others in future
                                     "hsa")
              df <- callNextMethod(object=object, pvalue=pvalue,
                                   categorySize=categorySize)
              if(nrow(df) == 0){
                  df$Term <- character(0)
                  return(df)
              }
              keggIds <- df[[1]]
              keggTerms <- unlist(getPathNames(keggIds, orgSpecifier))
              if(htmlLinks){
                  keggIdUrls <- sapply(keggIds,
                                       function(x)
                                       sprintf(KEGG_URL, orgSpecifier, x))
                  keggTerms <- paste('<a href="', keggIdUrls, '">', keggTerms,
                                     '</a>', sep="")
              }
              df$Term <- keggTerms
              df
          })

setMethod("summary", signature(object="PFAMHyperGResult"),
          function(object,pvalue=pvalueCutoff(object),
                   categorySize=NULL, htmlLinks=FALSE){
              PFAM_URL <- "http://pfam.sanger.ac.uk/family?acc=%s"
              df <- callNextMethod(object=object, pvalue=pvalue,
                                   categorySize=categorySize)
              if(nrow(df) == 0){
                  df$Term <- character(0)
                  return(df)
              }
              pfamIds <- df[[1]]
              if(htmlLinks){
                  pfamIdUrls <- sapply(pfamIds,
                                       function(x) sprintf(PFAM_URL, x))
                  pfamTerms <- paste('<a href="', pfamIdUrls, '">', pfamIds,
                                     '</a>', sep="")
              }else{
                  pfamTerms <- pfamIds
              }
              df$Term <- pfamTerms
              df
          })

setMethod("summary", signature(object="LinearMResult"),
          function(object, pvalue = pvalueCutoff(object),
                   categorySize = NULL)
          {
            wanted <- getWantedResults(object, pvalue, categorySize)
            if (!any(wanted)) {
              warning("No results met the specified criteria.  ",
                      "Returning 0-row data.frame", call.=FALSE)
            }
            df <- data.frame(ID = names(pvalues(object)),
                             Pvalue = pvalues(object),
                             Effect = effectSize(object),
                             Size = universeCounts(object),
                             Conditional = conditional(object),
                             TestDirection = testDirection(object),
                             stringsAsFactors = FALSE,
                             row.names = NULL)[wanted,]
            names(df)[1] <- paste(paste(testName(object), collapse=""),
                                  "ID", sep="")
            df
          })

htmlReportFromDf <- function(r, caption, file="", append=FALSE, digits=3)
{
    have_xtable <- suppressWarnings({
        requireNamespace("xtable", quietly=TRUE)
    })
    if (!have_xtable)
      stop("htmlReport needs the xtable package and it is not",
           "available", call.=FALSE)
    if (nrow(r) == 0) {
        warning("No rows to report.  Skipping")
        return(invisible(FALSE))
    }
    ## XXX: Hard-coded column formatting here
    dig <- rep(digits, ncol(r)+1)  ## need +1 for xtable row name
    dig[5:7] <- 0
    xt <- xtable::xtable(r, caption=caption, digits=dig)
    print(xt, type="html", file=file, append=append,
          caption.placement="top",
          sanitize.text.function=function(x) x,
          include.rownames=FALSE)
    return(invisible(TRUE))
}

XXX_getSummaryGeneric_XXX <- function() {
    ## FIXME: the methods packge is broken for this case
    ## so we have to find the right summary method ourselves
    places <- find("summary")
    ## take the first standardGeneric
    f <- NULL
    for (i in seq(along=places)) {
        f <- get("summary", places[i])
        if (is(f, "standardGeneric"))
          break
        else
          f <- NULL
    }
    if (is.null(f))
      stop("could not find appropriate summary method")
    f
}

setMethod("htmlReport", signature(r="HyperGResultBase"),
          function(r, file="", append=FALSE, label="", digits=3,
                   summary.args=NULL)
          {
              summary <- XXX_getSummaryGeneric_XXX()
              if (!is.null(summary.args) && !is.list(summary.args))
                stop("'summary.args' must be NULL or a list of arguments for",
                     " the summary method")
              df <- do.call(summary, c(list(r), summary.args))
              htmlReportFromDf(r=df,
                               caption=paste(label, description(r)),
                               file=file, append=append, digits=digits)
          })

setMethod("htmlReport", signature(r="KEGGHyperGResult"),
          function(r, file="", append=FALSE, label="",
                   digits=3, summary.args=list(htmlLinks=TRUE)){
              callNextMethod(r=r, file=file, append=append,
                             label=label, digits=digits,
                             summary.args=summary.args)
          })

setMethod("htmlReport", signature(r="PFAMHyperGResult"),
          function(r, file="", append=FALSE, label="",
                   digits=3, summary.args=list(htmlLinks=TRUE)){
              callNextMethod(r=r, file=file, append=append,
                             label=label, digits=digits,
                             summary.args=summary.args)
          })

checkChrOrg <- function(org) {
    supportedChrOrgs <- c("ORGANISM:Homo sapiens", "ORGANISM:Mus musculus")
    if (!(org %in% supportedChrOrgs)) {
        msg <- paste(sprintf("'%s' is not a supported.\n", org),
                     "organism must be one of:",
                     paste(supportedChrOrgs, collapse=", "))
        stop(msg, call.=FALSE)
    }
}

probes2MAP <- function (pids, data = "hgu133plus2") {
    pEnv = getAnnMap("MAP", chip=data)
    inMAP = mget(pids, pEnv, ifnotfound = NA)
    inMAP[!is.na(inMAP)]
}
### ------------------------------------------------------------------
cleanMapAndsOrs <- function(e2m, org) {
    checkChrOrg(org)
    func <- switch(org,
       "ORGANISM:Homo sapiens"= cleanMapAndsOrs_Hs,
       "ORGANISM:Mus musculus"= function(x) x)
    func(e2m)
}
cleanMapAndsOrs_Hs <- function(e2m) {
    ## The map data is a bit odd and we get things like: "Xq28 and Yq12"
    ## and "Xp22.3 or Yp11.3".  This function maps these to a proper
    ## vector of length 2.
    for (eg in ls(e2m)) {
        e2m[[eg]] <- unlist(strsplit(e2m[[eg, exact=TRUE]], " and | or "))
    }
    e2m
}


cleanMapWeird <- function(e2m, org) {
    checkChrOrg(org)
    func <- switch(org,
       "ORGANISM:Homo sapiens"= cleanMapWeird_Hs,
       "ORGANISM:Mus musculus"= function(x) x)
    func(e2m)
}
cleanMapWeird_Hs <- function(e2m) {
    ## Current MAP annotation data contains weird entries like:
    ##     "1 69.9 cM"
    ##     "1 E4"
    ##     "11 29.0 cM"
    ##     "11 B1.3"
    ##     "19 23.0 cM"
    ##     "19 C1"
    ##     "1q23.1 according to Sierra (Genomics 79, 177, 2002) [AFS]"
    ##     "2 A1"
    ##
    ## This function removes these for now.  The strategy is simply to
    ## look for annotations with spaces and drop them.
    for (eg in ls(e2m)) {
        val <- e2m[[eg, exact=TRUE]]
        badIdx <- grep(" ", val)
        if (length(badIdx) > 0) {
            good <- val[-badIdx]
            if (length(good) == 0)
              good <- as.character(NA)
        } else {
            good <- val
        }
        e2m[[eg]] <- good
    }
    e2m
}

cleanRanges <- function(e2m, org) {
    checkChrOrg(org)
    func <- switch(org,
       "ORGANISM:Homo sapiens"= cleanRanges_Hs,
       "ORGANISM:Mus musculus"= cleanRanges_Mm)
    func(e2m)
}

cleanRanges_Hs<- function(e2m) {
    ## We use (p|q) even though it will match nonsense annotations like
    ## "6p23-q24".  These seem to appear in the data.  But it is OK
    ## since we are taking the longest common suffix and that will just
    ## be the chromosome in these cases.
    rangePat <- "[0-9XY]+(q|p)([0-9.]+|ter)-(q|p)([0-9.]+|ter)"
        cenPat <- "cen"
        for (eg in ls(e2m)) {
            e2m[[eg]] <- sapply(e2m[[eg, exact=TRUE]], function(z) {
                if (length(grep("-", z, fixed=TRUE))) {
                    if (length(grep(rangePat, z, perl=TRUE))) {
                        arm.loc <- gregexpr("(q|p|-)", z)[[1L]]
                        lhs <- substr(z, 1L, arm.loc[2L]-1L)
                        rhs <- paste(substr(z, 1L, arm.loc[1L]-1L),
                                     substr(z, arm.loc[2L]+1L, nchar(z)), sep="")
                        ans <- lcPrefixC(c(lhs, rhs))
                        nc <- nchar(ans)
                        if (substr(ans, nc, nc) == ".")
                          ans <- substr(ans, 1L, nc - 1L)
                        ans
                    } else if (length(grep(cenPat, z, perl=TRUE))) {
                        ## FIXME:
                        ## for range including centromere, we just
                        ## take the entire chromosome.
                        ## Could we do better and include arm info?
                        substr(z, 1L, regexpr("cen|q|p", z) - 1L)
                    } else {
                        parts <- strsplit(z, "-", fixed=TRUE)[[1]]
                        ans <- lcPrefix(parts)
                        if (nchar(ans) < 1)
                          ans <- substr(z, 1L, regexpr("cen|q|p", z) - 1L)
                        warning("unexpected band notation: ", z, " using ", ans,
                                call.=FALSE)
                        ans
                    }
                } else {
                    z
                }
            })
        }
        e2m
}

cleanRanges_Mm<- function(e2m) {
    ## We use (p|q) even though it will match nonsense annotations like
    ## "6p23-q24".  These seem to appear in the data.  But it is OK
    ## since we are taking the longest common suffix and that will just
    ## be the chromosome in these cases.
    cmPat <- "[[:space:]].*cM$"
    cenPat <- "[[:space:]]centromere$"
    rangePat <- "[0-9XY]+[[:space:]][A-Z]+(-|/)[A-Z]+"
    for (eg in ls(e2m)) {
        e2m[[eg]] <- sapply(e2m[[eg, exact=TRUE]], function(z) {
            if (length(grep(rangePat, z, fixed=TRUE))) {
                arm.loc <- gregexpr("(q|p|-)", z)[[1L]]
            }else  if (length(grep(cenPat, z, perl=TRUE))){
                substr(z, 1L, regexpr(cenPat, z) - 1L)
            } else if (length(grep(cmPat, z, perl=TRUE))){
                substr(z, 1L, regexpr(cmPat, z) - 1L)
            }
            else {
                z
            }
        })
    }
    e2m
}

annBaseName <- function(p) {
    baseName <- p@datPkg@baseName
}

makeChrMapToEntrez <- function(datPkg, univ) {
    
    .getMap <- function(map)
      getAnnMap(map=map, chip=datPkg@name)
    probe2chr <- .getMap(map="MAP")
    org <- getOrganism(datPkg@name)

    if (!is.environment(probe2chr))
      probe2chr <- list2env(as.list(probe2chr))

    egs <- unique(unlist(mget(ls(probe2chr), ID2EntrezID(datPkg))))
    egs <- egs[!is.na(egs)]
    if (!is.null(univ))
      egs <- intersect(egs, univ)
    egs <- as.character(egs)
    eg2chr <- new.env(parent=emptyenv(), hash=TRUE,
                      size=as.integer(1.20 * length(egs)))
    ## XXX: need to define a revmap method for environments
    eg2p <- list2env(mget(egs, revmap(ID2EntrezID(datPkg))))
    for (eg in egs) {
        bands <- mget(eg2p[[eg, exact=TRUE]], probe2chr)
        bands <- bands[!is.na(bands)]
        if (length(bands))
          eg2chr[[eg]] <- unique(unlist(bands))
    }

    eg2chr <- cleanMapAndsOrs(eg2chr, org)
    eg2chr <- cleanMapWeird(eg2chr, org)
    eg2chr <- cleanRanges(eg2chr, org)

    m2eg <- reverseSplit(as.list(eg2chr))

    ## XXX: remove all annotations that remain that have '- or /'
    hasRange <- grep("-|/", names(m2eg))
    if (length(hasRange)) {
        warning("dropping ", length(hasRange), " items with weird range")
        m2eg <- m2eg[-hasRange]
    }
    onlyDot <- grep("^\\.$", names(m2eg))
    if (length(onlyDot))
      m2eg <- m2eg[-onlyDot]

    ## leave that way for the moment but could be cleaner
    if(org == "ORGANISM:Homo sapiens"){
        invalid <- grep("[^0-9.qpXYter]+", names(m2eg))
        if (length(invalid)) {
            warning("dropping invalid items: ",
                    paste(names(m2eg)[invalid], collapse=", "))
            m2eg <- m2eg[-invalid]
        }
    }
    m2eg
}


cb_parse_band_Hs <- function(x) {
    ## Given a chromosome band annotation (see examples below),
    ## return a vector giving the path from the root:
    ##
    ## 20p12.2 => 20, 20p, 20p1, 20p12, 20p12.2
    ##
    ## x1 <- "2q32"
    ## x2 <- "4q21.22"
    ## x3 <- "20p12.2"
    ## x4 <- "2qter"
    ##
    arm.pos <- regexpr("p|q", x)
    if (arm.pos < 0)
      return(x)
    chr <- substr(x, 1, arm.pos - 1)
    chrArm <- substr(x, 1, arm.pos)
    if (substr(x, arm.pos + 1, arm.pos + 3) == "ter")
      return(c(chr, chrArm, x))
    if (nchar(x) == nchar(chrArm))
      return(c(chr, chrArm))
    sbs <- strsplit(substr(x, arm.pos+1, nchar(x)), "")[[1]]
    bands <- character(length(sbs)+1)

    bands[1] <- chrArm
    i <- 1
    j <- 2
    prev <- bands[1]
    while (TRUE) {
        if (sbs[i] == ".") {
            prev <- paste(prev, ".", sep="")
            i <- i + 1
            next
        }
        bands[j] <- paste(prev, sbs[i], sep="")
        prev <- bands[j]
        i <- i + 1
        j <- j + 1
        if (i > length(sbs))
          break
    }
    if (i == j)                         # there was a '.'
      c(chr, bands[1:(i-1)])
    else
      c(chr, bands)
}


makeChrBandGraph <- function(chip, univ=NULL) {
    if(class(chip) == "character"){
        ##chip must be a datPkg (of correct type)
        chip = DatPkgFactory(chip)
    }
    
    org <- getOrganism(chip@name)
    checkChrOrg(org)

    parser <- switch(org,
       "ORGANISM:Homo sapiens"= cb_parse_band_Hs,
       "ORGANISM:Mus musculus"= cb_parse_band_Mm)

    m2p <- makeChrMapToEntrez(chip, univ)
    allMaps <- lapply(names(m2p), parser)
    vv <- lapply(allMaps, function(x) {
        L <- length(x)
        ## XXX: some will have L == 1, will induce NA's
        rbind(x[1:(L - 1)], x[2:L])
    })
    vvAll <- do.call(cbind, vv)
    vvStr <- paste(vvAll[1, ], vvAll[2, ], sep="+")
    ## remove duplicate edges
    dupIdx <- which(duplicated(vvStr))
    if (length(dupIdx) > 0 && any(dupIdx > 0))
      vvUniq <- vvAll[, -dupIdx]
    else
      vvUniq <- vvAll
    dimnames(vvUniq) <- list(c("from", "to"), NULL)
    vvT <- t(vvUniq)
    allNodes <- unique(unlist(allMaps))
    ## remove NA's
    NAitems <- which(is.na(vvT[, 2]))
    if (length(NAitems) > 0 && any(NAitems > 0))
      vvT <- vvT[-NAitems, ]
    ## remove self-loops
    selfLoops <- which(vvT[, 1] == vvT[, 2])
    if (length(selfLoops) > 0 && any(selfLoops > 0))
      vvT <- vvT[-selfLoops, ]
    ## add root node
    chrNames <- names(getAnnMap("CHRLENGTHS", chip@name))
    orgLinks <- cbind(rep(org, length(chrNames)), chrNames)
    vvT <- rbind(orgLinks, vvT)

    g <- ftM2graphNEL(vvT, edgemode="directed")
    g <- addChrBandAnnotation(g, m2p)
    g
}

addChrBandAnnotation <- function(g, m2p) {
    nodeDataDefaults(g, "geneIds") <- as.character(NA)
    bands <- rev(tsort(g)[-1])          # remove root node
    gEdges <- edges(g)
    for (n in bands) {
        ids <- m2p[[n, exact=TRUE]]
        kids <- gEdges[[n, exact=TRUE]]
        if (length(kids)) {
            ids <- unique(c(ids, unlist(nodeData(g, n=kids, "geneIds"))))
        }
        if (length(ids) > 0)
          nodeData(g, n, "geneIds") <- list(ids[!is.na(ids)])
    }
    g
}

makeChrBandInciMat <- function(chrGraph) {
    gnodes <- nodes(chrGraph)
    rootIdx <- grep("^ORGANISM", gnodes)
    v1 <- nodeData(chrGraph, n=gnodes[-rootIdx] , attr="geneIds")
    v1 <- v1[sapply(v1, function(z) !any(is.na(z)))]
    v1 <- lapply(v1, as.character)
    allGeneIDs <- unique(unlist(v1))
    nr <- length(v1)
    nc <- length(allGeneIDs)
    mat <- matrix(0L, nrow=nr, ncol=nc)
    dimnames(mat) <- list(names(v1), allGeneIDs)
    for (cb in names(v1)) {
        mat[cb, v1[[cb, exact=TRUE]]] <- 1L
    }
    mat
}

makeChrBandGSC <- function(chrGraph) {
  gnodes <- nodes(chrGraph)
  rootIdx <- grep("^ORGANISM", gnodes)
  v1 <- nodeData(chrGraph, n=gnodes[-rootIdx] , attr="geneIds")
  v1 <- v1[sapply(v1, function(z) !any(is.na(z)))]
  v1 <- lapply(v1, as.character)
  GeneSetCollection(mapply(GeneSet, v1, setIdentifier = names(v1),
                           setName = names(v1),
                           MoreArgs = list(geneIdType = EntrezIdentifier())))
}

MAPAmat <- function(chip, univ=NULL, minCount=0) {
    m <- makeChrBandInciMat(makeChrBandGraph(chip=chip, univ=univ))
    if (minCount > 0) {
        rs <- rowSums(m)
        m <- m[rs >= minCount, ]
    }
    m
}

ann_list_to_mat <- function(L) {
    gsets <- names(L)
    nr <- length(gsets)
    allgenes <- unique(unlist(L))
    nc <- length(allgenes)
    M <- matrix(0L, nrow=nr, ncol=nc,
                dimnames=list(gsets, allgenes))
    for (gs in gsets) {
        M[gs, L[[gs]]] <- 1L
    }
    M
}

cb_children <- function(n, chrGraph) {
    if (length(n) != 1)
      stop("arg 'n' must be a length one character vector")
    if (!(n %in% nodes(chrGraph)))
      return(character(0))
    edges(chrGraph)[[n, exact=TRUE]]
}

cb_childAnnList <- function(n, g) {
    ## n - node label, e.g., 12p1
    ## g - graph object representing the tree of chrom bands
    ##
    ## Returns a list named by children of n, values
    ## are vectors of gene IDs.  If n is not a node in
    ## g or has no children, return list()
    kids <- cb_children(n, g)
    if (length(kids))
      nodeData(g, n=kids, attr="geneIds")
    else
      list()
}

cb_buildInciMat <- function(n, g) {
    ## Return an incidence matrix for the gene sets
    ## (rows) specified by n in graph
    ## g.  The columns are the gene IDs
    dat <- nodeData(g, n=n, attr="geneIds")
    ann_list_to_mat(dat)
}

expCounts <- function(tab, wh.row=1L) {
    ## Expected counts for cells in wh.row of a
    ## contingency table
    ##
    ## Rule of thumb: E <= 5 implies ChiSq not reliable
    N <- sum(tab)
    rs <- sum(tab[wh.row, ])
    cs <- colSums(tab)
    (cs * rs) / N
}

## chiSqrSummands <- function(tab, wh.row=1L) {
##     ## this is just abs(residual)
##     E <- expCounts(tab, wh.row=wh.row)
##     sqrt((tab[wh.row, ] - E)^2 / E)
## }

cb_buildContinTable <- function(imat, selids, min.expected=1L, min.k=1L) {
    ## Return a 2 x k contingency table.  First row is selected,
    ## second is not.  Columns are gene sets (rows of imat).  Columns
    ## with expected count for selected row < min.expected are
    ## omitted.  Tables with fewer than min.k columns result in a
    ## return of integer(0).
    ##
    if (any(dim(imat) == 0))
      return(integer(0))
    if (nrow(imat) < min.k)
      return(integer(0))
    wh <- match(selids, colnames(imat), 0)
    submat <- imat[ , wh, drop=FALSE]
    if (ncol(submat)  == 0)
      return(integer(0))
    tot <- rowSums(imat)
    found <- rowSums(submat)
    tab <- t(cbind(found, tot - found))
    dimnames(tab) <- list(c("selected", "not"), rownames(imat))
    eCnt <- expCounts(tab)
    keep.cols <-
      if (!is.null(min.expected))
        eCnt > min.expected
      else
        as.logical(eCnt)
    tab <- tab[ , keep.cols, drop=FALSE]
    if (ncol(tab) > 0)
      tab
    else
      integer(0)
}

cb_contingency <- function(selids, chrVect, chrGraph, testFun=chisq.test,
                           min.expected=5L, min.k=1L) {
    ## filter unknown nodes
    chrVect <- chrVect[chrVect %in% nodes(chrGraph)]
    chrMat <- cb_buildInciMat(chrVect, chrGraph)
    conTab <- cb_buildContinTable(chrMat, selids,
                                  min.expected=min.expected, min.k=min.k)
    if (is.matrix(conTab) && nrow(conTab) > 1 && ncol(conTab) > 1)
        list(table=conTab, result=testFun(conTab))
    else
        list()
}


cb_sigBands <- function(b, p.value=0.01) {
    bands <- lapply(b, function(x) {
        if (x$result$p.value < p.value)
          colnames(x$table)
        else
          as.character(NA)
    })
    ans <- unlist(bands[!is.na(bands)])
    if (!length(ans))
      character(0)
    else
      ans
}

## ----

## This is another non-incidence matrix approach to
## generating the contingency tables.  Probably can
## junk this.
makeContinTable <- function(selids, ann_dat) {
    ## selids - vector of Gene IDs
    ## ann_dat - a list of vectors of Gene IDs; each
    ##           represents the IDs in a cateogry.
    ##           We assume the categories are close
    ##           to a partition of the genes.
    ##
    ## Returns a matrix with rows 'selected' and 'not' and a column for
    ## each element of ann_dat giving the counts for the contingency
    ## table.
    do.call(cbind, lapply(ann_dat, function(x) {
        found <- sum(selids %in% x)
        c(selected=found, not=(length(x) - found))
    }))
}
cb_makeContinTable <- function(n, g, selids) {
    dat <- cb_childAnnList(n, g)
    if (length(dat))
      makeContinTable(selids, dat)
    else
      dat
}

## ----

setMethod("linearMTest",
          signature(p = "LinearMParams"),
          function(p) .linearMTestInternal(p))

.linearMTestInternal <- function(p, className="LinearMResult")
{
    if (length(categoryName(p)))
        className <- paste(categoryName(p), className, sep = "")
    ## p <- makeValidParams(p) # FIXME: not written yet

    ans <-
        .doLinearMTest(stats = p@geneStats,
                       g = p@graph,
                       gsc = p@gsc,
                       upreg = p@testDirection == "up",
                       cutoff = pvalueCutoff(p),
                       global = TRUE,
                       min.genes = p@minSize,
                       conditional = conditional(p))
    
    ## 'NA' introduced when minimum size violated
    g <- p@graph
    pvals <- if (p@conditional) ans$conditional.pvals else ans$marginal.pvals
    ans_pvals <- setNames(rep(NA, numNodes(g)), nodes(g))
    ans_pvals[names(pvals)] <- pvals

    effects <- if (p@conditional) ans$conditional.effects
               else ans$marginal.effects
    ans_effects <- setNames(rep(NA, numNodes(g)), nodes(g))
    ans_effects[names(effects)] <- effects
    
    new(className,
        pvalues = ans_pvals,
        effectSize = ans_effects,
        annotation = p@annotation,
        geneIds = p@universeGeneIds,
        testName = p@categoryName,
        pvalueCutoff = p@pvalueCutoff,
        testDirection = p@testDirection,
        minSize = p@minSize,
        pvalue.order = order(ans_pvals),
        conditional = p@conditional,
        graph = p@graph,
        gsc = p@gsc)
}



## Here's the background for determining conditional significance of
## chromosome bands: Assume that we know how to determine marginal
## significance (this is done by fitting the model t ~ 1 + x, where t
## is the vector of t-stats and x is the indicator of category
## membership, where rows are genes or probesets).  Let's say we have
## a graph that represents the chromosome band structure, and an
## incidence matrix giving membership of genes in chromosome bands.
## Start with graph induced by marginally (as opposed to
## conditionally) significant categories.  Alternatively, start with
## full graph.  Declare significant leaves (there cannot be
## non-significant leaves in the first case, I think) to be
## conditionally significant.  For every other marginally significant
## node, assume conditional significance has been determined for all
## children (if not, do those first).  Fit a linear model with t-stats
## as response, and indicators for all CONDITIONALLY (not marginally)
## significant children as predictors, as well as an indicator for the
## current category.  If the current category is significant in the
## result, declare current node as conditionally significant.

## This approach is also valid for non-nested genesets such as GO, but
## then we will need to worry about overlap between rest (which we
## could try to do pairwise).  This is not an issue with chromosome
## bands.

## ok, so how to do this?

## First, we need a method for finding all children of a node in a
## graph.  We'll use a quick and dirty solution that repeatedly
## follows edges until there are no more to find

.getChildrenInGraph <- function(g, init)
    ## g: graph
    ## init: vector (typically length-1?) of node names
{
    gm <- as(g, "matrix")
    final <- logical(nrow(gm))
    names(final) <- rownames(gm)
    final[init] <- TRUE
    done <- FALSE
    while (!done) {
        ncur <- sum(final)
        direct.children <-
            apply(gm[final, , drop = FALSE], 1,
                  function(x) which(x != 0))
        if (is.list(direct.children))
            direct.children <- unlist(direct.children)
        final[direct.children] <- TRUE
        nnew <- sum(final)
        done <- nnew == ncur
    }
    names(final)[final]
}



## With this tool, we're now ready to do the sequence of tests.
## Basically, we can test a node if it has no children.  If it does,
## then the children must have been tested first, and those that are
## significant would be included in the model.

## This function does the (trivial) marginal test for one category

linearMTestMarginal <- function(stats, x, upreg = TRUE)
{
    lm.res <- lm(stats ~ 1 + x)
    tt <- summary(lm.res)$coef[2, 3]
    if (upreg) 1 - pnorm(tt) else pnorm(tt)
}

linearMEffectSizeMarginal <- function(stats, x)
{
    tail(coef(lm(stats ~ 1 + x)), 1)
}


## This function does the conditional test (for the whole graph, not
## just one category).  As the marginal/unconditional tests are done
## anyway, we allow skipping the conditional testing through an
## argument.

.doLinearMTest <-
    function(stats, g, gsc, upreg = TRUE,
             cutoff = 0.01,
             global = TRUE,
             min.genes = 4,
             conditional = TRUE,
             verbose = getOption("verbose"))
    ## upreg: whether looking for H_1: mu > 0 or mu < 0
    ## global: global means all genes in 'stats' considered universe.
    ##    local means only ones which belong in some node in g.  
    ##    FIXME: not sure if this is the correct interpretation.
    ## min.genes: keep only nodes with these many genes
{
    ## print(system.time(
    ## inc.mat <- t(MAPAmat(chip, univ = names(stats)))
    inc.mat <- t(addHierarchyToIncidenceMatrix(incidence(gsc), g))
    ## ))
    common.genes <- intersect(names(stats), rownames(inc.mat))
    inc.mat <- inc.mat[common.genes,]
    ## leave out nodes that contain every gene
    ## leave out nodes that have no genes (or rather, too few genes)
    cb.names <- colnames(inc.mat)
    cb.names <- cb.names[ (colSums(inc.mat) >= min.genes) &
                          (colSums(inc.mat) < nrow(inc.mat)) &
                          (cb.names %in% nodes(g)) ]
    ## cb.names <- cb.names[grep("^[1-9XY]", cb.names)]
    g <- subGraph(cb.names, g)
    inc.mat <- inc.mat[, cb.names]
    gene.names <- rownames(inc.mat)
    stats <- stats[gene.names]
    
    if (!global)
    {
        keep <- rowSums(inc.mat) > 0
        stats <- stats[keep]
        inc.mat <- inc.mat[keep, ]
    }

    ## set up vectors to track quantities of interest
    marginal.pvals <-
        as.numeric(sapply(cb.names,
                          function(n) {
                            linearMTestMarginal(stats, inc.mat[, n],
                                                upreg = upreg)
                          }))
    marginal.effects <-
        as.numeric(sapply(cb.names,
                          function(n) {
                            linearMEffectSizeMarginal(stats, inc.mat[, n])
                          }))
    names(marginal.pvals) <- names(marginal.effects) <- cb.names
    marginal <- marginal.pvals < cutoff
    if (!conditional)
        return(list(marginal = marginal,
                    marginal.pvals = marginal.pvals,
                    marginal.effects = marginal.effects))

    ## otherwise, proceed with conditional testing
    excluded <- conditional <- logical(length(cb.names))
    conditional.effects <- marginal.effects
    names(excluded) <- names(conditional) <- cb.names

    ## containter for adjusted (conditional) p-values
    adjusted.pvals <- marginal.pvals

    ## first iteration, figure out which ones have no children, i.e.,
    ## the ones that are leaves
    gl <- leaves(g, "out")
    conditional[gl] <- adjusted.pvals[gl] < cutoff
    excluded[gl] <- TRUE

    while (!all(excluded))
    {
        subg <- subGraph(cb.names[!excluded], g)
        subl <- leaves(subg, "out")
        if (verbose) cat(subl, "\n")

        ## loop over leaves and decide if they are conditionally
        ## significant

        for (i in subl)
        {
            ## FIXME: does it make sense to check conditional
            ## significance when not marginally significant?
            if (!is.na(marginal[i]) && marginal[i]) ## else nothing to do
            {
                if (verbose) cat("Processing node: ", i, "\n")
                ## need to find children that are conditionally
                ## significant

                ## how to find children?  Use function .getChildrenInGraph.
                ## Work on a smaller graph: current + excluded.  This
                ## is not necessary, but is done in the hope that this
                ## is more efficient.  FIXME: Don't know that for a
                ## fact, should profile.
                smallg <- subGraph(c(i, cb.names[excluded]), g)
                ## get all children of i.  This always includes i
                all.children <- .getChildrenInGraph(smallg, i)
                ## keep only those that are signif. This step
                ## automtically excludes i since conditional[i] is
                ## guaranteed to be FALSE at this point
                sig.children <- all.children[conditional[all.children]]

                ## Great. Now to fit a linear model.  We already have
                ## the response.  The design matrix should have (1) an
                ## intercept (unless we think the response should be
                ## centered) (2) one column for each sig.children and
                ## (3) one column for the current node

                ## this is a bit inefficient, but easiest on the mind
                ## (the more efficient alternative would be to use
                ## lm.fit)
                design.mat <-
                    cbind(y = stats,
                          inc.mat[, c(sig.children, i), drop = FALSE])
                design.mat <- as.data.frame(design.mat)
                lm.res <- lm(formula(design.mat), design.mat)
                cond.t <- (summary(lm.res)$coef[, 3])[ncol(design.mat)]
                cond.pval <- if (upreg) 1 - pnorm(cond.t) else pnorm(cond.t)

                ## good, now set these results
                conditional.effects[i] <- tail(coef(lm.res), 1)
                adjusted.pvals[i] <- cond.pval
                conditional[i] <- !is.na(cond.pval) && cond.pval < cutoff
            }
        }
        ## done with all the leaves, so exclude them (and move on to
        ## next layer)
        excluded[subl] <- TRUE
    }
    list(marginal = marginal,
         conditional = conditional,
         marginal.pvals = marginal.pvals,
         conditional.pvals = adjusted.pvals,
         marginal.effects = marginal.effects,
         conditional.effects = conditional.effects)
}

addHierarchyToIncidenceMatrix <- function(x, g) {
  nodes <- rev(tsort(g))
  new_nodes <- setdiff(nodes, rownames(x))
  new_x <- matrix(0L, nrow = length(new_nodes), ncol = ncol(x),
                  dimnames = list(new_nodes, colnames(x)))
  x <- rbind(x, new_x)
  edges_g <- edges(g)
  for (node in setdiff(nodes, leaves(g, "out"))) {
    children <- edges_g[[node, exact=TRUE]]
    x[node,] <- x[node,] | (colSums(x[children,,drop=FALSE]) > 0)
  }
  x
}

setMethod("chrGraph", signature(r="ChrMapHyperGResult"),
          function(r) r@chrGraph)


setMethod("pvalues", signature(r="ChrMapHyperGResult"),
          function(r) {
              unlist(nodeData(r@chrGraph, attr="pvalue"))[r@pvalue.order]
              })


setMethod("oddsRatios", signature(r="ChrMapHyperGResult"),
          function(r) {
              unlist(nodeData(r@chrGraph, attr="oddsRatio"))[r@pvalue.order]
              })


setMethod("expectedCounts", signature(r="ChrMapHyperGResult"),
          function(r) {
              unlist(nodeData(r@chrGraph, attr="expCount"))[r@pvalue.order]
              })


entrezGeneUniverse <- function(r) {
    nodeData(r@chrGraph, n=nodes(r@chrGraph)[r@pvalue.order],
             attr="geneIds")
}


setMethod("geneIdUniverse", signature(r="ChrMapHyperGResult"),
          function(r, cond=TRUE) {
              if (cond && conditional(r))
                nodeData(r@chrGraph, n=nodes(r@chrGraph)[r@pvalue.order],
                         attr="condGeneIds")
              else
                entrezGeneUniverse(r)
          })

##take an adjacency matrix, where the genes are columns, and a test
##stat vector and find the interesting sums
findAMstats = function(Amat, tstats) {
    pwLens = rowSums(Amat)
    expDE = Amat %*% tstats
    names(expDE) <- rownames(Amat)
    pw1s = pwLens > 1
    return(list(eDE = expDE[pw1s], lens=pwLens[pw1s]))
}

## Amat: adjacency matrix of gene -> categories
applyByCategory = function(stats, Amat, FUN=mean, ...)
{
  if(!(is.matrix(Amat)))
    stop("'Amat' needs to be a matrix.")
  if(ncol(Amat) != length(stats) )
    stop("'ncol(Amat)' needs to be the same as 'length(stats)'.")
  if(!is.logical(Amat)){
    if(!is.numeric(Amat))  
      stop("'Amat' needs to be logical or numeric.")
    Amat = (Amat!=0)
  }
  
  res = apply(Amat, 1, function(x) FUN(stats[x], ...))
  names(res) = rownames(Amat)
  return(res)
}

##given a set of AffyIDs, which have pathway data
probes2Path = function(pids, data="hgu133plus2") {
    pEnv = getAnnMap("PATH", data)
    inPW = mget(pids, pEnv, ifnotfound=NA)
    inPW[!is.na(inPW)]
}

getPathNames = function(iPW, organism = "hsa") {
    klist <- KEGGREST::keggList("pathway", organism = organism)
    knames <- paste0("path:", organism, iPW)
    kind <- lapply(stats::setNames(nm = knames), match, names(klist))
    kres <- vapply(kind, as.character, character(1L))
    kres[!is.na(kind)] <- klist[unlist(kind[!is.na(kind)])]
    lapply(strsplit(kres, " - "), `[`, 1L)
}

ttperm = function(x, fac, B=100, tsO=TRUE) {
    obs = rowttests(x, fac, tstatOnly= tsO)
    ans = vector(mode="list", length=B)
    for(i in 1:B) {
        p1 = sample(fac)
        ans[[i]] = rowttests(x, p1, tstatOnly = tsO)
    }
    return(list(obs=obs, perms=ans))
}

makeEBcontr = function(f1, hival) {
    if (!requireNamespace("EBarrays", quietly=TRUE))
      stop("need the EBarrays package for this feature")
    contr = rep(1, length(f1))
    p1 = paste(contr, collapse=",")
    contr2 = contr
    contr2[f1 == hival] = 2
    p2 = paste(contr2, collapse=",")
    EBarrays::ebPatterns(c(p1,p2))
}

qRequire <- function(pkg) {
    suppressWarnings(require(pkg, character.only=TRUE,
                             quietly=TRUE, warn.conflicts=FALSE))
}

pvalFromPermMat <- function(obs, perms) {
    N <- ncol(perms)
    pvals <- matrix(as.double(NA), nrow=nrow(perms), ncol=2)
    dimnames(pvals) <- list(rownames(perms), c("Lower", "Upper"))

    tempObs <- rep(obs, ncol(perms))
    dim(tempObs) <- dim(perms)
    pvals[ , 1] <- rowSums(perms <= tempObs)/N
    pvals[ , 2] <- rowSums(perms >= tempObs)/N
    pvals
}

gseattperm <- function(eset, fac, mat, nperm) {
    mkSparseMat <-
        function(x) Matrix::Matrix(x, sparse=TRUE)

    geneNames <- colnames(mat)
    if (is.null(geneNames))
      stop("'mat' argument must have column names")
    eset <- eset[colnames(mat), ]
    if (nrow(eset) != ncol(mat))
      warning("'eset' and 'mat' genes not identical")
    if (nrow(eset) < 2)
      stop("need at least two genes in common between 'eset' and 'mat'")
    cAmat <- mkSparseMat(mat)

    obs <- rowttests(eset, fac, tstatOnly=TRUE)[["statistic"]]
    obs <- cAmat %*% obs
    obs <- as.vector(obs)

    permMat <- matrix(0, nrow=nrow(eset), ncol=nperm)
    i <- 1L
    while (i < (nperm + 1)) {
        p1 <- sample(fac)
        permMat[ , i] <- rowttests(eset, p1, tstatOnly=TRUE)[["statistic"]]
        i <- i + 1L
    }
    permMat <- as.matrix(cAmat %*% permMat)
    pvalFromPermMat(obs, permMat)
}

local_test_factory <- function(selids, tableTest=chisq.test)
{
    force(selids)
    anyexist <- function(keys, env) {
        ans <- mget(keys, env, ifnotfound=list(NULL))
        any(sapply(ans, function(x) !is.null(x)))
    }
    
    function(start, g, prev_ans) {
        seen <- new.env(hash=TRUE, parent=emptyenv())
        for (a in prev_ans) {
            sapply(a[["nodes"]], function(i) seen[[i]] <- 1L)
        }
        ans <- lapply(start, function(sibs) {
            if (!length(sibs) || anyexist(sibs, seen)) {
                return(NULL)
            }
            sapply(sibs, function(s) seen[[s]] = 1L)
            band2gene <- lgeneIds(g, sibs)
            imat <- ann_list_to_mat(band2gene)
            ctab <- cb_buildContinTable(imat, selids, min.expected=NULL)
            if (length(ctab))
              c(list(nodes=sibs), tableTest(ctab))
            else
              list(nodes=sibs,
                   statistic=NA, parameter=NA, p.value=NA,
                   method="Test NOT PERFORMED",
                   data.name=sibs,
                   observed=NA, expected=NA, residuals=NA)
        })
        ans[!sapply(ans, is.null)]
    }
}

make_hyperg_result <- function(node, numW, numB, numDrawn, numWdrawn, OVER)
{
    n11 <- numWdrawn
    n21 <- numW - numWdrawn
    n12 <- numDrawn - numWdrawn
    n22 <- numB - n12

    obs <- matrix(c(n11, n21, n12, n22), nrow=2, ncol=2,
                  byrow=FALSE,
                  dimnames=list(
                    c("selected", "not"),
                    c(node, paste("NOT", node, sep=""))))
    ans <- .doHyperGInternal(numW, numB, numDrawn, numWdrawn, OVER)
    list(nodes=node,
         p.value=ans$p,
         method="Hypergeometric test using pyhper",
         observed=obs)
}

hg_test_factory <- function(selids, PCUT=0.05, COND=FALSE, OVER=TRUE)
{
    force(selids)
    function(start, g, prev_ans) {
        ## This is a global-level test using the Hypergeometric distribution.
        prev_ans_e <- new.env(hash=TRUE, parent=emptyenv())
        if (COND && length(prev_ans)) {
            nms <- sapply(prev_ans, function(x) x[["nodes"]])
            names(prev_ans) <- nms
            list2env(prev_ans, prev_ans_e)
        }
        
        lapply(unique(unlist(start)), function(aNode) {
            aNodeGenes <- geneIds(g, aNode)
            univ <- allGeneIds(g)
            if (COND) {
                ## XXX: this only makes sense with bottomup_tree_visitor for now.
                kids <- childrenOf(g, aNode)[[1]]
                kids_ans <- list()
                if (length(kids)) {
                    kids_ans <- mget(kids, prev_ans_e, ifnotfound=NA)
                    kids_ans <- kids_ans[sapply(kids_ans,
                                                function(x) !is.na(x[1]))]
                }
                if (length(kids_ans)) {
                    sigKids <- sapply(kids_ans, function(x) x[["nodes"]])
                    sigKids <- sigKids[sapply(kids_ans,
                                              function(x) x[["p.value"]] <= PCUT)]
                    sigKidGenes <- unlist(lapply(sigKids,
                                                 function(x) geneIds(g, x)))
                    sigKidGenes <- unique(sigKidGenes)
                    aNodeGenes <- setdiff(aNodeGenes, sigKidGenes)
                    univ <- setdiff(univ, sigKidGenes)
                    selids <- setdiff(selids, sigKidGenes)
                }
            }
            numW <- length(aNodeGenes)
            numB <- length(univ) - numW
            numWdrawn <- sum(selids %in% aNodeGenes)
            numDrawn <- length(selids)

            make_hyperg_result(aNode, numW, numB, numDrawn,
                               numWdrawn, OVER)
        })
    }
}

## these are for testing the tree_visitor functions
global_tdummy <- function(start, g, prev_ans)
{
    set.seed(0xeeff)
    lapply(unlist(start), function(n) {
        p <- runif(1)
        list(nodes=n, p=p)
    })
}

local_tdummy <- function(start, g, prev_ans)
{
    set.seed(0xeeff)
    lapply(start, function(theNodes) {
        p <- runif(1)
        list(nodes=theNodes, p=p)
    })
}

## basically, the test function tfun needs to accept a list.  For global
## mode, it will unlist and operate on individual nodes.  For local
## mode, it will operate on each element, assuming it is a "litter".

global_ndummy <- function(ans, g)
{
    ret <- lapply(ans, function(x) {
        if (x$p.value < 0.3)
          x$nodes
        else NULL
    })
    unlist(ret[!sapply(ret, is.null)])
}

ans_acceptor_factory <- function(PCUT=0.05)
{
    function(ans, g) {
        ret <- lapply(ans, function(x) {
            p <- x[["p.value"]]
            if (!is.na(p) && p <= PCUT)
              x$nodes
            else NULL
        })
        unlist(ret[!sapply(ret, is.null)])
    }
}

simple_acceptor <- function(ans, g) {
    unlist(lapply(ans, function(x) x$nodes))
}

toDF <- function(ans) {
    lapply(ans, function(sub) {
        do.call(rbind, lapply(sub, function(r) {
            data.frame(bands=paste(r$nodes, collapse=", "),
                       p.value=r$p.value)
        }))
    })
}

cb_test <- function(selids, chrtree, level,
                    dir=c("up", "down"),
                    type=c("local", "global"),
                    next.pval=0.05,
                    cond.pval=0.05,
                    conditional=FALSE)
{
    dir <- match.arg(dir)
    type <- match.arg(type)
    if (!(as.character(level) %in% names(chrtree@level2nodes)))
      stop("level must be one of ",
           paste(names(chrtree@level2nodes), collapse=", "))
    if ((type == "local" || dir == "down") && conditional)
      stop('conditional can only be used for type="global" and dir="up"')
    if (next.pval < 0 || next.pval > 1)
      stop("'pval' must be bewteen 0 and 1")

    iter <- switch(dir,
                   up=bottomup_tree_visitor,
                   down=topdown_tree_visitor,
                   stop("'dir' must be 'up' or 'down'"))
    tfun <- switch(type,
                   local=local_test_factory(selids),
                   global=hg_test_factory(selids, cond.pval, COND=conditional),
                   stop("'type' must be 'local' or 'global'"))

    start <- switch(type,
                    local={
                        oneUp <- as.character(level - 1L)
                        ## FIXME: don't we need to branch on dir here?
                        ## using either childrenOf or parentOf?
                        s <- childrenOf(chrtree, chrtree@level2nodes[[oneUp]])
                        s[listLen(s) > 0]
                    },
                    global=chrtree@level2nodes[[level]])

    iter(chrtree, start, tfun, ans_acceptor_factory(next.pval))
}

## notes: return the table because then odds ratio becomes a method,
## e.g. as expected.  This also starts to unify results from global and
## local tests.  oddratio will give NA if k x 2 table.

setMethod("universeBuilder", signature(p="KEGGHyperGParams"),
          function(p) {
                getUniverseViaKegg(p)
          })

setMethod("universeBuilder", signature(p="GOHyperGParams"),
          function(p) {
                getUniverseViaGo(p)
          })

setMethod("universeBuilder", signature(p="PFAMHyperGParams"),
          function(p) {
                getUniverseViaPfam(p)
          })

## this is for OBO
setMethod("universeBuilder", "OBOHyperGParams",
    function(p)
{
    allAnnotatedGenes <- unique(unlist(geneIds(p@datPkg@geneSetCollection)))
    intersect(unlist(universeGeneIds(p)), allAnnotatedGenes)
})

getUniverseViaGo <- function(p) {
    datPkg <- p@datPkg
    ontology <- ontology(p)
    entrezIds <- universeGeneIds(p)
    ## Return all Entrez Gene Ids that are annotated at one or more
    ## GO terms belonging to the specified GO ontology.
    ## If 'entrezIds' is given, return the intersection of 'entrezIds'
    ## and the normal return value.
    ontology <- match.arg(ontology, c("BP", "CC", "MF"))
    ## FIXME: put dispatch code here depending on whether we get DB-based
    ## maps or env-based maps
    ontIds <- aqListGOIDs(ontology)
    probe2go <- eapply(ID2GO(datPkg), function(goids) {
        if (length(goids) == 0 || (length(goids) == 1 && is.na(goids)))
          return(FALSE)
        ## FIXME: *Mapping packages don't have a list,
        ##        but just the GO ID, so we have to branch
        if (!is.character(goids[[1]])) ## the Affy case
          goids <- subListExtract(goids, "GOID", simplify=TRUE)
        if (any(goids %in% ontIds))
          return(TRUE)
        FALSE
    })
    probe2go <- probe2go[unlist(probe2go)]
    probes <- names(probe2go)
    getUniverseHelper(probes, datPkg, entrezIds)
}


getUniverseViaKegg_db <- function(p) {
    entrezIds <- universeGeneIds(p)
    SQL <- "select distinct gene_id from genes, kegg where genes._id = kegg._id"
    univ <- dbGetQuery(p@datPkg@getdb(), SQL)[[1]]
    if (!is.null(entrezIds) && length(entrezIds) > 0)
      univ <- intersect(univ, unlist(entrezIds))
    if (length(univ) < 1)
      stop("No Entrez Gene ids left in universe")
    univ
}


getUniverseViaKegg <- function(p) {
    entrezIds <- universeGeneIds(p)
##    probe2kegg <- as.list(getDataEnv("PATH", annotation(p)))
    probe2kegg <- as.list(ID2KEGG(p@datPkg))
    notNA <- sapply(probe2kegg, function(x) !(length(x) == 1 && is.na(x)))
    probe2kegg <- probe2kegg[notNA]
    probes <- names(probe2kegg)
    getUniverseHelper(probes, p@datPkg, universeGeneIds(p))
}


getUniverseViaPfam_db <- function(p) {
    entrezIds <- universeGeneIds(p)
    SQL <- "select distinct gene_id from genes, pfam where genes._id = pfam._id"
    univ <- dbGetQuery(p@datPkg@getdb(), SQL)[[1]]
    if (!is.null(entrezIds) && length(entrezIds) > 0)
      univ <- intersect(univ, unlist(entrezIds))
    if (length(univ) < 1)
      stop("No ids left in gene universe")
    univ
}



## modified to do the same thing as before (but another way)
getUniverseViaPfam <- function(p) {
    entrezIds <- universeGeneIds(p)
    obj <- get(paste0(annotation(p),'.db'))
    keytype <- .setKeytype(p)
    probes <- keys(obj, keytype=keytype, column='PFAM')
    getUniverseHelper(probes, p@datPkg, entrezIds)
}


getUniverseHelper <- function(probes, datPkg, entrezIds) {
    univ <- unique(unlist(mget(probes, ID2EntrezID(datPkg))))
    if (!missing(entrezIds) && !is.null(entrezIds) && length(entrezIds) > 0)
      univ <- intersect(univ, unlist(entrezIds))
    if (length(univ) < 1)
      stop("After filtering, there are no valid IDs that can be used as the Gene universe.\n  Check input values to confirm they are the same type as the central ID used by your annotation package.\n  For chip packages, this will still mean the central GENE identifier used by the package (NOT the probe IDs).")
    univ
}

setMethod("categoryName", signature(r="HyperGParams"),
          function(r) {
              r@categoryName
          })

setMethod("categoryName", signature(r="GOHyperGParams"),
          function(r) {
              c(r@categoryName, r@ontology)
          })



setMethod("categoryName", signature(r="LinearMParams"),
          function(r) {
              r@categoryName
          })

tree_visitor <- function(g, start, tfun, nfun, relationOf)
{
    i <- 1L
    e <- new.env(hash=TRUE, parent=emptyenv())
    prev_ans <- list()
    while (TRUE) {
        if (!length(start)) break
        ans <- tfun(start, g, prev_ans)
        k <- as.character(i)
        e[[k]] <- ans
        prev_ans <- ans
        i <- i + 1L
        accepted <- unique(nfun(ans, g))
        if (length(accepted))
          start <- relationOf(g, accepted)
        else
          start <- character(0)
    }
    as.list(e)
}

topdown_tree_visitor <- function(g, start, tfun, nfun)
{
    tree_visitor(g, start, tfun, nfun, childrenOf)
}

bottomup_tree_visitor <- function(g, start, tfun, nfun)
{
    tree_visitor(g, start, tfun, nfun, parentOf)
}

setMethod("show", signature(object="HyperGResultBase"),
          function(object) {
              cat(description(object), "\n")
              nSig <- sum(pvalues(object) < object@pvalueCutoff[1])
              cat(length(pvalues(object)), testName(object), "ids tested ")
              cat("(", nSig, " have p < ", object@pvalueCutoff[1],
                  ")\n", sep="")
              cat("Selected gene set size:", geneMappedCount(object), "\n")
              cat("    Gene universe size:", universeMappedCount(object), "\n")
              cat("    Annotation package:", annotation(object), "\n")
          })


setMethod("show", signature(object="HyperGParams"),
          function(object) {
              cat("A", class(object), "instance\n")
              cat("  category:", object@categoryName, "\n")
              cat("annotation:", object@annotation, "\n")
          })


setMethod("show", signature(object="ChrBandTree"),
          function(object) {
              cat(class(object), "object\n")
              cat("Root:", object@root, "\n")
              cat("Number of bands: ", numNodes(object@toParentGraph), "\n")
              cat("Number of levels: ", length(object@level2nodes) - 1, "\n")
          })


setMethod("show", signature(object="LinearMResultBase"),
          function(object) {
              cat(description(object), "\n")
              nSig <- sum(pvalues(object) < object@pvalueCutoff[1])
              cat(length(pvalues(object)), testName(object), "ids tested ")
              cat("(", nSig, " have p < ", object@pvalueCutoff[1],
                  ")\n", sep="")
              cat("Selected gene set size:", geneMappedCount(object), "\n")
              cat("    Gene universe size:", universeMappedCount(object), "\n")
              cat("    Annotation package:", annotation(object), "\n")
          })


setMethod("show", signature(object="LinearMParams"),
          function(object) {
              cat("A", class(object), "instance\n")
              cat("  category:", object@categoryName, "\n")
              cat("annotation:", object@annotation, "\n")
          })

.geneSetParamListFlip <- function(p){
    ## yes, it is already a list, but its backwards of what we
    ## want here.
    coll <- p@geneSetCollection
    genes <- geneIds(coll)
    genesLengths <- lapply(genes, length)
    IDs <- names(coll)
    IDReps <- rep(IDs, genesLengths)
    collFrame <- cbind(IDReps, unlist(genes))
    collList <- split(as.character(collFrame[,1]),
                      as.character(collFrame[,2]))
    return(collList)
}

getDbAnnMap <- function(col, db){
    if(col %in% columns(db))
        return(AnnotationDbi:::makeFlatBimapUsingSelect(db, col))
    else
        stop(paste("The database", db, "you are using has no", col, "data."), call. = FALSE)
}

annMapChooser <- function(col, datpkg){
    if(datpkg@installed)
        getAnnMap(col, datpkg@name)
    else
        getDbAnnMap(col, datpkg@db)
}

setMethod("ID2GO", "DatPkg",
          function(p) annMapChooser("GO", p))

setMethod("ID2GO", "GeneSetCollectionDatPkg",
          function(p) list2env(.geneSetParamListFlip(p)))

setMethod("ID2KEGG", "DatPkg",
          function(p) annMapChooser("PATH", p))

setMethod("ID2KEGG", "GeneSetCollectionDatPkg",
          function(p) list2env(.geneSetParamListFlip(p)))




setMethod("ID2EntrezID", "AffyDatPkg",
          function(p) annMapChooser("ENTREZID", p))


##FIXME: this is seriously slow - try list2env to speed up a bit
.createIdentityMap <- function(keys) {
    keys = as.list(keys)
    names(keys) = keys 
    list2env(keys)
#    e <- new.env(parent=emptyenv(), hash=TRUE)
#    for (n in keys) {
#        e[[n]] <- n
#    }
#    e
}

##this needs to handle all new, old and org based yeast packages
setMethod("ID2EntrezID", "YeastDatPkg",
          function(p) {
              bname = p@name 
              if( exists( paste(bname, "ORF", sep="")) ) 
	        return(getAnnMap("ORF", p@name))
              else if(p@installed)
                  return(.createIdentityMap(allValidKeys(p@name)))
              else
                  .createIdentityMap(allValidKeys(p@db))
})

setMethod("ID2EntrezID", "ArabidopsisDatPkg",
          function(p) {
              bname = p@name 
              if( exists( paste(bname, "ACCNUM", sep="")) ) 
	        return(getAnnMap("ACCNUM", p@name))
              else if(p@installed)
                  return(.createIdentityMap(allValidKeys(p@name)))
              else
                  .createIdentityMap(allValidKeys(p@db))
})

setMethod("ID2EntrezID", "Org.XX.egDatPkg",
          function(p) {
    if(p@installed)
        return(.createIdentityMap(allValidKeys(p@name)))
    else
        .createIdentityMap(allValidKeys(p@db))
})


setMethod("ID2EntrezID", "GeneSetCollectionDatPkg", function(p) {
    ## This method does not need to really do anything "real" since
    ## they are going to get out the ID type that they put in: no
    ## matter what.
    coll <- p@geneSetCollection
    genes <- unique(unlist(geneIds(coll)))
    collList <- split(genes,genes)
    res <- list2env(collList)
    res
})

setMethod("GO2AllProbes", "DatPkg",
          function(p, ontology=c("BP", "CC", "MF")) {
    ontIds <- aqListGOIDs(ontology)
    if(p@installed)
        go2all <- getAnnMap("GO2ALLPROBES", p@name)
    else
        go2all <- getDbAnnMap("GOALL", p@db)
    ontIds <- intersect(ontIds, ls(go2all))
    go2allOnt <- mget(ontIds, go2all, ifnotfound=NA)
    go2allOnt <- removeLengthZeroAndMissing(go2allOnt)
    list2env(go2allOnt)
})


setMethod("GO2AllProbes", "YeastDatPkg",
          function(p, ontology=c("BP", "CC", "MF")) {
    conn <- dbconn(p@db)
    schema <- dbmeta(conn, "DBSCHEMA")
    env = environment()
    if(schema == "YEASTCHIP_DB"){
        env = callNextMethod()
        return(env)
    }
    ontIds <- aqListGOIDs(ontology)
    if(p@installed)
        go2all <- getAnnMap("GO2ALLORFS", p@name)
    else
        go2all <- getDbAnnMap("GOALL", p@db)
    ontIds <- intersect(ontIds, ls(go2all))
    go2allOnt <- mget(ontIds, go2all, ifnotfound=NA)
    go2allOnt <- removeLengthZeroAndMissing(go2allOnt)
    env = list2env(go2allOnt)
    return(env)
})





setMethod("GO2AllProbes", "Org.XX.egDatPkg",
          function(p, ontology=c("BP", "CC", "MF")) {

              #db <- get("db_conn", paste("package:", p@name, sep=""))
              db <- dbconn(p@db)
              sqlQ <- "SELECT DISTINCT gene_id, go_id
              FROM genes INNER JOIN go_%s USING (_id)"
              sqlQ <- sprintf(sqlQ, tolower(ontology))
              go2egTab <- dbGetQuery(db, sqlQ)
              go2eg <- list2env(split(go2egTab[["gene_id"]], go2egTab[["go_id"]]))

              goEnvName <- paste(ontology, "OFFSPRING", sep="")
              offspring <- mget(ls(go2eg),
                                getAnnMap(goEnvName, "GO"),
                                ifnotfound=NA)
              go2allEg <- new.env(parent=emptyenv(), hash=TRUE,
                                  size=length(go2eg)*1.20)
              for (goid in names(offspring)) {
                  goids <- c(goid, offspring[[goid]])
                  goids <- goids[!is.na(goids)]
                  if (length(goids)) {
                      egids <- mget(goids, go2eg, ifnotfound=NA)
                      egids <- unique(unlist(egids))
                      go2allEg[[goid]] <- egids[!is.na(egids)]
                  }
              }
              go2allEg
          })



setMethod("GO2AllProbes", "GeneSetCollectionDatPkg",  
          function(p, ontology=c("BP", "CC", "MF")) {
            coll <- p@geneSetCollection
            ## Lets put the GeneSetCollection into a format that is
            ## easier to filter (from the left OR right)
            genes = geneIds(coll)
            genesLengths = lapply(genes, length)
            GOIDs = names(coll)
            GOIDReps = rep(GOIDs, genesLengths)
            collFrame = cbind(GOIDReps, unlist(genes))
            
            ##Now filter out all GOIDs not from the selected ontology
            ontology <- ontology
            ontology <- match.arg(ontology, c("BP", "CC", "MF"))
            ontIds <- aqListGOIDs(ontology)
            ontFilt <- collFrame[,1] %in% ontIds
            collFrame <- collFrame[ontFilt,]
            
            ##Then put things back into a list format
            result <- split(as.character(collFrame[,2]),
                            as.character(collFrame[,1]))
            
            if(length(result)==0)
              stop("no annotations for selected genes")
            list2env(result)
          })




setMethod("KEGG2AllProbes", "DatPkg",
          function(p) revmap(annMapChooser("PATH", p)))

setMethod("KEGG2AllProbes", "GeneSetCollectionDatPkg",  
          function(p) {
            coll <- p@geneSetCollection
            ## Lets put the GeneSetCollection into a format that is
            ## easier to filter (from the left OR right)
            genes = geneIds(coll)
            genesLengths = lapply(genes, length)
            KEGGIDs = names(coll)
            KEGGIDReps = rep(KEGGIDs, genesLengths)
            collFrame = cbind(KEGGIDReps, unlist(genes))
            
            ##Then put things back into a list format
            result <- split(as.character(collFrame[,2]),
                            as.character(collFrame[,1]))
            
            if(length(result)==0)
              stop("no annotations for selected genes")
            list2env(result)
          })



setMethod("isDBDatPkg","DatPkg",
          function(d) length(d@db) > 0)

















####################################################################
## DatPkgFactory methods (default has to be back in AllClasses) :(

## setMethod("DatPkgFactory", "missing", function(chip) {
##   new("AffyDatPkg", name="UNKNOWN")
## })

.strMatch <- function(pat, s){length(grep(pat, s)) > 0}

setMethod("DatPkgFactory", "character", function(chip) {
    if (.strMatch(".db$",chip))
        chip<- sub(".db","",chip)
    pkg <- paste(chip,".db",sep="")    
    if(!require(pkg, character.only = TRUE))
        stop("annotation package '", pkg, "' not available")
    
    ## Use standardized schema names to decide
    db <- get(pkg)
    conn <- dbconn(db)
    schema <- dbmeta(conn, "DBSCHEMA")
    if (schema == "YEAST_DB" || schema == "YEASTCHIP_DB")
        new("YeastDatPkg", name=chip, db=db, installed=TRUE)
    else if( schema == "ARABIDOPSIS_DB" || schema == "ARABIDOPSISCHIP_DB" )
        new("ArabidopsisDatPkg", name=chip, db=db, installed=TRUE)
    else if( .strMatch("CHIP_DB$", schema))
        new("AffyDatPkg", name=chip, db=db, installed=TRUE)
    else ## Otherwise its an ordinary org package
        new("Org.XX.egDatPkg", name=chip, db=db, installed=TRUE)
})

setMethod("DatPkgFactory", "OrgDb", function(chip) {
    ## don't act like installed package is from AnnotationHub
    pkName <- get("packageName", chip@.xData)
    if(length(pkName) > 0)
        return(DatPkgFactory(pkName))
    conn <- dbconn(chip)
    schema <- dbmeta(conn, "DBSCHEMA")
    orgn <- dbmeta(conn, "ORGANISM")
    chiptype <- dbmeta(conn, "Db type")
    name <- paste(chiptype, "for", orgn)
    if (schema == "YEAST_DB")
        new("YeastDatPkg", name=name, db=chip, installed=FALSE)
    else if( schema == "ARABIDOPSIS_DB")
        new("ArabidopsisDatPkg", name=name, db=chip, installed=FALSE)
    else ## Otherwise its an ordinary org package
        new("Org.XX.egDatPkg", name=name, db=chip, installed=FALSE)
})

setMethod("DatPkgFactory", "ChipDb", function(chip) {
    ## don't act like installed package is from AnnotationHub
    pkName <- get("packageName", chip@.xData)
    if(length(pkName) > 0)
        return(DatPkgFactory(pkName))
    conn <- dbconn(chip)
    schema <- dbmeta(conn, "DBSCHEMA")
    orgn <- dbmeta(conn, "ORGANISM")
    chiptype <- dbmeta(conn, "Db type")
    name <- paste(chiptype, "for", orgn)
    if (schema == "YEASTCHIP_DB")
        new("YeastDatPkg", name=name, db=chip, installed=FALSE)
    else if(schema == "ARABIDOPSISCHIP_DB" )
        new("ArabidopsisDatPkg", name=name, db=chip, installed=FALSE)
    else
        new("AffyDatPkg", name=name, db=chip, installed=FALSE)
})



## this is for OBO
OBOCollectionDatPkg <- function(oboCollection, geneSetCollection) {
  new("OBOCollectionDatPkg", oboCollection=oboCollection,
      oboGraph=as(oboCollection, "graphNEL"),
      geneSetCollection=geneSetCollection,
      installed = FALSE)
}
                                       
OBOHyperGParams <- function(...)
    new("OBOHyperGParams", ...)

####################################################################
## Classes and constructors to support use of GSEABase objects inside
## of GOstats:

setClass("GeneSetCollectionAnnotation", contains="character")

.GeneSetCollectionAnnotation <- function(annotation)
    new("GeneSetCollectionAnnotation", annotation)


GeneSetCollectionDatPkg <- function(geneSetCollection) 
{
    new("GeneSetCollectionDatPkg",
        geneSetCollection=geneSetCollection, installed = FALSE)
}




## Constructor function for parameter object needed by GOstats
GSEAGOHyperGParams <-
    function(name, geneSetCollection, geneIds, universeGeneIds,
             ontology, pvalueCutoff, conditional, testDirection, ...)
{
    if(!extends(class(geneIdType(geneSetCollection[[1]])),
                "GOAllFrameIdentifier"))
    {
        GSCTypeWarning <-
            paste("'geneSetCollection' elements must use GO2ALL",
                  "mappings; use GeneSetCollection constructors",
                  "that start with 'GOAllFrame'")
        stop(paste(strwrap(GSCTypeWarning, exdent=2),collapse="\n"))
    }
    if(length(geneSetCollection)==0)
        stop("geneSetCollection has length 0")
    new("GOHyperGParams",
        geneIds=geneIds,
        universeGeneIds=universeGeneIds,
        ontology=ontology,
        annotation=.GeneSetCollectionAnnotation(name),
        datPkg=GeneSetCollectionDatPkg(geneSetCollection),
        pvalueCutoff=pvalueCutoff,
        conditional=conditional,
        testDirection=testDirection,
        ...)
}

## Constructor function for parameter object needed by GOstats
GSEAKEGGHyperGParams <-
    function(name, geneSetCollection, geneIds, universeGeneIds,
             pvalueCutoff, testDirection, ...)
{

    if(length(geneSetCollection)==0)
        stop("geneSetCollection has length 0")
    new("KEGGHyperGParams",
        geneIds=geneIds,
        universeGeneIds=universeGeneIds,
        annotation=.GeneSetCollectionAnnotation(name),
        datPkg=GeneSetCollectionDatPkg(geneSetCollection),
        pvalueCutoff=pvalueCutoff,
        testDirection=testDirection,
        ...)
}


####################################################################
## configureDatPkg methods

setMethod("configureDatPkg", "character",
          function(annotation, ...) DatPkgFactory(annotation))

setMethod("configureDatPkg", "GeneSetCollectionAnnotation",
          function(annotation, object, ...) object@datPkg)

setMethod("configureDatPkg", "OrgDb",
          function(annotation, ...) DatPkgFactory(annotation))



####################################################################
## organism method

setMethod("organism", "HyperGParams",
          function(object) organism(object@datPkg) )


setMethod("organism", "GeneSetCollectionDatPkg",
          function(object) organism(object@geneSetCollection[[1]]) )

setMethod("organism", "DatPkg",
          function(object)
    if(object@installed) organism(object@name) else dbmeta(dbconn(object@db), "ORGANISM"))


setMethod("organism", "HyperGResult",
          function(object) object@organism )

setMethod("categoryToEntrezBuilder",
          signature(p="KEGGHyperGParams"),
          function(p) {
              getKeggToEntrezMap(p)
          })

setMethod("categoryToEntrezBuilder",
          signature(p="GOHyperGParams"),
          function(p) {
              if (isDBDatPkg(p@datPkg))
                getGoToEntrezMap_db(p)
              else
                getGoToEntrezMap(p)
          })

setMethod("categoryToEntrezBuilder",
          signature(p="PFAMHyperGParams"),
          function(p) {
              getPfamToEntrezMap(p)
          })

## this is for OBO
setMethod("categoryToEntrezBuilder", "OBOHyperGParams",
    function(p)
{
    geneids <- geneIds(p@datPkg@geneSetCollection)
    genesByGS <- lapply(geneids, intersect, universeGeneIds(p))
    genesByGS[lengths(genesByGS) > 0]
})


getGoToEntrezMap_db <- function(p) {
    keep.all <- switch(testDirection(p),
                       over=FALSE,
                       under=TRUE,
                       stop("Bad testDirection slot"))
    #annPkgNS <- getNamespace(annotation(p))
    #db <- get("db_conn", annPkgNS)
    datPkg <- p@datPkg
    if(datPkg@installed)
        db <- do.call(paste0(datPkg@name, "_dbconn"), list())
    else
        db <- dbconn(datPkg@db)
    univ <- unlist(universeGeneIds(p), use.names=FALSE)

    ## For over representation:
    ## Obtain all unique GO IDs from specified ontology that have at
    ## least one of the genes from geneIds(p) annotated at it.
    ##
    ## For under representation:
    ## Obtain all unique GO IDs from specified ontology that have at
    ## least one of the genes from univ annotated at it.
    ##
    ## These are the GO IDs that form the keys in our GO_to_Entrez map.
    ## First we need to handle the fact that different species have different
    ## mappings for their names. And treat installed vs bare OrgDbs differently
    ## also need to account for different names for NOSCHEMA OrgDbs
    if(datPkg@installed){
         if (is(datPkg, "YeastDatPkg") || is(datPkg, "Org.Sc.sgdDatPkg")) {
             TABLENAME = "sgd"
             GENEIDS = "systematic_name"
             goColNames <- c("go_id","ontology")
         } else {
             TABLENAME = "genes"
             if(dbmeta(db, "DBSCHEMA") == "NOSCHEMA_DB") {
                 GENEIDS <- dbListFields(db, TABLENAME)[2]
                 goColNames <- dbListFields(db, "go_all")[c(2,4)]
                 } else {
                     GENEIDS = "gene_id"
                     goColNames <- c("go_id","ontology")
                 }
         }

    } else {
        if( is(datPkg, "YeastDatPkg") || is(datPkg, "Org.Sc.sgdDatPkg") ) {
            TABLENAME = "sgd"
        } else {
            TABLENAME = "genes"
        }
        GENEIDS <- dbListFields(db, TABLENAME)[2]
        goColNames <- dbListFields(db, "go_all")[c(2,4)]
    }
     ## Not all DBs have e.g. go_bp_all, so we just get from go_all
    SQL <- "SELECT DISTINCT %s
FROM go_all INNER JOIN %s USING (_id)
WHERE %s IN (%s) AND %s=%s"
    inClause1 <- if (!keep.all)
      geneIds(p)
    else
      univ
    inClause1 <- toSQLStringSet(inClause1) # may get reused below
    SQL <- sprintf(SQL, goColNames[1], TABLENAME, GENEIDS, inClause1, goColNames[2], toSQLStringSet(ontology(p)))
    ##Check to make sure that we have some geneIds(p) to actually get GO terms FOR...
    ##basically, the "gene universe" at this point, is only those genes that
    ##have representation for on this ontology, and none of your genes were in
    ##THAT list, so there are not going to be any GO terms to find in this query.
    if(inClause1=="")
        stop("genes being tested do not have corresponding GO terms")
    wantedGO <- dbGetQuery(db, SQL)[[1]]
    ## Now collect the Entrez IDs annotated at our wantedGO IDs making
    ## sure to only keep those that are in the gene ID universe
    ## specified in p.
    SQL <- "SELECT DISTINCT %s, %s
FROM go_all INNER JOIN %s USING (_id)
WHERE %s IN (%s) AND %s IN (%s)"
    inClauseGO <- toSQLStringSet(wantedGO)
    if (!keep.all)                      # avoid recomputing
      inClause1 <- toSQLStringSet(univ)
    SQL <- sprintf(SQL, GENEIDS, goColNames[1], TABLENAME, GENEIDS, inClause1, 
                   goColNames[1], inClauseGO)
    ans <- dbGetQuery(db, SQL)
    if (nrow(ans) == 0)
        list()
    else 
        split(ans[[GENEIDS]], ans[[goColNames[1]]])
}

getGoToEntrezMap <- function(p) {
    keep.all <- switch(testDirection(p),
                       over=FALSE,
                       under=TRUE,
                       stop("Bad testDirection slot"))
    lib <- p@datPkg
    ontology <- ontology(p)
    ## Return a list mapping GO ids to the Entrez Gene ids annotated
    ## at the GO id.  Only those GO ids that are in the specified
    ## ontology and have at least one annotation in the set of 
    ## Entrez Gene ids specified by 'selected' are included.
    go2allprobes <- GO2AllProbes(lib, ontology)
    probeAnnot <- getGoToProbeMap(go2allprobes, ontology)
    ## Map to Entrez Gene and flag GO ids that don't have any
    ## annotations in our selected set.  No sense testing these.
    probeToEntrezMapHelper(probeAnnot, geneIds(p), lib, universeGeneIds(p),
                           keep.all=keep.all)
}


getKeggToEntrezMap <- function(p) {
    keep.all <- switch(testDirection(p),
                       over=FALSE,
                       under=TRUE,
                       stop("Bad testDirection slot"))
##     lib <- annotation(p) ##not unless we want to dispatch on 'character'

##  Need to instead get the value from datPkg like this:
  lib <- p@datPkg
    
    ##kegg2allprobes <- revmap(getDataEnv("PATH", lib))  ## this means I should make a generic method for this step that will dispatch on the type and return something that can be "as.list()'ed"
    kegg2allprobes <- KEGG2AllProbes(lib)
    
    probeAnnot <- getKeggToProbeMap(kegg2allprobes)    
    probeToEntrezMapHelper(probeAnnot, geneIds(p), lib, universeGeneIds(p),
                           keep.all=keep.all)
}

.setKeytype <- function(p){
    keytype <- switch(class(p@datPkg),
                      YeastDatPkg = "ORF",
                      Org.XX.egDatPkg = "ENTREZID",
                      "PROBEID")
     keytype
}

## basically this function returns a list of PFAM IDs (names) where each element contains a vector of the entrez gene IDs that go with thos PFAM IDs.
getPfamToEntrezMap <- function(p) {
    keep.all <- switch(testDirection(p),
                       over=FALSE,
                       under=TRUE,
                       stop("Bad testDirection slot"))
    keytype <- .setKeytype(p)
    ##alteration:
    obj <- get(paste0(annotation(p),'.db'))
    probes <- keys(obj, keytype=keytype, column='PFAM')
    suppressWarnings(
       tab <- select(obj, keys=probes, columns='PFAM',keytype=keytype))
    probeAnnot <- split(tab[[keytype]], f=as.factor(tab$PFAM))
    probeToEntrezMapHelper(probeAnnot, geneIds(p), p@datPkg, universeGeneIds(p),
                           keep.all=keep.all)
}


probeToEntrezMapHelper <- function(probeAnnot, selected, lib, universe,
                                   keep.all=FALSE) {
    ## Given a list 'probeAnnot' mapping category => probe, convert the probe
    ## ids to Entrez Gene ids (unless we are using YEAST, in which case we skip
    ## this step).  Then reduce the Entrez Gene ids to the specified universe
    ## and only keep those entries in the list that have a non-empty
    ## intersection with the selected genes.  If keep.all is TRUE, then we keep
    ## entries even if the list of gene IDs includes no gene ID from the
    ## selected list.
    id2entrezEnv <- ID2EntrezID(lib)
    egAnnot <- lapply(probeAnnot, function(x) {
        z <- unique(unlist(mget(unique(x), id2entrezEnv)))
        z  <- intersect(z, universe)
        ## would be nice to have a short-circuiting way to do this
        if (length(z) > 0 && (keep.all || any(selected %in% z))) {
              return(z)
        }
        NULL
    })
    notNull <- sapply(egAnnot, function(x) !is.null(x))
    egAnnot[notNull]
}


getGoToProbeMap <- function(go2allprobes, ontology, goids) {
    ## Return a list with one element for each GO id in the specified 
    ## ontology that has at least one Probe probe id annotated at it.  
    ## The elements are vectors of Probe ids.  Names are GO ids.
    ##
    probeAnnot = as.list(go2allprobes)
    if (!missing(goids))
      probeAnnot = probeAnnot[goids]

    ## Remove "all" artifact GO nodes
    whAll = match("all", names(probeAnnot), 0)
    ## This is a tad faster than if (any(whAll > 0)) 
    ## because we stop early
    for (el in whAll) {
        if (el > 0) {
            probeAnnot = probeAnnot[-whAll]
            break
        }
    }

    goids = names(probeAnnot)

    ## filter on desired GO ontology
    inOnt <- filterGOByOntology(goids, ontology)
    probeAnnot = probeAnnot[inOnt]

    ## remove any GO ids that don't map to an probe id (NA)
    removeLengthZeroAndMissing(probeAnnot)
}


getKeggToProbeMap <- function(kegg2allprobes, keggIds) {
    probeAnnot = as.list(kegg2allprobes)
    if (!missing(keggIds))
      probeAnnot = probeAnnot[keggIds]
    removeLengthZeroAndMissing(probeAnnot)
}

getPfamToProbeMap <- function(pfam2allprobes, pfamIds) {
    probeAnnot = as.list(pfam2allprobes)
    if (!missing(pfamIds))
      probeAnnot = probeAnnot[pfamIds]
    removeLengthZeroAndMissing(probeAnnot)
}



removeLengthZeroAndMissing <- function(map) {
    notNA = sapply(map, function(x) {
        len = length(x)
        !(len == 0 || (len == 1 && is.na(x)))
    })
    map <- map[notNA]
}    

splitOrfByPfam <- function(ypfEnv){
  ## There should be a PFAM to ORF data set in addition to YEASTPFAM...a
  ## YEASTPFAM2PROBE, but for now we invert
  probe2pfam <- as.list(ypfEnv)
  probe2pfam <- removeLengthZeroAndMissing(probe2pfam)
  pfamlen <- listLen(probe2pfam)
  orf <- rep(names(probe2pfam), pfamlen)
  pfam <- unlist(probe2pfam)
  orfByPfam <- split(orf, pfam)
  orfByPfam

}

getDataEnv <- function(name, lib) {
    get(paste(lib, name, sep=""))
}

## here getAnnMap is NOT returning a bimap.
## It's also not really accessing one.
getOrganism <- function(chip){
    paste("ORGANISM:", getAnnMap("ORGANISM", chip), sep="")
}

setMethod("geneIds", "LinearMParams", function(object, ...) object@geneIds)
setReplaceMethod("geneIds", "LinearMParams", function(object, value) {
    object@geneIds <- value
    object
})


setMethod("testDirection", "LinearMParams", function(r) r@testDirection)
setReplaceMethod("testDirection", "LinearMParams", function(r, value) {
    r@testDirection <- value
    r
})

setMethod("universeGeneIds", "LinearMParams", function(r) r@universeGeneIds)
setReplaceMethod("universeGeneIds", "LinearMParams", function(r, value) {
    r@universeGeneIds <- value
    r
})

setMethod("categoryName", "LinearMParams", function(r) r@categoryName)
setReplaceMethod("categoryName", "LinearMParams", function(r, value) {
    r@categoryName <- value
    r
})

setMethod("annotation", "LinearMParams", function(object) object@annotation)
setReplaceMethod("annotation", c("LinearMParams", "character"),
                 function(object, value) {
                   object@annotation <- value
                   object@datPkg <- DatPkgFactory(value)
                   object
                 })

setMethod("pvalueCutoff", "LinearMParams", function(r) r@pvalueCutoff)
setReplaceMethod("pvalueCutoff", "LinearMParams", function(r, value) {
    r@pvalueCutoff <- value
    r
})


setMethod("conditional", "LinearMParams", function(r) r@conditional)

setReplaceMethod("conditional", c("LinearMParams", "logical"),
                 function(r, value) {
                     if (is.na(value))
                         stop("value must be TRUE or FALSE")
                     r@conditional <- value
                     r
                 })

.makeValidParams <- function(object) {
    ## check HyperGParams instance for validity.
    ## If we can fix it, we do (and issue a warning)
    ## Return a more valid instance or error
    datPkg <- object@datPkg
    if (class(datPkg) !="GeneSetCollectionDatPkg") {
        if(!datPkg@installed && class(datPkg) !="OBOCollectionDatPkg") {
           ## use direct access so we don't rebuild the datPkg
            object@annotation <- datPkg@name
        } else {
            ## Check if annotation is installed but instantiated using the OrgDb
            ## rather than the name of the OrgDb
            if(is(annotation(object), "OrgDb")) annotation(object) <- datPkg@name
            ## Check if annotation has been written "long form". If it is,
            ## then shorten the name appropriately.
            ann <- annotation(object)
            if (length(ann) != 1)
                stop("'annotation' must be character(1)", .Call=FALSE)
            if (grepl(".db$", ann))
                annotation(object) <- sub("\\.db$", "", ann)
        }
    }
    sel <- geneIds(object)
    if (is.list(sel)) {
        warning("converting geneIds from list to atomic vector via unlist")
        sel <- unlist(sel)
    }
    if (any(duplicated(sel))) {
        warning("removing duplicate IDs in geneIds")
        sel <- unique(sel)
    }
    geneIds(object) <- sel
    univ <- universeGeneIds(object)
    if (length(univ)) {
        if (is.list(univ)) {
            warning("converting univ from list to atomic vector via unlist")
            univ <- unlist(univ)
        }
        if (typeof(sel) != typeof(univ))
            stop(paste("geneIds and universeGeneIds must have the same mode\n",
                       "geneIds:", typeof(sel), "\n",
                       "universeGeneIds:", typeof(univ)), .Call=FALSE)
        if (any(duplicated(univ))) {
            warning("removing duplicate IDs in universeGeneIds")
            univ <- unique(univ)
        }
        universeGeneIds(object) <- univ
        if (!all(sel %in% univ)) {
            warning("removing geneIds not in universeGeneIds")
            sel <- intersect(sel, univ)
            if (!length(sel))
                stop("no geneIds in universeGeneIds", .Call=FALSE)
            geneIds(object) <- sel
        }
    }
    pv <- pvalueCutoff(object)
    if (pv > 1 || pv < 0){
        stop("invalid pvalueCutoff, must be between 0 and 1", .Call=FALSE)}
    return(object)
}

setMethod("makeValidParams", "HyperGParams", .makeValidParams)


setMethod("geneIds", "HyperGParams", function(object, ...) object@geneIds)
setReplaceMethod("geneIds", "HyperGParams", function(object, value) {
    object@geneIds <- value
    object
})

setMethod("categorySubsetIds", "HyperGParams", function(r) r@categorySubsetIds)
setReplaceMethod("categorySubsetIds", "HyperGParams", function(r, value) {
    r@categorySubsetIds <- value
    r
})

setMethod("testDirection", "HyperGParams", function(r) r@testDirection)
setReplaceMethod("testDirection", "HyperGParams", function(r, value) {
    r@testDirection <- value
    r
})

setMethod("universeGeneIds", "HyperGParams", function(r) r@universeGeneIds)
setReplaceMethod("universeGeneIds", "HyperGParams", function(r, value) {
    r@universeGeneIds <- value
    r
})

setMethod("categoryName", "HyperGParams", function(r) r@categoryName)
setReplaceMethod("categoryName", "HyperGParams", function(r, value) {
    r@categoryName <- value
    r
})

setMethod("annotation", "HyperGParams", function(object) object@annotation)
setReplaceMethod("annotation", c("HyperGParams", "character"),
                 function(object, value) {
                   object@annotation <- value
                   object@datPkg <- DatPkgFactory(value)
                   object
                 })

setMethod("pvalueCutoff", "HyperGParams", function(r) r@pvalueCutoff)
setReplaceMethod("pvalueCutoff", "HyperGParams", function(r, value) {
    r@pvalueCutoff <- value
    r
})


setMethod("conditional", "HyperGParams", function(r) FALSE)

setMethod("conditional", "ChrMapHyperGParams", function(r) r@conditional)

setReplaceMethod("conditional", c("ChrMapHyperGParams", "logical"),
                 function(r, value) {
                     if (is.na(value))
                       stop("value must be TRUE or FALSE")
                     r@conditional <- value
                     r
                 })

setMethod("conditional", "GOHyperGParams", function(r) r@conditional)

.replaceGOConditional <- function(r, value) {
  if (is.na(value))
    stop("value must be TRUE or FALSE")
  r@conditional <- value
  r
}
setReplaceMethod("conditional", c("GOHyperGParams", "logical"), function(r, value) .replaceGOConditional(r, value))

## this is for OBO
setMethod("conditional", "OBOHyperGParams", function(r) r@conditional)

setReplaceMethod(
    "conditional", c("OBOHyperGParams", "logical"),
    function(r, value)
{
    if (length(value) != 1 || is.na(value))
        stop("'value' must be logical(1) and not NA")
    r@conditional <- value
    r
})

setMethod("ontology", "HyperGParams", function(object) NA)

setMethod("ontology", "GOHyperGParams", function(object) object@ontology)

.replaceGOOntology <- function(r, value) {
  if (is.na(value) || length(value) != 1)
    stop("value must be a length one character vector")
  r@ontology <- value
  r
}
setReplaceMethod("ontology", c("GOHyperGParams", "character"), function(r, value) .replaceGOOntology(r, value))




##FIXME, this shouldn't be as hard as it is :-( :-(
## autogenerate accessors
## theSlots <- slotNames("HyperGParams")

## getter <- function(r) slot(r, s)
## getterArgs <- alist(r=)

## setter <- function(r, v) {
##     slot(r, s) <- v
##     r
## }
## setterArgs <- alist(r=, v=)

## for (s in theSlots) {
##     cat("Creating get method for ", s, "\n")
##     newGetArgs <- eval(substitute(c(r),
##                                   list(r=getGeneric(s)@signature[1])))
##     names(getterArgs) <- newGetArgs
##     formals(getter) <- getterArgs
##     setMethod(s, signature("HyperGParams"), getter)

##     sSET <- paste(s, "<-", sep="")
##     cat("Creating set method for ", s, "\n")
##     newSetArgs <- eval(substitute(c(r, v),
##                                   list(r=getGeneric(sSET)@signature[1],
##                                        v=getGeneric(sSET)@signature[2])))
##     names(setterArgs) <- newSetArgs
##     formals(setter) <- setterArgs
    
##     setReplaceMethod(s, signature("HyperGParams"), setter)
## }

setClass("DatPkg",
         contains="VIRTUAL",
         representation=representation(
             name="character",
             db="ANY",
             installed="logical"))

##Annotion package DatPkgs
setClass("AffyDatPkg", contains="DatPkg")
setClass("YeastDatPkg", contains="DatPkg")
setClass("ArabidopsisDatPkg", contains="DatPkg")
setClass("Org.XX.egDatPkg", contains="DatPkg")


## Other types of DatPkgs
setClass("GeneSetCollectionDatPkg", contains="DatPkg",
         representation=representation(
           geneSetCollection="GeneSetCollection"))

## OBO gene set collection DatPkgs
setClass("OBOCollectionDatPkg", contains="DatPkg",
         representation=representation(
           oboCollection="OBOCollection",
           oboGraph="graph",
           geneSetCollection="GeneSetCollection"))


## These generics needed for AllClasses
setGeneric("configureDatPkg",
           function(annotation, ...) standardGeneric("configureDatPkg"))

setGeneric("DatPkgFactory",
           function(chip) standardGeneric("DatPkgFactory"),
           useAsDefault=function(chip) new("AffyDatPkg", name="UNKNOWN"))


setClass("HyperGParams",
         contains="VIRTUAL",
         representation=representation(geneIds="ANY",
           universeGeneIds="ANY",
           annotation="ANY",
           datPkg="DatPkg",
           categorySubsetIds="ANY",
           categoryName="character",
           pvalueCutoff="numeric",
           testDirection="character"),
         prototype=prototype(
           pvalueCutoff=0.01,
           testDirection="over",
           datPkg=DatPkgFactory()
           ))


setClass("GOHyperGParams",
         representation(ontology="character",
                        conditional="logical",
                        orCutoff="numeric",
                        minSizeCutoff="numeric",
                        maxSizeCutoff="numeric"),
         contains="HyperGParams",
         prototype=prototype(
           categoryName="GO",
           conditional=FALSE,
           orCutoff=1,
           minSizeCutoff=0,
           maxSizeCutoff=Inf,
           annotation="GO"))


setClass("KEGGHyperGParams",
         contains="HyperGParams",
         prototype=prototype(
           categoryName="KEGG",
           annotation="KEGG"))

setClass("PFAMHyperGParams",
         contains="HyperGParams",
         prototype=prototype(categoryName="PFAM"))

## this probably should not go here
setClass("GeneSetCollectionAnnotation", contains="character")

.GeneSetCollectionAnnotation <- function(annotation)
      new("GeneSetCollectionAnnotation", annotation)

## class for OBO gene set testing
setClass("OBOHyperGParams",
         contains="HyperGParams",
         representation(conditional="logical",
                        orCutoff="numeric",
                        minSizeCutoff="numeric",
                        maxSizeCutoff="numeric"),
         prototype=prototype(
           categoryName="OBO",
           conditional=FALSE,
           orCutoff=1,
           minSizeCutoff=0,
           maxSizeCutoff=Inf,
           annotation=.GeneSetCollectionAnnotation("OBO")))

setClass("ChrMapHyperGParams",
         contains="HyperGParams",
         representation=representation(
           chrGraph="graph",
           conditional="logical"),
         prototype=prototype(
           categoryName="ChrMap",
           chrGraph=new("graphNEL", edgemode="directed")))

setClass("HyperGResultBase",
         representation(annotation="character",
                        geneIds="ANY",
                        testName="character",
                        pvalueCutoff="numeric",
                        testDirection="character"),
         contains="VIRTUAL",
         prototype=prototype(pvalueCutoff=0.01))


setClass("HyperGResult",
         contains="HyperGResultBase",
         representation=representation(
           pvalues="numeric",
           oddsRatios="numeric",
           expectedCounts="numeric",
           catToGeneId="list",
           organism="character"))

setClass("KEGGHyperGResult",
         contains="HyperGResult")

setClass("PFAMHyperGResult",
         contains="HyperGResult")

setClass("ChrMapHyperGResult",
         contains="HyperGResultBase",
         representation=representation(
           pvalue.order="integer",
           conditional="logical",
           chrGraph="graph"),
         prototype=prototype(
           chrGraph=new("graphNEL", edgemode="directed")))

setClass("ChrBandTree",
         representation=representation(
           toParentGraph="graph",
           toChildGraph="graph",
           root="character",
           level2nodes="list"))

## this is for OBO
setClass("OBOHyperGResult",
         contains="HyperGResultBase",
         representation=representation(
           goDag="graph",
           gscDescriptions="character",
           pvalue.order="integer",
           conditional="logical"),
         prototype=prototype(
           testName="GO",
           pvalueCutoff=0.01,
           goDag=new("graphNEL")))



### DS: Similar structures for 'linear model'-based tests. minSize is
### needed mostly for conditional tests, because we don't want to leave
### out sub-genesets that are too small, even if they are significant
### (or do we?)

### ML: Pushed the ChrMap stuff up to the general classes; not sure
### why we even need a class for each type of category? Seems too
### formal for something that can be so arbitrary. Every type of
### category (GO, KEGG, chromosome bands, ...) fits into a graph.

### ML: Current thoughts are that every Params object should have a
### GeneSetCollection that refers (through its CollectionType) to an
### ontology. From that, a graph can be automatically derived. This
### favors composition over inheritance.

setClass("LinearMParams",
         representation =
         representation(geneStats="numeric", # this needs to be named
                        universeGeneIds="ANY", # not used, probably should be
                        annotation="character", # just carried through...
                        datPkg="DatPkg", # not currently used
                        categorySubsetIds="ANY", # not used, probably should be
                        categoryName="character",
                        pvalueCutoff="numeric",
                        minSize="integer",
                        testDirection="character",## less, greater, two-sided?
#### FIXME: could the graph be derived from the 'CollectionType's in the
#### 'GeneSetCollection', using info from e.g. GO.db?
                        graph = "graph",
                        conditional="logical",
                        ## instead of putting attributes on graph
                        gsc = "GeneSetCollection"), 
         prototype=
         prototype(pvalueCutoff=0.01,
                   testDirection="up",
                   minSize=5L,
                   datPkg=DatPkgFactory(),
                   conditional = FALSE,
                   graph = new("graphNEL", edgemode = "directed")),
         validity = function(object) {
           if (length(object@geneStats) && is.null(names(object@geneStats)))
             "'geneStats' must have names that match those in the gene sets"
           else NULL
         })

setClass("ChrMapLinearMParams",
         contains="LinearMParams",
         prototype =
         prototype(categoryName="ChrMap"))





setClass("LinearMResultBase",
         representation(annotation="character",
                        geneIds="ANY", # FIXME: should this be universeGeneIds?
                        testName="character",
                        pvalueCutoff="numeric",
                        minSize="integer",
                        testDirection="character",
                        conditional="logical",
                        graph = "graph",
                        gsc = "GeneSetCollection"),
         contains="VIRTUAL",
         prototype=prototype(pvalueCutoff=0.01))


setClass("LinearMResult",
         contains="LinearMResultBase",
         representation=
         representation(pvalues="numeric",
                        pvalue.order="integer",
                        effectSize="numeric"))


setClass("ChrMapLinearMResult",
         contains = "LinearMResult")

setMethod("hyperGTest",
          signature(p="HyperGParams"),
          function(p) .hyperGTestInternal(p))

.hyperGTestInternal <- function(p, className="HyperGResult") {
    className <- paste(categoryName(p), className, sep="")
    p <- makeValidParams(p)
    origGeneIds <- geneIds(p)
    universeGeneIds(p) <- universeBuilder(p)
    selected <- intersect(geneIds(p), universeGeneIds(p))
    geneIds(p) <- selected
    cat2Entrez <- categoryToEntrezBuilder(p)
    stats <- .doHyperGTest(p, cat2Entrez, list(),
                           selected)
    ord <- order(stats$p)
    new(className,
        pvalues=stats$p[ord],
        oddsRatios=stats$odds[ord],
        expectedCounts=stats$expected[ord],
        catToGeneId=cat2Entrez[ord],
        annotation=annotation(p),
        geneIds=geneIds(p),
        testName=categoryName(p),
        pvalueCutoff=pvalueCutoff(p),
        testDirection=testDirection(p),
        organism=organism(p))
}

chrMap_hg_test <- function(p) {
    chrGraph <- p@chrGraph

    nodeDataDefaults(chrGraph, "pvalue") <- 1
    nodeDataDefaults(chrGraph, "geneIds") <- numeric(0)
    nodeDataDefaults(chrGraph, "condGeneIds") <- numeric(0)
    nodeDataDefaults(chrGraph, "oddsRatio") <- numeric(0)
    nodeDataDefaults(chrGraph, "expCount") <- numeric(0)

    ## now iterate leaves first doing tests and conditioning
    ## on all significant children.
    ## FIXME: consider replacing with RBGL tsort?
    needsProc <- chrGraph
    complete <- character(0)
    SIGNIF <- p@pvalueCutoff
    cat2Entrez <- nodeData(chrGraph, attr="geneIds")
    while (length(nodes(needsProc))) {
        numKids <- sapply(edges(needsProc), length)
        noKids <- names(numKids[numKids == 0])
        curCat2Entrez <- cat2Entrez[noKids]
        if (p@conditional) {
            curCatKids <- edges(chrGraph)[names(curCat2Entrez)]
            curCatKids <- removeLengthZero(curCatKids)
            if (length(curCatKids)) { ## sanity check
                ## they should be all complete
                stopifnot(all(unlist(curCatKids) %in% complete))
            }
            curCat2Entrez <- removeSigKidGenes(curCatKids, chrGraph,
                                               curCat2Entrez,
                                               SIGNIF, cat2Entrez)
            ## Store the conditioned cat => entrez map
            nodeData(chrGraph, n=names(curCat2Entrez),
                     attr="condGeneIds") <- curCat2Entrez
        }
        stats <- .doHyperGTest(p, curCat2Entrez, cat2Entrez,
                               p@geneIds)

        ## store the pvals, mark these nodes as complete,
        ## then compute the next set of nodes to do.
        noKids <- names(curCat2Entrez)
        ## drop names on pvals to avoid weird names upon unlisting
        nodeData(chrGraph, n=noKids,
                 attr="pvalue") <- as.numeric(stats$p)
        nodeData(chrGraph, n=noKids,
                 attr="oddsRatio") <- as.numeric(stats$odds)
        nodeData(chrGraph, n=noKids,
                 attr="expCount") <- as.numeric(stats$expected)
        complete <- c(complete, noKids)
        hasKids <- names(numKids[numKids > 0])
        needsProc <- subGraph(hasKids, needsProc)
    } ## end while
    p@chrGraph <- chrGraph
    p
}


removeLengthZero <- function(x) {
    wanted <- sapply(x, function(z) length(z) > 0)
    x[wanted]
}

removeSigKidGenes <-
    function(curCatKids, goDag, curCat2Entrez, SIGNIF,
             cat2Entrez)
{
    if (length(curCatKids)) {
        ## keep only those kids with SIGNIF pvalue
        curCatKids <- lapply(curCatKids, function(x) {
            pvKids <- nodeData(goDag, n=x, attr="pvalue")
            idx <- which(pvKids < SIGNIF)
            if (length(idx))
              x[idx]
            else
              character(0)
        })
        curCat2EntrezCond <- list()
        for (goid in names(curCat2Entrez)) {
            ## remove entrez ids that came from
            ## SIGNIF children
            kids <- curCatKids[[goid]]
            if (length(kids)) {
                kidEgs <- unlist(cat2Entrez[kids])
                newEgs <- setdiff(curCat2Entrez[[goid]], kidEgs)
                ## newEgs may be length 0
                curCat2EntrezCond[[goid]] <- newEgs
            } else {
                curCat2EntrezCond[[goid]] <- curCat2Entrez[[goid]]
            }
        }
        curCat2Entrez <- curCat2EntrezCond
    }
    curCat2Entrez
}

setMethod("hyperGTest",
          signature(p="ChrMapHyperGParams"),
          function(p) {
              p <- makeValidParams(p)
              if (numNodes(p@chrGraph) == 0)
                p@chrGraph <- makeChrBandGraph(annotation(p), p@universeGeneIds)

              univ <- unlist(nodeData(p@chrGraph, attr="geneIds"))
              univ <- unique(univ)
              p@universeGeneIds <- univ
              ## preserve names on geneIds
              p@geneIds <- p@geneIds[p@geneIds %in% p@universeGeneIds]

              p <- chrMap_hg_test(p)
              pvals <- unlist(nodeData(p@chrGraph, attr="pvalue"))
              pvord <- order(pvals)
              new("ChrMapHyperGResult",
                  chrGraph=p@chrGraph,
                  annotation=p@annotation,
                  geneIds=p@geneIds,
                  testName=categoryName(p),
                  testDirection=p@testDirection,
                  pvalueCutoff=p@pvalueCutoff,
                  conditional=p@conditional,
                  pvalue.order=pvord)
          })

.doHyperGInternal <- function(numW, numB, numDrawn, numWdrawn, over) {
    n21 <- numW - numWdrawn
    n12 <- numDrawn - numWdrawn
    n22 <- numB - n12

    odds_ratio <-  (numWdrawn * n22) / (n12 * n21)

    expected <- (numWdrawn + n12) * (numWdrawn + n21)
    expected <- expected / (numWdrawn + n12 + n21 + n22)

    if (over) {
        ## take the -1 because we want evidence for as extreme or more
        pvals <- phyper(numWdrawn - 1L, numW, numB,
                        numDrawn, lower.tail=FALSE)
    } else {
        pvals <- phyper(numWdrawn, numW, numB,
                        numDrawn, lower.tail=TRUE)
    }
    list(p=pvals, odds=odds_ratio, expected=expected)
}


.doHyperGTest <- function(p, curCat2Entrez, cat2Entrez, selected) {
    ## Here is how we conceptualize the test:
    ##
    ## The urn contains genes from the gene universe.  Genes annotated at a
    ## given category term are white and the rest black.
    ##
    ## The number drawn is the size of the selected gene list.  The
    ## number of white drawn is the size of the intersection of the
    ## selected list and the genes annotated at the category.
    ##
    ## In the conditional case, the category ID annotation set has
    ## been reduced and we also adjust the selected list (num drawn)
    ## and gene universe according to what was removed by the
    ## conditioning.
    ##
    ## Here's a diagram based on using GO as the category:
    ##
    ##          inGO    notGO
    ##          White   Black
    ## selected  n11     n12
    ## not       n21     n22
    ##
    if (conditional(p)) {
        cat2RemovedEntrez <- lapply(names(curCat2Entrez),
                                    function(goid) {
                                        setdiff(cat2Entrez[[goid]],
                                                curCat2Entrez[[goid]])
                                    })

        ## White balls removed from urn by conditioning
        numSelectedRemoved <- sapply(cat2RemovedEntrez,
                                     function(x) sum(selected %in% x))

        numUnivRemoved <- listLen(cat2RemovedEntrez)
        ## Black balls removed from urn by conditioning
        numOtherRemoved <- numUnivRemoved - numSelectedRemoved
    } else {
        ##we need to deal with the case where the length of curCat2Entrez
        ## is zero, 
        numSelectedRemoved <- rep(0, min(length(curCat2Entrez), 1))
        numOtherRemoved <- numUnivRemoved <- numSelectedRemoved
    }

    ## Num white drawn (n11)
    numWdrawn <- sapply(curCat2Entrez,
                        function(x) sum(selected %in% x))

    if(length(numWdrawn) == 0 ) numWdrawn = 0 
    ## Num white
    numW <- listLen(curCat2Entrez)
    if(length(numW) == 0 ) numW = 0

    ## Num black
    numB <- length(universeGeneIds(p)) - numUnivRemoved - numW

    ## Num drawn
    numDrawn <- length(selected) - numSelectedRemoved

    over <- testDirection(p) == "over"
    .doHyperGInternal(numW, numB, numDrawn, numWdrawn, over)
}

setMethod(hyperg, "list",
    function(assayed, significant, universe, representation=c("over", "under"),
             ...)
{
    .hyperg <- selectMethod(hyperg, "character")
    result <- Map(.hyperg, assayed, significant,
        MoreArgs=list(universe=universe, representation=representation, ...))
    as.data.frame(t(simplify2array(result)))
})

setMethod(hyperg, "character",
     function(assayed, significant, universe, representation=c("over", "under"))
{
    if (!all(assayed %in% universe))
        stop("some 'assayed' genes not in 'universe'")
    if (!all(significant %in% assayed))
        stop("some 'significant' genes not in 'assayed'")
    representation <- match.arg(representation)
    test <- representation== "over"

    white.balls.drawn <- length(intersect(significant, assayed))
    white.balls.in.urn <- length(assayed)
    total.balls.in.urn <- length(universe)
    black.balls.in.urn <- total.balls.in.urn - white.balls.in.urn
    balls.pulled.from.urn <- length(assayed)

    c(assayed=length(assayed), significant=length(significant),
      universe=length(universe), representation=representation,
      .doHyperGInternal(white.balls.in.urn, black.balls.in.urn,
                        balls.pulled.from.urn, white.balls.drawn,
                        test))
})

cb_parse_band_Mm <- function(x) {
    ## Given a chromosome band annotation (see examples below),
    ## return a vector giving the path from the root:
    ##
    ## 10 B3.5 => 10, 10 B, 10 B3, 10B3.5
    ##

    ##remove cM ou centrome info for the moment
    ## not sure how to able it otherwise
    x <- gsub("[[:space:]].*cM$","" , x)
    x <- gsub("[[:space:]].*centromere$","", x)

    if(regexpr("[[:space:]]", x)==-1){
        return(x)
    }else{
        chr.pos <- regexpr("[[:space:]]", x)
        chr <-  substr(x, 1, chr.pos-1)

        chrBand <- substr(x, chr.pos+1, chr.pos+1)
        sbs <- strsplit(substr(x, chr.pos+2, nchar(x)), "")[[1]]

        if(length(sbs)==0) {
            return(c(chr, paste(chr, chrBand, sep=" ")))
        }else{

            bands <- character(length(sbs)+1)
            bands[1] <- paste(chr, chrBand, sep=" ")
            i <- 1
            j <- 2
            prev <- bands[1]
            print(prev)
            while (TRUE) {
                if (sbs[i] == ".") {
                    prev <- paste(prev, ".", sep="")
                    i <- i + 1
                    next
                }
                bands[j] <- paste(prev, sbs[i], sep="")
                prev <- bands[j]
                i <- i + 1
                j <- j + 1
                if (i > length(sbs))
                  break
            }
        }
        if (i == j)
          return(bands[1:(i-1)])
        else
          return(bands)
    }
}

setGeneric("hyperg", function(assayed, significant, universe,
                              representation=c("over", "under"), ...)
           standardGeneric("hyperg"),
           signature="assayed")

setGeneric("hyperGTest", 
           function(p) standardGeneric("hyperGTest"),
           valueClass="HyperGResultBase")

setGeneric("linearMTest", 
           function(p) standardGeneric("linearMTest"),
           valueClass="LinearMResultBase")

setGeneric("categoryToEntrezBuilder", 
           function(p) standardGeneric("categoryToEntrezBuilder"))

setGeneric("universeBuilder", 
           function(p) standardGeneric("universeBuilder"))


## Accessors for HyperGResult and LinearMResult objects
setGeneric("pvalues", function(r) standardGeneric("pvalues"))

setGeneric("effectSize", function(r) standardGeneric("effectSize"))

setGeneric("geneCounts", function(r) standardGeneric("geneCounts"))

setGeneric("universeCounts", function(r) standardGeneric("universeCounts"))

setGeneric("universeMappedCount",
           function(r) standardGeneric("universeMappedCount"))

setGeneric("geneMappedCount",
           function(r) standardGeneric("geneMappedCount"))

setGeneric("chrGraph",
           function(r) standardGeneric("chrGraph"))

setGeneric("geneIdUniverse", signature="r",
           function(r, cond=TRUE) standardGeneric("geneIdUniverse"))

## generic "annotation" defined in Biobase

#setGeneric("geneIds", function(r, ...) standardGeneric("geneIds"))
#setGeneric("geneIds<-", function(r, value) standardGeneric("geneIds<-"))

setGeneric("geneIdsByCategory", signature="r",
           function(r, catids = NULL) standardGeneric("geneIdsByCategory"))

setGeneric("sigCategories", signature="r",
           function(r, p) standardGeneric("sigCategories"))

setGeneric("testName", function(r) standardGeneric("testName"))

setGeneric("pvalueCutoff", function(r) standardGeneric("pvalueCutoff"))
setGeneric("pvalueCutoff<-", function(r, value) standardGeneric("pvalueCutoff<-"))

setGeneric("testDirection", function(r) standardGeneric("testDirection"))
setGeneric("testDirection<-",
           function(r, value) standardGeneric("testDirection<-"))

setGeneric("oddsRatios", function(r) standardGeneric("oddsRatios"))

setGeneric("expectedCounts",
           function(r) standardGeneric("expectedCounts"))


## accoessors for HyperGParams
setGeneric("categoryName", function(r) standardGeneric("categoryName"))

setGeneric("universeGeneIds", function(r) standardGeneric("universeGeneIds"))

#setGeneric("ontology", function(r) standardGeneric("ontology"))

setGeneric("conditional", function(r) standardGeneric("conditional"))

setGeneric("categoryName<-", function(r, value) standardGeneric("categoryName<-"))

setGeneric("universeGeneIds<-",
           function(r, value) standardGeneric("universeGeneIds<-"))

setGeneric("ontology<-", function(r, value) standardGeneric("ontology<-"))

setGeneric("conditional<-", function(r, value) standardGeneric("conditional<-"))

setGeneric("categorySubsetIds", function(r) standardGeneric("categorySubsetIds"))

setGeneric("categorySubsetIds<-", function(r, value) standardGeneric("categorySubsetIds<-"))

setGeneric("htmlReport", function(r, file="", append=FALSE, label="",
                                  digits=3, summary.args=NULL)
           standardGeneric("htmlReport"),
           signature=c("r"))

setGeneric("makeValidParams", function(object) {
           v <- standardGeneric("makeValidParams")
           if (class(object) != class(v))
             stop(paste("makeValidParams generic must return same class ",
                        "as its argument 'object'"))
           v
           })

## DatPkg
setGeneric("ID2GO", function(p) standardGeneric("ID2GO"))
setGeneric("ID2EntrezID", function(p) standardGeneric("ID2EntrezID"))
setGeneric("GO2AllProbes", signature=c("p"),
           function(p, ontology) standardGeneric("GO2AllProbes"))
setGeneric("isDBDatPkg", function(d) standardGeneric("isDBDatPkg"))

setGeneric("KEGG2AllProbes", function(p) standardGeneric("KEGG2AllProbes"))
setGeneric("ID2KEGG", function(p) standardGeneric("ID2KEGG"))



## ChrBandTree accessors
setGeneric("lgeneIds", function(r, ...) standardGeneric("lgeneIds"))

setGeneric("childrenOf",
           function(g, n, ...) standardGeneric("childrenOf"))

setGeneric("parentOf",
           function(g, n, ...) standardGeneric("parentOf"))

setGeneric("allGeneIds",
           function(g, ...) standardGeneric("allGeneIds"))

setGeneric("treeLevels",
           function(g, ...) standardGeneric("treeLevels"))

setGeneric("level2nodes",
           function(g, level, ...) standardGeneric("level2nodes"))

## x: a character vector of GO category IDs
augmentByAncestors = function(x) {
    if (!requireNamespace("GO.db"))
        stop("Use 'BiocManager::install(\"GO.db\")' to install the GO.db package")
   ## NOTE: this function implicitly does require("GO.db")
   s1 = x %in% ls(getAnnMap("MFANCESTOR", "GO", load=TRUE))
   s2 = x %in% ls(getAnnMap("BPANCESTOR", "GO"))
   s3 = x %in% ls(getAnnMap("CCANCESTOR", "GO"))
   s4 = is.na(x)

   ## throw warning if some not found
   notFound = x[!(s1|s2|s3|s4)]
   if(length(notFound)>0) {
     notFound[notFound==""] = "(zero length string)"
     got = if(length(notFound)>7) {
       paste(paste(notFound[1:7], collapse=", "), ", ... ")
     } else {
       paste(notFound, collapse=", ")
     }
     warning(if(length(notFound)==1) {
       sprintf("cateGOry: GO term %s is mentioned in 'x' but was not found in the GO package.\n", got)
     } else {
       sprintf("cateGOry: GO terms %s were mentioned in 'x' but were not found in the GO package.\n", got)
     })
   }

   res = vector(length=length(x), mode="list")
   res[s1] = mget(x[s1], GO.db::GOMFANCESTOR)
   res[s2] = mget(x[s2], GO.db::GOBPANCESTOR)
   res[s3] = mget(x[s3], GO.db::GOCCANCESTOR)
   res[s4] = NULL
   ## the genes in x itself are not their own ancestors,
   ## so we need to add them explicitely
   res     = mapply(c, res, x)
   names(res) = x
   return(res)
}

augmentByAncestorsSmart = function(x) {
   ## need to do the expensive search only once for each unique ID
   ux = unique(x)
   mt = match(x, ux)
   augmentByAncestors(ux)[mt]
}


## x: vector of gene identifiers (need not be unique)
## GOcat: vector of same length as x, with GO categories
cateGOry  = function(x, categ, sparse=FALSE) {

  if(!is.character(x))
    stop("'x' must be a character vector")
  if(!is.character(categ))
    stop("'categ' must be a character vector")
  if(length(x)!=length(categ))
    stop("length of 'x' and 'categ' must be the same")

  categAnc = augmentByAncestorsSmart(categ)

  gocats = sort(unique(unlist(categAnc)))
  genes  = sort(unique(unlist(x)))

  res = do.call(if(sparse) Matrix::Matrix else base::matrix,
    list(FALSE, nrow=length(gocats), ncol=length(genes)))

  ## res = matrix(FALSE, nrow=length(gocats), ncol=length(genes))

  rownames(res) = gocats
  colnames(res) = genes
  for(j in seq(along=x))
    res[categAnc[[j]], x[j]] = TRUE

  return(res)
}

setMethod("geneIds", "ChrBandTree",
          function(object, n, ...) {
              stopifnot(length(n) == 1L)
              nodeData(object@toChildGraph, n=n, attr="geneIds")[[1L]]
          })

setMethod("lgeneIds", "ChrBandTree",
          function(r, n, ...) {
              ans <- nodeData(r@toChildGraph, n=n, attr="geneIds")
              ans[sapply(ans, function(x) !is.na(x[1]))]
          })

setMethod("childrenOf", signature=signature(g="ChrBandTree", n="character"),
          function(g, n, ...) {
              edges(g@toChildGraph)[n]
          })

setMethod("parentOf", signature=signature(g="ChrBandTree", n="character"),
          function(g, n, ...) {
              edges(g@toParentGraph)[n]
          })

setMethod("allGeneIds", "ChrBandTree",
          function(g, ...) {
              unique(unlist(nodeData(g@toChildGraph,
                                     attr="geneIds")))
          })

setMethod("treeLevels", "ChrBandTree",
          function(g, ...) {
              as.integer(names(g@level2nodes))
          })

setMethod("level2nodes", c("ChrBandTree", "numeric"),
          function(g, level, ...) {
              level2nodes(g, as.character(level), ...)
          })

setMethod("level2nodes", c("ChrBandTree", "character"),
          function(g, level, ...) {
              ans <- g@level2nodes[[level, exact=TRUE]]
              if (is.null(ans))
                stop("no such level: ", as.character(level))
              ans
          })

exampleLevels <- function(g)
{
    sapply(g@level2nodes, function(x) x[1])
}

make_level2node_map <- function(g)
{
    # Given a graph representing a tree with the root node given by
    # nodes(g)[1], return a list mapping levels of the tree to nodes.
    paths <- dijkstra.sp(g)
    split(names(paths[["distances"]]), paths[["distances"]])
}

ChrBandTreeFromGraph <- function(g) {
    root <- nodes(g)[1]
    gToParent <- reverseEdgeDirections(g)
    l2n <- make_level2node_map(g)
    new("ChrBandTree", toParentGraph=gToParent,
        toChildGraph=g, root=root, level2nodes=l2n)
}

NewChrBandTree <- function(chip, univ) {
    g <- makeChrBandGraph(chip, univ)
    ChrBandTreeFromGraph(g)
}

setMethod("initialize", "HyperGParams",
          function(.Object, ...)
{
    .Object <- callNextMethod()
    .Object@datPkg <- 
        configureDatPkg(annotation(.Object), .Object)
    makeValidParams(.Object)
})

library("Category")
library("org.Sc.sgd.db")

test_KEGG1 <- function() {
    set.seed(434)    
    allYeast <- ls(org.Sc.sgdCHR)
    selGenes <- sample(allYeast, 80)
    kp <- new("KEGGHyperGParams",
              geneIds=selGenes,
              annotation="org.Sc.sgd")
    ans <- hyperGTest(kp)
    checkEquals("KEGG", testName(ans))
}

library("hgu95av2.db")
## Set to FALSE to see warning/error messages when running tests
## that verify that errors/warnings are raised.
quiet <- TRUE

.setUp <- function() {
    options(warn=2)
}

.tearDown <- function() {
    options(warn=0)
}

testValid <- function() {
    univ <- 1:100
    sel <- sample(univ, 10)
    p1 <- new("KEGGHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation="hgu95av2",
                       pvalueCutoff=0.05,
                       testDirection="over")

    p2 <- new("GOHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation="hgu95av2",
                       pvalueCutoff=0.05,
                       testDirection="over")
    
    ## verify that our fixups work as expected
    options(warn=0)
    p3 <- new("GOHyperGParams",
              geneIds=c(sel, sel, 9000L),
              universeGeneIds=c(univ, univ),
              annotation="hgu95av2",
              pvalueCutoff=0.05,
              testDirection="over")
    checkEquals(sel, geneIds(p3))
    checkEquals(univ, universeGeneIds(p3))
}

testDupGeneIds <- function() {
    univ <- 1:100
    sel <- as.integer(c(1, 1, 1, 2, 3, 4, 5))
    checkException(new("KEGGHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation="hgu95av2",
                       pvalueCutoff=0.05,
                       testDirection="over"),
                   silent=quiet)

    checkException(new("GOHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation="hgu95av2",
                       pvalueCutoff=0.05,
                       testDirection="over"),
                   silent=quiet)

}

testDupUniverseGeneIds <- function() {
    univ <- rep(1:100, 2)
    sel <- 1:10
    checkException(new("KEGGHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation="hgu95av2",
                       pvalueCutoff=0.05,
                       testDirection="over"),
                   silent=quiet)

    checkException(new("GOHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation="hgu95av2",
                       pvalueCutoff=0.05,
                       testDirection="over"),
                   silent=quiet)
}

testGeneIdsMatchUniverseGeneIds <- function() {
    univ <- 1:100
    sel <- as.character(1:10)
    checkException(new("KEGGHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation="hgu95av2",
                       pvalueCutoff=0.05,
                       testDirection="over"),
                   silent=quiet)

    checkException(new("GOHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation="hgu95av2",
                       pvalueCutoff=0.05,
                       testDirection="over"),
                   silent=quiet)
}

testBadPvalueCutoff <- function() {
    univ <- as.character(1:100)
    sel <- as.character(1:10)
    checkException(new("KEGGHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation="hgu95av2",
                       pvalueCutoff=5,
                       testDirection="over"),
                   silent=quiet)

    checkException(new("GOHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation="hgu95av2",
                       pvalueCutoff=-.05,
                       testDirection="over"),
                   silent=quiet)
}

library("hgu95av2.db")
library("org.Sc.sgd.db")
library("GOstats")
##library("test_basic_regression_hgu95av2")

makeSimpleGOHyperGParams <- function(chip=c("hgu95av2", "org.Sc.sgd")) {
    set.seed(344)
    chip <- match.arg(chip)
    if (chip == "hgu95av2") {
        probeIds <- ls(hgu95av2ENTREZID)
    } else if (chip == "org.Sc.sgd") {
        probeIds <- ls(org.Sc.sgdCHR)
    }
    randProbeIds <- sample(probeIds, 500)
    if (chip == "org.Sc.sgd") {
        entrezUniverse <- randProbeIds
    } else {
        entrezUniverse <- mget(randProbeIds, hgu95av2ENTREZID,
                               ifnotfound=NA)
    }
    entrezUniverse <- entrezUniverse[!is.na(entrezUniverse)]
    selectedEntrezIds <- sample(entrezUniverse, 30)
    params <- new("GOHyperGParams",
                  geneIds=selectedEntrezIds, 
                  universeGeneIds=entrezUniverse,
                  annotation=chip, 
                  ontology="BP",
                  pvalueCutoff=0.05,
                  conditional=FALSE,
                  testDirection="over")
    params
}
    

test_basic_regression_hgu95av2 <- function() {
    p <- makeSimpleGOHyperGParams(chip="hgu95av2")
    res <- hyperGTest(p)

    checkEquals("hgu95av2", annotation(res))
    checkEquals(c("GO", "BP"), testName(res))
}


test_basic_regression_YEAST <- function() {
    p <- makeSimpleGOHyperGParams(chip="org.Sc.sgd")
    res <- hyperGTest(p)

    checkEquals("org.Sc.sgd", annotation(res))
    checkEquals(c("GO", "BP"), testName(res))
}

## Set to FALSE to see warning/error messages when running tests
## that verify that errors/warnings are raised.
quiet <- TRUE

.setUp <- function() {
    options(warn=2)
}

.tearDown <- function() {
    options(warn=0)
}

testValidDb <- function() {
    annodb <- AnnotationDbi::loadDb(system.file("UnitTests","test.sqlite",
                                                package = "Category", mustWork = TRUE))
    univ <- keys(annodb)
    sel <- sample(univ, 10)
    p1 <- new("KEGGHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation=annodb,
                       pvalueCutoff=0.05,
                       testDirection="over")

    p2 <- new("GOHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation=annodb,
                       pvalueCutoff=0.05,
                       testDirection="over")
    
    ## verify that our fixups work as expected
    options(warn=0)
    p3 <- new("GOHyperGParams",
              geneIds=c(sel, sel, 9000L),
              universeGeneIds=c(univ, univ),
              annotation=annodb,
              pvalueCutoff=0.05,
              testDirection="over")
    checkEquals(sel, geneIds(p3))
    checkEquals(univ, universeGeneIds(p3))
    dbFileDisconnect(dbconn(annodb))
}

testDupGeneIdsDb <- function() {
    annodb <- AnnotationDbi::loadDb(system.file("UnitTests","test.sqlite",
                                                package = "Category", mustWork = TRUE))
    univ <- keys(annodb)
    sel <- rep(sample(univ, 4), c(4,3,2,1))
    checkException(new("KEGGHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation=annodb,
                       pvalueCutoff=0.05,
                       testDirection="over"),
                   silent=quiet)

    checkException(new("GOHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation=annodb,
                       pvalueCutoff=0.05,
                       testDirection="over"),
                   silent=quiet)
    dbFileDisconnect(dbconn(annodb))
}

testDupUniverseGeneIdsDb <- function() {
    annodb <- AnnotationDbi::loadDb(system.file("UnitTests","test.sqlite",
                                                package = "Category", mustWork = TRUE))
    univ <- rep(keys(annodb), 2)
    sel <- unique(sample(univ, 5))
    checkException(new("KEGGHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation=annodb,
                       pvalueCutoff=0.05,
                       testDirection="over"),
                   silent=quiet)

    checkException(new("GOHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation=annodb,
                       pvalueCutoff=0.05,
                       testDirection="over"),
                   silent=quiet)
    dbFileDisconnect(dbconn(annodb))
}

testGeneIdsMatchUniverseGeneIdsDb <- function() {
    annodb <- AnnotationDbi::loadDb(system.file("UnitTests","test.sqlite",
                                                package = "Category", mustWork = TRUE))
    univ <- keys(annodb)
    sel <- as.character(1:10)
    checkException(new("KEGGHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation=annodb,
                       pvalueCutoff=0.05,
                       testDirection="over"),
                   silent=quiet)

    checkException(new("GOHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation=annodb,
                       pvalueCutoff=0.05,
                       testDirection="over"),
                   silent=quiet)
    dbFileDisconnect(dbconn(annodb))
}

testBadPvalueCutoffDb <- function() {
    annodb <- AnnotationDbi::loadDb(system.file("UnitTests","test.sqlite",
                                                package = "Category", mustWork = TRUE))
    univ <- keys(annodb)
    sel <- sample(univ, 5)
    checkException(new("KEGGHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation=annodb,
                       pvalueCutoff=5,
                       testDirection="over"),
                   silent=quiet)

    checkException(new("GOHyperGParams",
                       geneIds=sel,
                       universeGeneIds=univ,
                       annotation=annodb,
                       pvalueCutoff=-.05,
                       testDirection="over"),
                   silent=quiet)
    dbFileDisconnect(dbconn(annodb))
}

#!/bin/bash

RUNIT_DIR=`pwd`
INST_PATH=`dirname $RUNIT_DIR`
PKG_PATH=`dirname $INST_PATH`
PKG=`basename $PKG_PATH`

TEST_FILE=$1

if test $# -ne 1; then
    echo "Usage: runfile.sh SOMETEST.R"
    exit 1
fi

R_CODE="library('RUnit')"
R_CODE="${R_CODE};library('${PKG}')"
R_CODE="${R_CODE};res <- runTestFile('${TEST_FILE}', rngKind='default', rngNormalKind='default')"
R_CODE="${R_CODE};printTextProtocol(res, showDetails=FALSE)"

echo $R_CODE | R --slave

%
% NOTE -- ONLY EDIT THE .Rnw FILE!!!  The .tex file is
% likely to be overwritten.
%
% \VignetteIndexEntry{Using Categories to Analyze Microarray Data}
%\VignetteDepends{Biobase, annotate, hgu95av2.db, KEGGREST, genefilter, ALL,
% RColorBrewer)
%\VignetteSuggests{Rgraphviz}
%\VignetteKeywords{EDA, graphs, ontology, visualization}
%\VignettePackage{Category}
\documentclass[11pt]{article}

\usepackage{times}
\usepackage{hyperref}

\usepackage[authoryear,round]{natbib}
\usepackage{times}
\usepackage{comment}

\textwidth=6.2in
\textheight=8.5in
%\parskip=.3cm
\oddsidemargin=.1in
\evensidemargin=.1in
\headheight=-.3in


\newlength{\smallfigwidth}
\setlength{\smallfigwidth}{6cm}

\newcommand{\scscst}{\scriptscriptstyle}
\newcommand{\scst}{\scriptstyle}

\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textsf{#1}}}
\newcommand{\Rclass}[1]{{\textit{#1}}}

\newcommand{\MAT}[1]{{\bf #1}}
\newcommand{\VEC}[1]{{\bf #1}}

\newcommand{\Amat}{{\MAT{A}}}

%%notationally this is going to break
\newcommand{\Emat}{{\MAT{E}}}
\newcommand{\Xmat}{{\MAT{X}}}
\newcommand{\Xvec}{{\VEC{X}}}
\newcommand{\xvec}{{\VEC{x}}}


\newcommand{\Zvec}{{\VEC{Z}}}
\newcommand{\zvec}{{\VEC{z}}}

\newcommand{\calG}{\mbox{${\cal G}$}}

\bibliographystyle{plainnat}

\title{Using Categories to Model Genomic Data}
\author{R. Gentleman}

\begin{document}

\maketitle

\section*{Introduction}

The analysis of genomic data is an important challenge in
bioinformatics. The particular aspect of that problem that we will
address is the analysis of gene expression data in conjunction with
category data. Category data, are data that map from specific entities
(genes in this case) to categories, or classes. In some cases the
mapping will be a partition, so that each gene maps to one and only
one category, but in most cases genes will map to multiple categories.
One important aspect of category data is that the categories and the
mappings from entities to categories are known \textit{a priori} and
are not determined from the experimental data.

There are very many biological examples of category data, and to some
extent they may be approached using similar statistical methods.
Examples include the mapping from genes to chromosomes (which is
nearly a partition), the mappings from genes to pathways, the mappings
from genes to GO \citep{GO} classifications, or the mappings from
genes to protein complexes. The categories themselves may have complex
relationships, as is the case with GO and protein complexes, but for
now we concentrate on the mappings between genes and categories.

To make some of the concepts concrete and to provide extensive, but
related, examples we will make use of a microarray data set
\citep{Chiaretti2004}. The data come from a clinical trial in acute
lymphoblastic leukemia (ALL). We will focus our attention on the
patients with B-cell derived ALL, and in particular on comparing the group
with BCR/ABL (9;22 translocation) to those with no observed
cytogenetic abnormalities.

Category analysis is similar to the approach taken in
\cite{Mootha2003}. However, category analysis can be viewed as an
extension of that methodology that makes its application both simpler
and richer. Among the important concepts is the notion that the search
for sets of differentially expressed genes is not always the right
approach for analyzing gene expression data. Given that a microarray
typically measures the levels of coordinated gene expression averaged
over a few thousands of cells it seems that a more holistic approach
is sometimes warranted. We are often more interested in categories
where the constituent genes show coordinated changes in expression
over the experimental conditions than in sets of differentially
expressed genes. The methods presented in this paper are one approach
to taking a more global view.

<<Setup, echo=TRUE, results=hide>>=
library("Biobase")
library("annotate")
library("hgu95av2.db")
library("KEGGREST")
library("genefilter")
library("Category")
library("ALL")
@

Let's consider the comparison of two different groups, or phenotypes
and we assume that there are some number of DNA microarrays that have
been obtained for each group. The actual method of comparing the
expression levels in the two groups is, in some sense, irrelevant to
the subsequent discussion and readers can easily substitute their own
favorite methods. We refer readers to \cite{WileyEncy} or
\cite{Smyth2004} for a general discussion of some of the issues
involved in gene filtering. Here we will use a $t$-test.

First we subset the ALL data to the two phenotypes that we would like
to compare, those with BCR/ABL and those with no cytogenetic
abnormalities, labeled NEG.

<<Example>>=
## subset of interest: 37+42 samples
data(ALL)
esetA <- ALL[, intersect(grep("^B", as.character(ALL$BT)),
          which(as.character(ALL$mol) %in% c("NEG", "BCR/ABL")))]

esetA@annotation = "hgu95av2"

esetA$mol.biol = factor(esetA$mol.biol)


esetASub = nsFilter(esetA, var.cutoff=0.5)$eset

##set up some colors

BCRcols = ifelse(esetASub$mol == "BCR/ABL","goldenrod", "skyblue")
library("RColorBrewer")
cols = brewer.pal(10, "RdBu")

@


\subsection*{Category Analysis}

We have a set of data where there have been $G$ measurements on each
of $n$ samples. We use $\Emat$ to denote the $G$ by $n$ data
matrix. We consider the case where a univariate test statistic can be
computed for each entity (gene in our case) and denote the resulting
$G$ vector by $\xvec$.

There is a given fixed set of categories, $\mathcal{C}$, and a set of
entities (genes) $\mathcal{G}$ from which we can compute the incidence
matrix $\Amat$, where $a[i,j] =1$ if entity $j$ is in category
$i$. The question of interest is the identification of the elements of
$\zvec = \Amat \xvec$ that are unusually large or small.

\subsection*{Implementation}

Consider the two-sample problem. Assume that there are $n$ microarrays
available, and that they have collected data on $n$ samples under two
conditions. Suppose that we have chosen to use a test statistics,
$T$. There are several different methods of generating values of
$\xvec_T$ under the null hypothesis that there is no difference
between the two conditions. These include permuting the sample labels,
carrying out a bootstrap simulation, or using any one of a number of
other methods for generating a reference distribution. Once this
reference distribution, $\{\xvec^b\}_{b=1}^B$ has been computed, it
induces a distribution on $\zvec$, where $\zvec^b = \Amat
\xvec^b$. Hence, for each $z_i$ we can compute marginal tests of
whether that particular $z_i$ is extreme relative to the joint
distribution.

\subsubsection*{Parametric Assumptions}

Suppose that $\Xvec$ is multivariate $N(\mu, \Sigma)$.  The statistics
are computed as $\Zvec = \Amat \Xvec$, and hence $\Zvec$ also follows
a multivariate Normal distribution with mean $\Amat \mu$ and variance
$\Amat \Sigma \Amat^\prime$. But if $\Sigma$ is unknown then it is not
possible to carry out inference on $\Zvec$. For our situation $\Sigma$
is too large to easily be estimated.

We note that if $\Xvec$ is made up of two sample $t$-statistics with
$n$ reasonably large then the elements of $\Xvec$ are approximately
$N(0,1)$ random variables. If the genes were independent $\Zvec$
divided by the square root of the row sums of $\Amat$ will itself be
approximately multivariate Normal with mean zero and $\Sigma$ will be
the $m$ by $m$ identity matrix. We use this approximation below.

To carry this out we make use of the \Rfunction{rowttests} function in the
\Rpackage{genefilter} package.

<<parametric1>>=

ttests = rowttests(esetASub, "mol.biol")

##find the probes that we are going to use
fL = findLargest(featureNames(esetASub), abs(ttests$statistic), "hgu95av2")
fL2 = probes2Path(fL, "hgu95av2")
length(fL2)
inBoth = fL %in% names(fL2)
fL = fL[inBoth]

@

In the computation to get \Robject{fL2} we first found all duplicate
mappings to LocusLink identifiers and then selected the one with the
largest $t$--statistic. Note that we passed the absolute value of the
observed $t$--statistic in and so obtain the most extreme value.
Then we remove all LocusLink identifiers that do not map to any known
pathway and find that we have \Sexpr{length(fL2)} genes left.

In the next code segment, we first reorder the genes by the \Robject{fl2}
values and then compute $t$-tests, on a per row basis, using 
\Robject{mol.biol} variable in the phenotypic data to define the groups.
<<reorder>>=
eS = esetASub[match(names(fL2), featureNames(esetASub)),]

tobs = rowttests(eS, "mol.biol")
@

Next we create the adjacency matrix that maps genes/probes to
pathways. We compute \Robject{Amat} and rearrange its columns 
to follow the order of genes.

<<Amat>>=
Amat = t(PWAmat("hgu95av2"))
AmER = Amat[,names(fL)]
@ 

We will make the decision that we are not interested in
pathways that have fewer than 5 members that are in the experimentally
observed genes. In your own analysis, you will need to adapt this choice.

<<rs>>=
rs = rowSums(AmER)
AmER2 = AmER[rs>5,]
rs2   = rs[rs>5]
nCats = length(rs2)
@

There are \Sexpr{nCats} pathways (categories) that will be used for
the analysis.

<<computeCatStats>>=
##compute observed stats
 tA = AmER2 %*% tobs$statistic
 tA = tA/sqrt(rs2)
 names(tA) = row.names(AmER2)

@

And now we can examine the resultant qq-plot, which is shown in
Figure~\ref{fig:qq}.

\begin{figure}[tp]
  \centering
<<qqplot, fig=TRUE, include=FALSE, width=5, height=5, echo=TRUE, results=hide>>=
 qqnorm(tA)
@
\includegraphics[width=1.25\smallfigwidth]{Category-qqplot}
\caption{\label{fig:qq}%
A qq-plot where each point represents a different category (in this
  case a pathway). There is one pathway with a remarkably low value. }
\end{figure}

In Figure~\ref{fig:qq} we see that there is one pathway for which the
aggregate statistic seems to be unusually large, and negative. While
there are a number of pathways that seem to have elevated levels of
activity among those with BCR/ABL, the ribosome pathway stands out
with a large and negative aggregate statistic. We can find that 
pathway and examine the data a bit more. Then in the next section we
make use of the permutation approach to assess significance.

<<findCat>>=

 byTT = names(tA)[ tA< -7]

@

\begin{figure}[tp]
  \centering
<<KEGGmnplot, fig=TRUE, include=FALSE, echo=TRUE, results=hide>>=
    KEGGmnplot(byTT, eS, group=eS$mol, data="hgu95av2",
               main=paste(getPathNames(byTT)[[1]], paste("Overall:",
               round(tA[byTT], 3)), sep="\n"))

@
\includegraphics[width=1.25\smallfigwidth]{Category-KEGGmnplot}
\caption{\label{fig:KEGGmn}
The mean plot for the \Sexpr{getPathNames(byTT)[[1]]} pathway. Each
point represents a gene in the pathway and the $x$-value is determined
by the mean expression in the BCR/ABL group while the $y$-value is
determined by the mean in the NEG group.}
\end{figure}

Figure~\ref{fig:KEGGmn} suggests that the level of activity among genes
involved in the Ribosome pathway seems larger in those labeled NEG
than the samples from patients with BCR/ABL. Perhaps indicating that
ribosomal activity is suppressed in those with BCR/ABL.

We can further investigate the relationship between gene expression
and the Ribosome pathway by examining heatmaps for this
pathway. First, we examine the heatmap for those genes that were
selected by our filtering approach, Figure~\ref{fig:hmRib1}, the colors
in the top bar for Figure~\ref{fig:hmRib1} are gold for BCR/ABL and
blue for NEG, while in Figure~\ref{fig:hmRib2} they are green for
males and grey for females.

<<setupHMcols, results=hide, echo=TRUE>>=
hmcol <- rev(colorRampPalette(brewer.pal(10, "RdBu"))(256))
spcol <- ifelse(eS$mol.biol=="BCR/ABL", "goldenrod", "skyblue")
@

\begin{figure}[tp]
  \centering
<<Ribosomeheatmap, fig=TRUE, include=FALSE, echo=TRUE, results=hide>>=
    tmp1 = KEGG2heatmap(byTT, eS, data="hgu95av2",
               main=paste(getPathNames(byTT)[[1]], paste("Overall:",
               round(tA[byTT], 3)), sep="\n"), col=hmcol,
                 ColSideColors=spcol)

@
\includegraphics[width=1.25\smallfigwidth]{Category-Ribosomeheatmap}
\caption{\label{fig:hmRib1}
The heatmap for the category with the Ribosome pathway, using all gene.}
\end{figure}

Here the patterns of expression do not seem to corroborate the
finding. Yes there are differences in the patient samples with respect
to the expression of mRNA for ribosomal mRNAs, but the differences are
not so striking. So what is going on? Well, one of the genes gives us
a hint - if you examine the heatmap carefully you will notice that one
probe seems to be expressed in one set of samples and not in another -
and the probe is \verb+41214_at+, which corresponds to RPS4Y1, which
is a sex-linked ribosomal protein. If we then redraw the heatmap but
color the sidebar according to sex, (green for males and slate for
females) we see that these probes almost exactly separate the males
and females (and suggest that there may be some errors in the meta-data).

\begin{figure}[tp]
  \centering
<<HMbysex, fig=TRUE, include=FALSE, echo=TRUE, results=hide>>=

  spcol2 <- ifelse(eS$sex=="M", "lightgreen", "slategrey")
    tmp2 = KEGG2heatmap(byTT, eS, data="hgu95av2",
               main=paste(getPathNames(byTT)[[1]], paste("Overall:",
               round(tA[byTT], 3)), sep="\n"), col=hmcol,
                 ColSideColors=spcol2)

@
\includegraphics[width=1.25\smallfigwidth]{Category-HMbysex}
\caption{\label{fig:hmRib2}%
A heatmap of the selected probes for mRNA in the Ribosome pathway. The
color bar is green for males and grey for females.}
\end{figure}

So we have found something that appears to be real, at least
biological, but which may well be uninteresting. Gender is slightly
confounded with the relationship between BCR/ABL and NEG cytogenetics
and hence we will sometimes find effects that may be more correctly
attributed to sex.  To better understand this effect it will be
necessary to modify the testing procedure so that sex can be adjusted
for.  This is reasonably straightforward and involves fitting a per
gene linear model, with both a BCR/ABL effect and a sex effect and
then using the standardized coefficients for the BCR/ABL effect in the
GSEA analysis.

\subsection*{A permutation distribution}

<<setupperms, results=hide, echo=FALSE>>=
NPERM = 500
set.seed(444)
@

If you are uncomfortable with the Normal-theory argument given in the
previous section then it is important to assess the significance of
the observed test statistics with respect to a reference
distribution. To that end, we consider permuting the sample labels
(that is which of the two groups, \verb+BCR/ABL+ or \verb+NEG+, a
patient belongs to). In the code below we consider \Sexpr{NPERM}
permutations.

<<ttperms>>=
v1 = ttperm(exprs(eS), eS$mol.biol, B=NPERM)

permDm <- matrix(0.0, nrow=length(v1$perms[[1]]$statistic),
                 ncol=length(v1$perms))
for (j in 1:ncol(permDm)) {
    permDm[ , j] <- v1$perms[[j]]$statistic
}

permD = AmER2 %*% permDm
##no need to do this second step - if we don't do if for tobs
permD2 = sweep(permD, 1, sqrt(rs2), "/")


pvals = matrix(NA, nr=nCats, ncol=2)
dimnames(pvals) = list(row.names(AmER2), c("Lower", "Upper"))

for(i in 1:nCats) {
    pvals[i,1] = sum(permD2[i,] < tA[i])/NPERM
    pvals[i,2] = sum(permD2[i,] > tA[i])/NPERM
}

ord1 = order(pvals[,1])
lowC = (row.names(pvals)[ord1])[pvals[ord1,1]< 0.05]

highC = row.names(pvals)[pvals[,2] < 0.05]


 getPathNames(lowC)
@

<<setupHC, echo=TRUE, results=hide>>=
lnhC = length(highC)
@

There are \Sexpr{lnhC} pathways that have $p$-values less than $0.05$
where the mean is higher in the BCR/ABL group. We print the

<<getHighNames>>=
 getPathNames(highC)[1:5]

@

Notice that we have used quite a large $p$-value, although our
adjustment should be for the \Sexpr{nCats} categories that we are
testing, and so it will not be too dramatic.  We can visualize the
differences in group means, just as we did before. These are shown in
Figures~\ref{fig:mn1} through \ref{fig:mn3}.

\begin{figure}[tp]
  \centering
<<mnplot1, fig=TRUE, width=5, height=5, include=FALSE, echo=TRUE, results=hide>>=
    KEGGmnplot(highC[lnhC], eS, group=eS$mol, data="hgu95av2",
               main=paste(getPathNames(highC[lnhC])[[1]], paste("Overall:",
               round(tA[highC[lnhC]], 3)), sep="\n"), pch=16, col="blue")


@
\includegraphics[width=1.25\smallfigwidth]{Category-mnplot1}
\caption{\label{fig:mn1}%
The per category mean plot for pathways that are deemed to be
differentially expressed using a permutation approach.}
\end{figure}



\begin{figure}[tp]
  \centering
<<mnplot2, fig=TRUE, include=FALSE, echo=TRUE, results=hide>>=
    KEGGmnplot(highC[lnhC-1], eS, group=eS$mol, data="hgu95av2",
               main=paste(getPathNames(highC[lnhC-1])[[1]], paste("Overall:",
               round(tA[highC[lnhC-1]], 3)), sep="\n"))


@
\includegraphics[width=1.25\smallfigwidth]{Category-mnplot2}
\caption{\label{fig:mn2}%
The per category mnplot for pathways that are deemed to be
differentially expressed using a permutation approach.}
\end{figure}

\begin{figure}[tp]
  \centering
<<mnplot3, fig=TRUE, include=FALSE, echo=TRUE, results=hide>>=
    KEGGmnplot(highC[lnhC-2], eS, group=eS$mol, data="hgu95av2",
               main=paste(getPathNames(highC[lnhC-2])[[1]], paste("Overall:",
               round(tA[highC[lnhC-2]], 3)), sep="\n"))


@
\includegraphics[width=1.25\smallfigwidth]{Category-mnplot3}
\caption{\label{fig:mn3}%
The per category mean plot pathway that are deemed to be
differentially expressed using a permutation approach.}
\end{figure}

Unfortunately, as we can see from the visualizations none of these
category plots are especially compelling. If we return to the category
\Sexpr{getPathNames(byTT)[[1]]} then we see that the permutation
$p$-value is \Sexpr{pvals[byTT, 2]}.

%%FIXME: this one is a heatmap so I needed to move it

\begin{figure}[tp]
  \centering
<<HMperm, fig=TRUE, include=FALSE, echo=TRUE, results=hide>>=
    tmp2 = KEGG2heatmap(highC[lnhC-2], eS, data="hgu95av2",
               main=paste(getPathNames(highC[lnhC-2])[[1]], paste("Overall:",
               round(tA[highC[lnhC-2]], 3)), sep="\n"), col=hmcol,
                 ColSideColors=spcol)

@
\includegraphics[width=1.25\smallfigwidth]{Category-HMperm}
\caption{\label{fig:HMperm}%
}
\end{figure}


\section*{Using other functions}

In the examples above we used, perhaps the simplest form of per
category statistic, the summation. Extending the model to deal with
virtually any other per-category function is quite simple and support
for doing this is available with the (very simple) 
\Rfunction{applyByCategory} function.

<<appByCat>>=

med = applyByCategory(tobs$statistic, AmER2, FUN=median)
wt  = applyByCategory(tobs$statistic, AmER2, 
  FUN = function(x) with(wilcox.test(x), c(statistic, p=p.value)))

head(t(wt[,order(wt[2,])]))
     
@

\bibliography{cat}

\end{document}

%
% NOTE -- ONLY EDIT THE .Rnw FILE!!!  The .tex file is
% likely to be overwritten.
%
%\VignetteIndexEntry{Using Chromosome Bands as Categories}
%\VignetteDepends{ALL, hgu95av2.db, annotate, genefilter, SNPchip,
% geneplotter, limma, lattice, graph}
%\VignetteKeywords{graphs, visualization}
%\VignettePackage{Category}
\documentclass[11pt]{article}

\usepackage{times}
\usepackage{hyperref}

\usepackage[authoryear,round]{natbib}
\usepackage{times}
\usepackage{comment}

\textwidth=6.2in
\textheight=8.5in
%\parskip=.3cm
\oddsidemargin=.1in
\evensidemargin=.1in
\headheight=-.3in

\newcommand{\Rclass}[1]{\textit{``#1''}}
\newcommand{\Rpackage}[1]{{\textsf{#1}}}
\newcommand{\Rfunction}[1]{{\texttt{#1()}}}


\usepackage{Sweave}
\SweaveOpts{keep.source=TRUE,eps=FALSE,pdf=TRUE}

\bibliographystyle{plainnat}

\title{Using Categories defined by Chromosome Bands}
\author{D. Sarkar, S. Falcon, R. Gentleman}
\date{August 7, 2008}

\begin{document}

\maketitle

% \section{Introduction}

% Categories or gene sets (mappings of genes to classes) are becoming an
% important tool for the analysis of microarray data.  The
% \Rpackage{Category} package enables category based analysis
% approaches; this vignette demonstrates tools that allow the use of
% categories derived from chromosome bands.  Although in normal
% multi-cellular eukaryotic organisms there is generally not a strong
% relationship between chromosomal location and expression (notable
% exceptions being the HOX family of genes), there are a number of
% diseases where either genetic (amplification or deletion of DNA) or
% epigenetic (methylation or demethylation) events will result in
% changes in gene expression that can be associated with particular
% chromosomal locations.  Our goal is to identify genomic regions with
% systematic alterations in gene expression levels.  

% We consider the use
% of chromosome bands as gene sets to address this question.


% how microarray gene expression
% data can be analyzed using  as gene sets, and how the
% hierarchical nature of that information can be used in systematic
% statistical testing.

% This vignette describes tools in the
% \Rpackage{Category} package that work with gene sets derived from
% chromosomal location.



\section{Chromosome bands}

Typically, in higher organisms, each chromosome has a centromere and
two arms.  The short arm is called the p arm and the longer arm the q
arm.  Chromosome bands (see Figure~\ref{fig:chr12ideogram}) are
identified by differential staining, usually with Giemsa-based stains,
and many disease-related defects have been mapped to these bands; such
mappings have played an important role in classical cytogenetics.
With the availability of complete sequences for several genomes, there
have been efforts to link these bands with specific sequence locations
\citep{FureyHaussler}.  The estimated location of the bands in the
reference genomes can be obtained from the UCSC genome browser, and
data linking genes to particular bands can be obtained from a variety
of sources such as the NCBI.  This vignette demonstrates tools that
allow the use of categories derived from chromosome bands, that is,
the relevant categories are determined \textit{a priori} by a mapping
of genes to chromosome bands.

Figure~\ref{fig:chr12ideogram} shows an ideogram of human chromosome
12, with the band 12q21 shaded.  As shown in the figure, 12q21 can be
divided into more granular levels 12q21.1, 12q21.2, and 12q21.3.
12q21.3 can itself be divided at an even finer level of resolution
into 12q21.31, 12q21.32, and 12q21.33.  Moving towards less granular
bands, 12q21 is a part of 12q2 which is again a part of 12q.  We take
advantage of this nested structure of the bands in our analysis.
<<setup1>>=
library("Category")
library("ALL")
library("hgu95av2.db")
library("annotate")
library("genefilter")
##library("SNPchip")
library("karyoploteR")
library("geneplotter")
library("limma")
library("lattice")
library("graph")
@ 




%
\begin{figure}[tb]
\begin{center}
%
<<chr12ideogram, fig=TRUE, include=FALSE, prefix=FALSE, echo=FALSE, width=9, height=3>>=
build <- "hg19" ## or hg18
kp <- plotKaryotype(genome=build, chromosomes="chr12", main="Human chromosome 12")
## cyt2 <- getCytoband(build=build)
## cyt2$gieStain <- "foo"
## c12p12_idx <- intersect(grep("^q21", cyt2$name),
##                         which(cyt2$chrom == "12"))
## cyt2[c12p12_idx, "gieStain"] <- rep(c("gpos50", "gpos75"),
##                                     length=length(c12p12_idx))
##
##
## plotCytoband2(chromosome="12", build=build, cytoband=cyt2, outer=FALSE,  ## From SNPchip
##     cex.axis=0.6, main="Human chromosome 12")
## plotCytoband2(chromosome="12", build=build, cytoband=cyt2, outer=FALSE,
##               cex.axis=0.6,
##               main="Human chromosome 12")
@ 
\end{center}
\includegraphics[width=.98\textwidth]{chr12ideogram}
\caption{Ideogram for human chromosome 12.  The p arm is on the left,
  the q arm is on the right, and the centromere is indicated by a
  notch.  The shaded bands together represent 12q21.  This band is
  composed of three sub-bands: 12q21.1, 12q21.2, and 12q21.3.  The
  last of these is composed of sub-sub-bands 12q21.31, 12q21.32, and
  12q21.33. }
\label{fig:chr12ideogram}
\end{figure}
%


\section{Data Preparation}

For illustration, we use a microarray dataset \citep{Chiaretti2005}
from a clinical trial in acute lymphoblastic leukemia (ALL).  The data
are described in Chapter~2 of \cite{biocCaseStudies}.  For the
analysis presented here, we consider the comparison of two subsets:
patients identified as having a BCR/ABL gene fusion present, typically
as a result of a translocation of chromosomes 9 and 22 (labeled
BCR/ABL), and those that have no observed cytogenetic abnormalities
(labeled NEG).  The full dataset is available in the \Rpackage{ALL}
package, and the relevant subset of the data can be obtained by
%
<<bcrAblOrNegSubset, cache=TRUE>>=
data(ALL, package="ALL")

subsetType <- "BCR/ABL"
Bcell <- grep("^B", as.character(ALL$BT))
bcrAblOrNegIdx <- which(as.character(ALL$mol.biol) %in% c("NEG", subsetType))

bcrAblOrNeg <- ALL[, intersect(Bcell, bcrAblOrNegIdx)]
bcrAblOrNeg$mol.biol <- factor(bcrAblOrNeg$mol.biol)
@ 
%
We also create relevant annotation maps to go from feature names to
Entrez ID, gene symbol, and chromosome band.
%
<<annMaps, cache=FALSE>>=
annType <- c("db", "env")
entrezMap <- getAnnMap("ENTREZID", annotation(bcrAblOrNeg),
                       type=annType, load=TRUE)
symbolMap <- getAnnMap("SYMBOL", annotation(bcrAblOrNeg),
                       type=annType, load=TRUE)
bandMap <- getAnnMap("MAP", annotation(bcrAblOrNeg),
                     type=annType, load=TRUE)
@ 
%


We applied a non-specific filter to the dataset to remove probesets
lacking the desired annotation as well as those with an interquartile
range (IQR) below the median IQR, as probesets with little variation
across samples are uninformative.  We also ensured that each Entrez
Gene identifier maps to exactly one probeset by selecting the probeset
with the largest IQR when two or more probesets map to the same Entrez
Gene ID.
%
<<nsFiltering, cache=TRUE>>=
filterAns <- nsFilter(bcrAblOrNeg,
                      require.entrez = TRUE, 
                      remove.dupEntrez = TRUE, 
                      var.func = IQR, var.cutoff = 0.5)
nsFiltered <- filterAns$eset
@ 
%
We also remove probesets with no gene symbol, as well as those with no
mapping to a chromosome band.
%
<<>>=
hasSYM <- sapply(mget(featureNames(nsFiltered), symbolMap, ifnotfound=NA),
                 function(x) length(x) > 0 && !is.na(x[1]))
hasMAP <- sapply(mget(featureNames(nsFiltered), bandMap, ifnotfound=NA),
                 function(x) length(x) > 0 && !is.na(x[1]))
nsFiltered <- nsFiltered[hasSYM & hasMAP, ]
@ 
%
We define the \textit{gene universe} to be the subset of genes that
remain after this filtering.
%
<<defineGeneUniverse, cache=TRUE>>=
affyUniverse <- featureNames(nsFiltered)
entrezUniverse <- unlist(mget(affyUniverse, entrezMap))
names(affyUniverse) <- entrezUniverse
if (any(duplicated(entrezUniverse)))
    stop("error in gene universe: can't have duplicate Entrez Gene Ids")
@ 




We assessed differential expression between the \Sexpr{subsetType} and
NEG groups using an empirical Bayes approach, as implemented in the
software package \Rpackage{limma}~\citep{limma}, yielding an
attenuated $t$-statistic for each gene.
%
<<parametric1, cache=TRUE>>=
design <- model.matrix(~ 0 + nsFiltered$mol.biol)
colnames(design) <- c("BCR/ABL", "NEG")
contr <- c(1, -1) ## NOTE: we thus have BCR/ABL w.r.t NEG
fm1 <- lmFit(nsFiltered, design)
fm2 <- contrasts.fit(fm1, contr)
fm3 <- eBayes(fm2)
ttestLimma <- topTable(fm3, number = nrow(fm3), adjust.method = "none")
ttestLimma <- ttestLimma[featureNames(nsFiltered), ]

tstats <- ttestLimma$t
names(tstats) <- entrezUniverse[rownames(ttestLimma)]
##
@
%
We used a $p$-value cutoff of $0.01$ to identify a list of potentially
differentially expressed genes.
<<selectedSubset,cache=TRUE>>=
ttestCutoff <- 0.01
smPV  <- ttestLimma$P.Value < ttestCutoff
pvalFiltered <- nsFiltered[smPV, ]
selectedEntrezIds <- unlist(mget(featureNames(pvalFiltered), entrezMap))
##
@ 




% \section{Types of tests}

% % We describe the use of marginal and conditional Hypergeometric tests
% % to identify unusual chromosome bands, and a contingency table approach
% % to perform local tests.  We also describe an implementation of GSEA
% % that we use to perform marginal and conditional tests.

% We explore three approaches for using the chromosome band gene sets in
% our example analysis: Hypergeometric testing, a generic contingency
% table approach that allows for testing multiple gene sets
% simultaneously, and gene set enrichment analysis (GSEA).  

% Hypergeometric and contingency table testing rely on a list of
% selected genes, created by artificially dividing genes into two
% classes (differentially expressed or not).  The GSEA approach does not
% rely on the identification of differentially expressed genes, but
% rather on computed per-gene summary statistics.  

% To motivate these analytical approaches, we observe that there are two
% important features of gene sets based on chromosome bands:
% (1) the bands are nested hierarchically, and (2) they almost form a
% partition (for most species almost all genes appear in only one
% location in the genome).

% The first observation is important because it allows us to perform
% conditional tests.  When testing a particular sub-band, we are
% interested in knowing whether genes annotated at that sub-band are
% unusual relative to some larger set of genes.  If we have already
% answered that question in the affirmative for one or more
% sub-sub-bands, then to declare the band under consideration unusual,
% we may desire evidence for that assertion beyond that furnished by the
% unusual sub-sub-bands---such tests are conditional tests.
% \citet{Falcon-GOstats} address this question in the context of
% Hypergeometric tests for over-representation of GO terms ~\citep{GO}.


% The second observation suggests that it is possible to improve upon
% methods that test each band separately.  One can either treat each
% chromosome as a unit, or, since it is unlikely that any important
% event (amplification, deletion, methylation) spans the centromere in
% humans, divide the data into 48 groups (one for each half of each
% chromosome).  Then, within each group of gene sets, one can determine
% the number of top-level chromosome bands, $K_c$, for each chromosome
% arm $c$ and perform a standard 2 by $K_c$ contingency table
% analysis. This approach reduces the issues that arise due to multiple
% testing.  If one of these tests is rejected, other contingency tables
% constructed from the sub-band annotations can help identify the
% location(s) most responsible for the rejection.





\section{Methods}



There are two important features of gene sets based on chromosome
bands: (1) the bands are nested hierarchically, and (2) they almost
form a partition (for most species almost all genes appear in only one
location in the genome).  This naturally leads to two different
dichotomies in approaches to testing: one is a top-down versus a
bottom-up approach, and the other contrasts local tests with global
tests.  We use human chromosome 12 as an example to describe these
approaches.  Users will need to identify which of the approaches best
align with their objectives and then make use of the software
appropriately.

Conceptually one can start a sequential testing approach either at the
most coarse level of organization (probably the level of the arm: p or
q), or the most specific level (that of sub-sub-bands). In the
top-down approach, one first tests the hypothesis of interest on a
coarse level of organization, and if rejected, the next level of
organization is considered.  For example, we might first consider the
band 12q21, and if that hypothesis is rejected, test each of the
sub-bands 12q21.1, 12q21.2, and 12q21.3. If the hypothesis for 12q21.3
is rejected, then we may subsequently examine each of 12q21.31,
12q21.32, and 12q21.33.

The bottom-up approach differs in that we begin at the most granular
level and move upward, amalgamating adjacent bands at each level.  The
bottom-up approach is easier to put into a conditional hypothesis
testing framework.  Our initial null hypotheses involve the smallest,
or most granular bands, and if there is evidence that these are
unusual (i.e., we reject the null hypothesis) then moving to a larger,
or less granular, band requires additional information to declare it
significant, over and above what we have used to identify the smaller
band.  In our example, we would first test 12q21.31, 12q21.32, and
12q21.33, and then move up and test 12q21.3.  If one or more of the
three sub-bands had been declared significant, we would exclude the
evidence from genes annotated in those sub-bands when testing the
coarser band.

It is important to note that the top-down versus bottom-up approaches
represent a fundamental trade-off between false positive and false
negative errors.  The bottom-up approach necessarily involves
performing a larger number of tests, yielding a correspondingly larger
absolute number of false positives for a given false positive rate at
which each individual test is controlled.  The top-down approach cuts
down on the number of false positives by starting with fewer top-level
tests, and performing further tests at sublevels only when a top-level
test is rejected.  The disadvantage to this approach is loss of power
to detect real departures that are localized to a sub-level, a
phenomenon commonly illustrated using Simpson's paradox
\citep[see, e.g.,][]{Simpson1982}.

Whether a test is local or global is a different question, orthogonal
to that of top-down or bottom-up.  There are two distinct but
potentially relevant questions that may be of interest.  The first is
whether genes in a particular gene set are ``different'' relative to
all other genes under consideration.  For a Hypergeometric test, this
question may be formalized as whether the proportion of interesting
genes in 12q21 is different from the proportion of interesting genes
in the rest of the genome, or equivalently, whether membership in
12q21 is independent of being selected.  Such tests are
\textit{global} in the sense that all genes in the gene universe are
used to determine whether or not genes at a location are unusual or
not.  An alternative is to ask whether genes in a genomic location are
different relative to other genes in some meaningfully defined
neighbourhood.  Such a test can be performed simply by restricting the
gene universe to a suitable subset; for example, when testing 12q21,
we may only consider genes in 12q.  A more natural approach is to use
a $2 \times 3$ contingency table to test the hypothesis that the
proportion of interesting genes is the same in 12q21, 12q22, and
12q23.  Both these tests are \textit{local} in the sense that only
nearby genes are used.  

Contingency table tests are inherently local and although they do not
naturally extend to conditional testing, we can use a top-down
approach to test at various resolutions.  Such tests can be performed
by the \Rfunction{cb\_contingency} function, which we do not discuss in
this vignette.  Instead, we focus on the bottom-up approach, which
allows for conditional testing.


% \section{Analysis}

% We begin by building a graph representing the tree structure of the
% chromosome band annotations for the genes in the gene universe, as
% such a representation is natural and convenient.  The chromosome band
% annotation for each gene in the universe is added to the tree along
% with all parent bands up to the chromosome level.  For example, if one
% of the genes is annotated at 12q21.1, we add the following nodes to
% the tree if they are not already present: 12q21, 12q2, 12q, and 12.
% As a consequence of the nesting structure of the bands, each parent
% band inherits the gene annotations of its children.  This situation is
% similar to a graph of GO terms where the \textit{is-a} relationship
% implies inheritance of gene annotations by less specific terms from
% more specific ones.

\section{Utility functions}

We first define a few utility functions that we subsequently use in
presentation.  The \Rfunction{chrSortOrder} function reorders rows of
data frame for display in a natural order.
<<>>=

chrSortOrder <- function(df) {
    chrs <- sub("([^pq]+).*$", "\\1", rownames(df))
    xyIdx <- chrs %in% c("X", "Y")
    xydf <- NULL
    if (any(xyIdx)) {
        chrs <- chrs[!xyIdx]
        xydf <- df[xyIdx, ]
        df <- df[!xyIdx, ]
    }
    ord <- order(as.integer(chrs), rownames(df))
    df <- df[ord, ]
    if (!is.null(xydf))
      df <- rbind(df, xydf)
    df
}
@ 
%
The \Rfunction{gseaTstatStripplot} function creates a comparative
strip plot of the $t$-statistics for specified bands.
<<>>=

gseaTstatStripplot <- function(bands, g, ..., include.all = FALSE)
{
    chroms <- c(1:22, "X", "Y")
    chromArms <- c(paste(chroms, "p", sep=""), paste(chroms, "q", sep=""))
    egid <- lapply(nodeData(g, bands), "[[", "geneIds")
    if (include.all) {
        egid$All <- 
            unique(unlist(lapply(nodeData(g)[chromArms], "[[", "geneIds")))
    }
    tdf <- do.call(make.groups, lapply(egid, function(x) tstats[x]))
    stripplot(which ~ data, tdf, jitter = TRUE, ...)
}


@ 

The \Rfunction{esetBWPlot} function creates box-and-whisker plots for
every gene in an \Rclass{ExpressionSet}.
<<>>=

esetBWPlot <- function(tmpSet, ..., layout=c(1, nrow(emat)))
{
    emat <- exprs(tmpSet)
    pd <- pData(tmpSet)
    probes <- rownames(emat)
    syms <- 
        sapply(mget(probes, hgu95av2SYMBOL, ifnotfound=NA),
               function(x) if (all(is.na(x))) "NA" else as.character(x)[1])
    selectedAffy <- 
        probes %in% affyUniverse[selectedEntrezIds]
    symsSelected <- syms[selectedAffy]
    symsWithStatus <- 
        paste(syms, 
              ifelse(selectedAffy, "*", ""), 
              sep = "")
    pdat <- 
        cbind(exprs=as.vector(emat),
              genes=factor(probes, levels = probes, labels = syms),
              pd[rep(seq_len(nrow(pd)), each=nrow(emat)), ])
    pdat <- transform(pdat, genes = reorder(genes, exprs))
    panels.to.shade <- levels(pdat$genes) %in% symsSelected
    bwplot(mol.biol ~ exprs | genes, data=pdat, 
           layout = layout,
           auto.key=TRUE,
           scales=list(x=list(log=2L)),
           xlab="Log2 Expression",
           panels.to.shade = panels.to.shade,
           panel = function(..., panels.to.shade) {
               if (panels.to.shade[packet.number()]) 
                   panel.fill(col = "lightgrey")
               panel.bwplot(...)
           },
           strip=FALSE,
           strip.left=TRUE, ...)
}

g1 <- makeChrBandGraph(annotation(nsFiltered), univ=entrezUniverse)
ct <- ChrBandTreeFromGraph(g1)

subsetByBand <- function(eset, ct, band) {
    egIDs <- unlist(nodeData(ct@toChildGraph, n=band, 
                             attr="geneIds"), use.names=FALSE)
    wantedProbes <- affyUniverse[as.character(egIDs)]
    eset[intersect(wantedProbes, featureNames(eset)), ]
}

@ 

\section{Hypergeometric Testing}

We use a method similar to that described in~\citet{Falcon-GOstats} to
conditionally test for over-representation of chromosome bands in the
selected gene list.  A test is set up by creating a suitable object of
class \Rclass{ChrMapHyperGParams}.  We first create an object to
perform a standard Hypergeometric analysis, treating each chromosome
band independently, and then modify a copy to represent a conditional
Hypergeometric computation.
%
%
<<basicParams, cache=TRUE>>=
params <- new("ChrMapHyperGParams",
              conditional=FALSE,
              testDirection="over",
              universeGeneIds=entrezUniverse,
              geneIds=selectedEntrezIds,
              annotation="hgu95av2",
              pvalueCutoff=0.05)

paramsCond <- params
paramsCond@conditional <- TRUE
@ 
%
The test computations are performed by
%
<<basicTest, cache=TRUE>>=
hgans <- hyperGTest(params)
hgansCond <- hyperGTest(paramsCond)
@ 
%
The results can be summarized by
% 
<<result1, cache=TRUE>>=
sumUn <- summary(hgans, categorySize=1)
chrSortOrder(sumUn)

sumCond <- summary(hgansCond, categorySize=1)
chrSortOrder(sumCond)
@ 
%


For the standard test, the structure of the chromosome band graph is
ignored and a Hypergeometric test is applied to each band
independently.  For the conditional test, the hierarchical
relationship among the bands as represented by the graph is used in
the computation.  The highest-resolution bands (those with no children
in the graph) are tested first.  Testing proceeds with the bands whose
children (sub-bands) have already been tested.  For these bands, the
gene annotations that are inherited from significant child nodes
(children with $p$-value smaller than the specified cutoff) are
removed prior to testing to yield a conditional test.

The effect of the conditional test is illustrated by examining the
results for 14q and its sub-bands.  In the standard test, we see that
14q22 and 14q22.2 both have a significant $p$-value.  In the
conditional test, only 14q22.2 remains.  The conclusion is that there
is not enough additional evidence beyond that provided by 14q22.2 to
mark 14q22 as significant.







\section{GSEA using linear models}



GSEA is a popular method that can be used to assess whether or not
particular gene sets are associated with a phenotype of interest
\citep{Subramanian2005, Tian2005, JiangGent2007}.  Most applications
of this method do not explicitly deal with structure of the gene sets,
but when analyzing chromosomal location such methods are desirable.
We present a simple approach that is similar in spirit to traditional
GSEA, and generalizes nicely to accommodate nested categories.
Consider the situation where gene $i$ has an associated measure of
differential expression $y_i$; for example, an attenuated
$t$-statistic derived from a differential expression analysis such as
\Rpackage{limma}~\citep{limma}.  Given a particular category, GSEA
asks whether the distribution of $y_i$-s restricted to the category of
interest is ``unusual''.  Thus, in Figure~\ref{fig:gseaStripplot}, we
might be interested in knowing whether the distribution of $y_i$
values for genes in 12q21 is different from that of the genes in 12q
(for a local test) or of all genes in the gene universe (for a global
test).  Figure ~\ref{fig:gseaStripplot} is produced by
<<>>=
gseaTstatStripplot(c("12q21.1", "12q21", "12q2", "12q"),
                   include.all = TRUE, 
                   g = g1,
                   xlab = "Per-gene t-statistics", 
                   panel = function(...) {
                       require(grid, quietly = TRUE)
                       grid.rect(y = unit(2, "native"), 
                                 height = unit(1, "native"),
                                 gp = 
                                 gpar(fill = "lightgrey", 
                                      col = "transparent"))
                       panel.grid(v = -1, h = 0)
                       panel.stripplot(...)
                       panel.average(..., fun = mean, lwd = 3)
                   })
@
<<gseaStripplot, fig=TRUE, include=FALSE, prefix=FALSE, echo=FALSE, width=9, height=4>>=
plot(trellis.last.object())
@ 
\begin{figure}[tb]
\begin{center}
\includegraphics[width=.98\textwidth]{gseaStripplot}
\end{center}
\caption{ Per-gene $t$-statistics, as computed by \Rpackage{limma},
  for selected chromosome bands.  The points are jittered vertically
  to alleviate overlap.  A thick grey line joins the mean value within
  each group.  A GSEA test would compare the genes in 12q21 with those
  in 12q, or the entire gene universe, by fitting a linear model with
  per-gene $t$-statistics as the response, and a term indicating
  membership in 12q21 as the predictor.  If 12q21.1 is declared as
  significant, a conditional GSEA test would include an additional
  term for 12q21.1. }
\label{fig:gseaStripplot}
\end{figure}





We fit a factorial model to see whether the distribution of $y_i$ is
associated with category membership.  Specifically, for category $j$,
we fit the model
\begin{equation}
  \label{eq:lm1}
  y_i = \beta_0 + \beta_1 a_{ij} + \varepsilon_{i}
\end{equation}
where $a_{ij} = 1$ if gene $i$ belongs to category $j$, and $0$
otherwise.  The index $i$ may range over the full gene universe, or a
subset, depending on whether one wishes to perform global or local
tests.  The null hypothesis of no association is represented by $H_0:
\beta_1 = 0$.  The model nominally assumes that the $y_i$-s are
Normally distributed with equal variance, but in practice the results
are robust against mild deviations.  The presence of an intercept term
allows nonzero overall mean, which can be important in many
situations, especially for local tests.  We expect the test to be
fairly insensitive to distributional assumptions on the $y_i$-s.


We can fit (\ref{eq:lm1}) by least squares and test $H_0: \beta_1 = 0$
to obtain a marginal test for each category $j$; in this case, each
chromosome band.  The procedure also generalizes to incorporate the
nesting structure of chromosome bands.  Specifically, if band $j_2$
(e.g., 12q21.1) is nested within a coarser band $j_1$ (e.g., 12q21)
and the more granular band $j_2$ is significant, then the effect of
membership in $j_1$ over and above the effect attributable to
membership in $j_2$ can be tested by fitting the model
\begin{equation}
  \label{eq:lm2}
  y_i = \beta_0 + \beta_1 a_{ij_1} + \beta_2 a_{ij_2} + \varepsilon_{i}
\end{equation}
and testing the null hypothesis $H_0: \beta_1 = 0$.  Multiple
significant sub-bands and multiple levels of nesting can be
incorporated by including further terms in the model.  The complete
process can be summarized as follows: Start by marginally testing each
band which has no sub-bands.  For all other bands, first test all
sub-bands, then test the current band using a linear model that
includes a term for each \emph{significant} sub-band.

We apply this procedure to perform global tests using per-gene
$t$-statistics as a measure of differential expression in BCR/ABL
relative to NEG samples.  As with the Hypergeometric tests, we start
by creating objects of class \Rclass{ChrMapLinearMParams}.
%
<<LMtestSetup,cache=TRUE>>=
params <- new("ChrMapLinearMParams",
              conditional = FALSE,
              testDirection = "up",
              universeGeneIds = entrezUniverse,
              geneStats = tstats,
              annotation = "hgu95av2",
              pvalueCutoff = 0.01, 
              minSize = 4L)
params@graph <- makeChrBandGraph(params@annotation, params@universeGeneIds)
params@gsc <- makeChrBandGSC(params@graph)
paramsCond <- params
paramsCond@conditional <- TRUE
@ 
The tests are performed, and the results summarized by
<<LMtest>>=

lmans <- linearMTest(params)
lmansCond <- linearMTest(paramsCond)

chrSortOrder(summary(lmans))
chrSortOrder(summary(lmansCond))

##
@ 
%%
These examples only test for consistently upregulated categories;
similar calls with \texttt{testDirection = "down"} can be used to test
for downregulation.  As we see, the GSEA approach picks out many more
bands as significant, but there is some concordance with the
Hypergeometric approach.  For example, 7q31, 8p22, and 14q22.2 come up
in both analyses.  Figure~\ref{fig:dotplot1p362} shows box-and-whisker
plots of genes in one category (1p36.2) that is declared as
significant by the Hypergeometric test, but not by GSEA.  It is
produced by
<<>>=
tmpSet <- subsetByBand(nsFiltered, ct, "1p36.2")
esetBWPlot(tmpSet, ylab="1p36.2", layout = c(2, 8),
           par.strip.text = list(cex = 0.8))
@ 
<<dotplot1p362,fig=TRUE,include=FALSE,echo=FALSE,prefix=FALSE,width=9,height=6>>=
plot(trellis.last.object())
@ 
%
\begin{figure}[tb]
\begin{center}
  \includegraphics[width=.98\textwidth]{dotplot1p362}
\end{center}
\caption{Expression values for the BCR/ABL and NEG samples for the
  genes located within 1p36.2, which is declared as significant by the
  Hypergeometric test, but not by GSEA.  Genes in the selected list
  are highlighted with a shaded background.  For these genes, The NEG
  samples, those with no known cytogenetic abnormalities, have
  significantly lower expression than the BCR/ABL samples.  However,
  the direction is reversed (albeit mildly) for many of the other
  genes.  }

\label{fig:dotplot1p362}
\end{figure}


\bibliography{cat}


\end{document}

\name{ChrMapHyperGParams-class}
\docType{class}
\alias{ChrMapHyperGParams-class}

\alias{conditional,ChrMapHyperGParams-method}
\alias{conditional<-,ChrMapHyperGParams,logical-method}

\title{Class "ChrMapHyperGParams"}
\description{
  This class encapsulates parameters needed for Hypergeometric testing
  of over or under representation of chromosome bands among a selected
  gene list using \code{\link{hyperGTest}}.
}
\section{Objects from the Class}{
  Objects can be created by calls of the form
  \code{new("ChrMapHyperGParams", ...)}.
}
\section{Slots}{
  \describe{

    \item{\code{chrGraph}:}{Object of class \code{"graph"}.  The nodes
      are the chromosome bands and the edges describe the tree structure
      of the bands.  Each node has a "geneIds" node attributes (see
      \code{nodeData}) which contains a vector of gene IDs annotated at
      the given band.}

    \item{\code{conditional}:}{Object of class \code{"logical"},
      indicating whether the test performed should be a conditional
      test.  }

    \item{\code{geneIds}:}{Object of class \code{"ANY"}: A vector of
      gene identifiers.  Numeric and character vectors are probably the
      only things that make sense.  These are the gene ids for the
      selected gene set.}

    \item{\code{universeGeneIds}:}{Object of class \code{"ANY"}: A
      vector of gene ids in the same format as \code{geneIds} defining a
      subset of the gene ids on the chip that will be used as the
      universe for the hypergeometric calculation.  If this is
      \code{NULL} or has length zero, then all gene ids on the chip will
      be used.}

    \item{\code{annotation}:}{A string giving the name of the
      annotation data package for the chip used to generate the data.}

    \item{\code{categorySubsetIds}:}{Object of class \code{"ANY"}:
      If the test method supports it, can be used to specify a subset of
      category ids to include in the test instead of all possible
      category ids.}

    \item{\code{categoryName}:}{A string describing the category.
      Usually set automatically by subclasses.  For example
      \code{"GO"}.}

    \item{\code{pvalueCutoff}:}{The p-value to use as a cutoff for
      significance for testing methods that require it.  This value will
      also be passed on to the result instance and used for display and
      counting of significant results.  The default is 0.01.}

    \item{\code{testDirection}:}{A string indicating whether the test
      should be for overrepresentation (\code{"over"}) or
      underrepresentation (\code{"under"}).}

    \item{\code{datPkg}:}{Object of class \code{"DatPkg"} used to assist
      with dispatch based on type of annotation data available. }

  }
}
\section{Extends}{
Class \code{"\linkS4class{HyperGParams}"}, directly.
}
\section{Methods}{
No methods defined with class "ChrMapHyperGParams" in the signature.
}

\author{Seth Falcon}

\examples{
showClass("ChrMapHyperGParams")
}
\keyword{classes}

\name{categoryToEntrezBuilder}
\docType{methods}

\alias{categoryToEntrezBuilder}
\alias{categoryToEntrezBuilder,GOHyperGParams-method}
\alias{categoryToEntrezBuilder,KEGGHyperGParams-method}
\alias{categoryToEntrezBuilder,PFAMHyperGParams-method}
\alias{categoryToEntrezBuilder,OBOHyperGParams-method}


\title{Return a list mapping category ids to Entrez Gene ids}
\description{
  Return a list mapping category ids to the Entrez Gene ids annotated at
  the category id.  Only those category ids that have at least one
  annotation in the set of Entrez Gene ids specified by the
  \code{geneIds} slot of \code{p} are included.
}
\usage{
categoryToEntrezBuilder(p)
}

\arguments{
  \item{p}{A subclass of \code{HyperGParams-class}}
}

\details{
  End users \bold{should not} call this directly.  This method gets
  called from \code{hyperGTest}.  To add support for a new
  category, a new method for this generic must be defined.  Its
  signature should match a subclass of
  \code{HyperGParams-class} appropriate for the new
  category.
}
\value{
  A list mapping category ids to Entrez Gene identifiers.
}

\author{S. Falcon}
\seealso{
  \code{\link{hyperGTest}}
  \code{\link{HyperGParams-class}}
}

\keyword{manip}

\name{effectSize}
\alias{effectSize}
%% move to a LinearMResult-accessors.Rd?
\title{ Extract estimated effect sizes }
\description{
  This function extracts estimated effect sizes from the results of a
  linear model-based gene-set / category enrichment test. 
}
\usage{
effectSize(r)
}
\arguments{
  \item{r}{ The results of the test }
}
\value{
  A numeric vector.
}
\author{ Deepayan Sarkar }
\seealso{ \code{linkS4class{LinearMResult}} }

\name{MAPAmat}

\alias{MAPAmat}
\alias{makeChrBandInciMat}
\alias{makeChrBandGSC}

\title{Mapping chromosome bands to genes}

\description{
  These functions return a mapping of chromosome bands to genes.
  \code{makeChrBandGSC} returns a
  \code{\link[GSEABase:GeneSetCollection-class]{GeneSetCollection}} object,
  with a \code{GeneSet} for each band. The other functions return a 0/1
  incidence matrix with a row for each chromosme band and a column for
  each gene.  Only those chromosome bands with at least one gene
  annotation will be included.
}

\usage{
MAPAmat(chip, univ = NULL, minCount = 0)
makeChrBandInciMat(chrGraph)
makeChrBandGSC(chrGraph)
}

\arguments{
  \item{chip}{A string giving the annotation source.  For example,
    \code{"hgu133plus2"}}

  \item{univ}{A vector of gene IDs (these should be Entrez IDs for
    most annotation sources).  The the annotations will be limited to
    those in the set specified by \code{univ}.  If \code{univ} is
    \code{NULL} (default), then the gene IDs are those found in the
    annotation data source.}

  \item{chrGraph}{A \code{graph} object as returned by
    \code{makeChrBandGraph}}

  \item{minCount}{ Bands with less than \code{minCount} genes will be
    excluded from the returned matrix.  If \code{minCount} is \code{0},
    no bands will be removed, this is the default.}
}


\value{

  For \code{makeChrBandGSC}, a \code{GeneSetCollection} object with
  a \code{GeneSet} for each band.

  For the other functions, (0/1) incidence matrix with chromosome bands
  as rows and gene IDs as columns.  A \code{1} in \code{m[i, j]}
  indicates that the chromosome band \code{rownames(m)[i]} contains the
  geneID \code{colnames(m)[j]}.

}

\author{Seth Falcon, Michael Lawrence}

\seealso{
  \code{\link{makeChrBandGraph}},
  \code{\link{cateGOry}},
  \code{\link{probes2MAP}}
}

\examples{
have_hgu95av2.db <- suppressWarnings(require("hgu95av2.db"))
if (have_hgu95av2.db)
   mam <- MAPAmat("hgu95av2.db")
}

\keyword{manip}

\name{cb_test}
\alias{cb_test}

\title{Chromosome Band Tree-Based Hypothesis Testing}
\description{
  \code{cb_test} is a flexible tool for discovering interesting
  chromosome bands relative to a selected gene list.  The function
  supports local and global tests which can be carried out in a top down
  or bottom up fashion on the tree of chromosome bands.
}
\usage{
cb_test(selids, chrtree, level, dir = c("up", "down"),
       type = c("local", "global"), next.pval = 0.05,
       cond.pval = 0.05, conditional = FALSE)
}

\arguments{
  
  \item{selids}{A vector of gene IDs.  The IDs should match those used
    to annotatate the \code{ChrBandTree} given by \code{chrtree}.  In most
    cases, these will be Entrez Gene IDs.}
  
  \item{chrtree}{A \code{ChrBandTree} object representing the chromosome
  bands and the mapping to gene identifiers.  The genes in the
  \code{ChrBandTree} are limited to the universe of gene IDs specified
  at object creation time.}

\item{level}{An integer giving the level of the chromosome band tree
  at which testing should begin.  The level is conceptualized as the set
  of nodes with a given path length to the root (organism) node of the
  chromosome band tree.  So level 1 is the chromosome and level 2 is the
  chromosome arms.  You can get a better sense by calling
  \code{exampleLevels(chrtree)}}

\item{dir}{A string giving the direction in which the chromosome band
  tree will be traversed when carrying out the tests.  A bottom up
  traversal, from leaves to root, is specified by \code{"up"}.  A top
  down, from root to leaves, traversal is specified by \code{"down"}.}

  \item{type}{A string giving the type of test to perform.  The current
    choices are \code{"local"} and \code{"global"}.  A local test
    carries out a chisq.test on each 2 x K contingency table induced by
    each set of siblings at a given level in the tree.  A global test
    uses the Hypergeometric distribution to compute a p-value for the 2
    x 2 tables induced by each band treated independently.}

  \item{next.pval}{The p-value cutoff used to determine whether the parents
    or children of a node should be tested.  After testing a given level
    of the tree, the decision of whether or not to continue testing the
    children (or parents) of the already tested nodes is made by
    comparing the p-value result for a given node with this cutoff;
    relatives of nodes with values strictly greater than the cutoff are
    skipped.}

  \item{cond.pval}{The p-value cutoff used to determine whether a node
    is significant during a conditional test.  See \code{conditional}.}

  \item{conditional}{A logical value.  Can only be used when
    \code{dir="up"} and \code{type="global"}.  In this case, a
    \code{TRUE} value causes a conditional Hypergeometric calculation to
    be performed.  The genes annotated at significant children of a
    given band are removed before testing.}
}

\value{
  A list with an element for each level of the tree that was tested.
  Note that the first element will correspond to the level given by
  \code{level} and that subsequent elements will be the next or previous
  depending on \code{dir}.

  Each level element is itself a list consisting of a result list for
  each node or set of nodes tested.  These inner-most lists will have,
  at least, the following components:

    \item{nodes}{A character vector of the nodes involved in the test.}
    \item{p.value}{The p-value for the test}
    \item{observed}{The contingency table}
    \item{method}{A brief description of the test method}
}

\author{Seth Falcon}

\name{makeValidParams}
\alias{makeValidParams}

\title{Non-standard Generic for Checking Validity of Parameter Objects}
\description{
  This function is not intended for end-users, but may be useful for
  developers extending the Hypergeometric testing capabilities
  provideded by the Category package.

  \code{makeValidParams} is intended to validate a parameter object
  instance (e.g. HyperGParams or subclass).  The idea is that unlike
  \code{validObject}, methods for this generic attempt to fix invalid
  instances when possible, and in this case issuing a warning, and
  only give an error if the object cannot be fixed.
}
\usage{
makeValidParams(object)
}
\arguments{
  \item{object}{A parameter object.  Consult \code{showMethods} to
  see signatures currently supported.}
}
\value{
  The value must have the same class as the \code{object} argument.
}

\author{Seth Falcon}

\keyword{manip}

\name{GOHyperGParams-class}
\docType{class}
\alias{GOHyperGParams-class}
\alias{show,GOHyperGParams-method}
\alias{categoryName,GOHyperGParams-method}

\alias{annotation,GOHyperGParams-method}

\alias{conditional,GOHyperGParams-method}
\alias{conditional<-,GOHyperGParams,logical-method}

\alias{ontology,GOHyperGParams-method}
\alias{ontology<-,GOHyperGParams,character-method}



\title{Class "GOHyperGParams"}

\description{
  A parameter class for representing all parameters needed for running
  the \code{hyperGTest} method with one of the GO
  ontologies (BP, CC, MF) as the category.
}

\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("GOHyperGParams", ...)}.
}

\section{Slots}{
  \describe{
    \item{\code{ontology}:}{A string specifying the GO ontology to use.
      Must be one of "BP", "CC", or "MF".}
    \item{\code{conditional}:}{A logical indicating whether the
      calculation should condition on the GO structure.}
    \item{\code{geneIds}:}{Object of class \code{"ANY"}: A vector of
      gene identifiers.  Numeric and character vectors are probably the
      only things that make sense.  These are the gene ids for the
      selected gene set.}
    \item{\code{universeGeneIds}:}{Object of class \code{"ANY"}: A
      vector of gene ids in the same format as \code{geneIds} defining a
      subset of the gene ids on the chip that will be used as the
      universe for the hypergeometric calculation.  If this is
      \code{NULL} or has length zero, then all gene ids on the chip will
      be used.}
    \item{\code{annotation}:}{A string giving the name of the
      annotation data package for the chip used to generate the data.}
    \item{\code{categorySubsetIds}:}{Object of class \code{"ANY"}:
      If the test method supports it, can be used to specify a subset of
      category ids to include in the test instead of all possible
      category ids.}
    \item{\code{categoryName}:}{A string describing the category.
      Usually set automatically by subclasses.  For example "GO".}
    \item{\code{datPkg}:}{Holds a DatPkg object which is of a
    particular type that in turn varies with the kind of annotation
    package this is.}
    \item{\code{pvalueCutoff}:}{A numeric values between zero and one
      used as a p-value cutoff for p-values generated by the
      Hypergeometric test. When the test being performed is
      non-conditional, this is only used as a default value for
      printing and summarizing the results. For a conditional
      analysis, the cutoff is used during the computation to determine
      perform the conditioning: child terms with a p-value less than
      pvalueCutoff are conditioned out of the test for their parent
      term.}
    \item{\code{orCutoff}:}{A numeric value used as an odds-ratio
      cutoff for odds ratios generated by the conditional
      Hypergeometric test. For such a test, it works like the
      \code{pvalueCutoff} but applied on the odds ratio. It has no
      effect when \code{conditional=FALSE}. }
    \item{\code{minSizeCutoff}:}{A numeric value used as a cutoff for
      minimum size of the gene sets being tested with the conditional
      Hypergeometric test. For such a test, it works like the
      \code{pvalueCutoff} but applied on the odds ratio. It has no
      effect when \code{conditional=FALSE}. }
    \item{\code{maxSizeCutoff}:}{A numeric value used as a cutoff for
      maximum size of the gene sets being tested with the conditional
      Hypergeometric test. For such a test, it works like the
      \code{pvalueCutoff} but applied on the odds ratio. It has no
      effect when \code{conditional=FALSE}. }
    \item{\code{testDirection}:}{A string which can be either "over"
      or "under". This determines whether the test performed detects
      over or under represented GO terms.}
  }
}
\section{Extends}{
Class \code{"HyperGParams"}, directly.
}
\section{Methods}{
  \describe{
    \item{\code{hyperGTest(p)}}{Perform hypergeometric tests to
      assess overrepresentation of category ids in the gene set.  See
      the documentation for the generic function for details.  This
      method must be called with a proper subclass of
      \code{HyperGParams}.}

    \item{\code{ontology(p)}, \code{ontology(p) <- value}}{Accessors
        for the GO ontology.  When setting, \code{value} should be one
      of "BP", "CC", or "MF".}

    \item{\code{conditional(p)}, \code{conditional(p) <-
    value}}{Accessors for the conditional flag.  When setting,
  \code{value} must be \code{TRUE} or \code{FALSE}.}

  }
}

\author{S. Falcon}

\seealso{
  \code{\link{HyperGResult-class}}
  \code{\link{GOHyperGParams-class}}
  \code{\link{hyperGTest}}
}

\keyword{classes}

\name{universeBuilder}
\docType{methods}
\alias{universeBuilder}
\alias{universeBuilder,GOHyperGParams-method}
\alias{universeBuilder,KEGGHyperGParams-method}
\alias{universeBuilder,PFAMHyperGParams-method}
\alias{universeBuilder,OBOHyperGParams-method}

\title{Return a vector of gene identifiers with category annotations}
\description{
  Return all gene ids that are annotated at one or more terms in the
  category.  If the \code{universeGeneIds} slot of \code{p} has length
  greater than zero, then the intersection of the gene ids specified in
  that slot and the normal return value is given.
}
\usage{
universeBuilder(p)
}

\arguments{
  \item{p}{A subclass of \code{HyperGParams-class}}
}
\details{
  End users \bold{should not} call this directly.  This method gets
  called from \code{hyperGTest}.  To add support for a new
  category, a new method for this generic must be defined.  Its
  signature should match a subclass of
  \code{HyperGParams-class} appropriate for the new
  category.
}
\value{
  A vector of gene identifiers.
}

\author{S. Falcon}
\seealso{
  \code{\link{hyperGTest}}
  \code{\link{HyperGParams-class}}
}

\keyword{manip}

\name{Category-defunct}
\alias{Category-defunct}
\title{Defunct Functions in Package \pkg{Category}}
%------ PLEASE: put \alias{.} here for  EACH !
\alias{condGeneIdUniverse}
\alias{isConditional}
\alias{geneGoHyperGeoTest}
\alias{geneKeggHyperGeoTest}
\alias{cb_parse_band_hsa}
\alias{chrBandInciMat}
%
\description{
  The functions or variables listed here are no longer part of the Category package.
}
\usage{
condGeneIdUniverse()
isConditional()
geneGoHyperGeoTest()
geneKeggHyperGeoTest()
cb_parse_band_hsa()
chrBandInciMat()
}
\seealso{
  \code{\link{Defunct}}
}

\name{ChrMapHyperGResult-class}
\docType{class}
\alias{ChrMapHyperGResult-class}

\alias{conditional,ChrMapHyperGResult-method}

\title{Class "ChrMapHyperGResult"}
\description{
  This class represents the results of a Hypergeometric test for
  over-representation of genes in a selected gene list in the
  chromosome band annotation.  The \code{hyperGTest} function returns
  an instance of \code{ChrMapHyperGResult} when given a parameter
  object of class \code{ChrMapHyperGParams}.  For details on accessing
  the results, see \link{HyperGResult-accessors}.
}
\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("ChrMapHyperGResult", ...)}.
}
\section{Slots}{
	 \describe{
    \item{\code{pvalue.order}:}{Object of class \code{"integer"} that
    gives the order of the p-values.}
    \item{\code{conditional}:}{Object of class \code{"logical"}
    is a flag indicating whether or not this result is from a
    conditional analysis. }
    \item{\code{chrGraph}:}{Object of class \code{"graph"}.  The nodes
    are the chromosome bands with edges representing the tree
    structure of the bands.  Each node has a \code{"geneIds"}
    attribute that gives the gene IDs annotated at that band.}
    \item{\code{annotation}:}{A string giving the name of the chip
      annotation data package used.}
    \item{\code{geneIds}:}{Object of class \code{"ANY"}: the input
      vector of gene identifiers intersected with the universe of gene
      identifiers used in the computation.  The class of this slot is
      specified as \code{"ANY"} because gene IDs may be integer or
      character vectors depending on the annotation package.}
    \item{\code{testName}:}{A string identifying the testing method
      used.}
    \item{\code{pvalueCutoff}:}{Numeric value used a a p-value
        cutoff.  Used by the \code{show} method to count number of
        significant terms.
      }
    \item{\code{testDirection}:}{Object of class \code{"character"}
    indicating whether the test was for over-representation
    (\code{"over"}) or under-representation (\code{"under"}). }
  }
}
\section{Extends}{
Class \code{"\linkS4class{HyperGResultBase}"}, directly.
}
\section{Methods}{
See \link{HyperGResult-accessors}.
}

\author{Seth Falcon}
\examples{
showClass("ChrMapHyperGResult")
## For details on accessing the results:
##     help("HyperGResult-accessors")
}
\keyword{classes}

\name{KEGGHyperGParams-class}
\docType{class}
\alias{KEGGHyperGParams-class}
\alias{PFAMHyperGParams-class}

\title{Class "KEGGHyperGParams" and "PFAMHyperGParams"}
\description{
  Parameter classes for representing all parameters needed for
  running the \code{hyperGTest} method with KEGG or PFAM as the
  category.
}

\section{Objects from the Class}{
Objects can be created by calls of the form
\code{new("KEGGHyperGParams", ...)} or 
\code{new("PFAMHyperGParams", ...)}.
}
\section{Slots}{
  \describe{
    \item{\code{geneIds}:}{Object of class \code{"ANY"}: A vector of
      gene identifiers.  Numeric and character vectors are probably the
      only things that make sense.  These are the gene ids for the
      selected gene set.}
    \item{\code{universeGeneIds}:}{Object of class \code{"ANY"}: A
      vector of gene ids in the same format as \code{geneIds} defining a
      subset of the gene ids on the chip that will be used as the
      universe for the hypergeometric calculation.  If this is
      \code{NULL} or has length zero, then all gene ids on the chip will
      be used.}
    \item{\code{annotation}:}{A string giving the name of the
      annotation data package for the chip used to generate the data.}
    \item{\code{cateogrySubsetIds}:}{Object of class \code{"ANY"}:
      If the test method supports it, can be used to specify a subset of
      category ids to include in the test instead of all possible
      category ids.}
    \item{\code{categoryName}:}{A string describing the category.
      This will be automatically set to "KEGG" or "PFAM" via the
      class's prototype.}

    \item{\code{pvalueCutoff}:}{The p-value to use as a cutoff for
        significance for testing methods that require it.  This value
        will also be passed on to the result instance and used for
        display and counting of significant results.  The default is 0.01.}

    \item{\code{testDirection}:}{A string indicating whether the
      test should be for overrepresentation (\code{"over"}) or
      underrepresentation (\code{"under"}).}

  }
}
\section{Extends}{
Class \code{"HyperGParams"}, directly.
}
\section{Methods}{
  \describe{
    \item{hyperGTest}{\code{signature(p =
        "HyperGParams")}: Perform hypergeometric tests to
      assess overrepresentation of category ids in the gene set.  See
      the documentation for the generic function for details.  This
      method must be called with a proper subclass of
      \code{HyperGParams}.}
  }
}

\author{S. Falcon}

\seealso{
  \code{\link{HyperGResult-class}}
  \code{\link{GOHyperGParams-class}}
  \code{\link{hyperGTest}}
}

\keyword{classes}

\name{local_test_factory}
\alias{local_test_factory}
\alias{hg_test_factory}

\title{Local and Global Test Function Factories}
\description{
  These functions return functions appropriate for use as the
  \code{tfun} argument to \code{topdown_tree_visitor} or \code{bottomup_tree_visitor}.
  In particular, it is these functions that are associated with the
  "local" and "global" options for the \code{type} argument to
  \code{cb_test}.
}
\usage{
local_test_factory(selids, tableTest = chisq.test)
hg_test_factory(selids, PCUT = 0.05, COND = FALSE, OVER = TRUE)
}

\arguments{
  \item{selids}{A vector of gene IDs.  The IDs should match those used
    to annotatate the \code{ChrBandTree} given by \code{chrtree}.  In most
    cases, these will be Entrez Gene IDs.}
  \item{tableTest}{A contingency table testing function.  The behavior
    of this function must be reasonably close to that of \code{chisq.test}.}
  \item{PCUT}{A p-value cutoff that will be used to determine if a given
    test is significant or not when using \code{hg_test_factory}
    with \code{COND=TRUE}.}
  \item{COND}{A logical value indicating whether a conditional test
  should be performed.}
  \item{OVER}{If \code{TRUE}, test for over representation, if \code{FALSE},
    test for under representation.}
}
\details{
  The returned functions have signature \code{f(start, g, prev_ans)}
  where \code{start} is a vector of start nodes, \code{g} is a
  chromosome band tree graph, and \code{prev_ans} can contain the
  previous result returned by a call to this function.
}
\value{
  A function that can be used as the \code{tfun} argument to the tree
  visitor functions.
}

\author{Seth Falcon}
\seealso{\code{\link{cb_test}}}

\keyword{internal}

\name{hyperg}
\alias{hyperg}
\alias{hyperg,character-method}
\alias{hyperg,list-method}

\title{

  Hypergeometric (gene set enrichment) tests on character vectors.

}

\description{

  This function performs a hypergeometric test for over- or
  under-representation of significant \sQuote{genes} amongst those
  assayed in a universe of genes. It provides an interface based on
  character vectors of identifying member of gene sets and the gene
  universe.

}

\usage{
hyperg(assayed, significant, universe,
    representation = c("over", "under"), ...)
}

\arguments{

  \item{assayed}{A vector of assayed genes (or other
    identifiers). \code{assayed} may be a character vector (defining a
    single gene set) or list of character vectors (defining a collection
    of gene sets).}

  \item{significant}{A vector of assayed genes that were
    differentially expressed. If \code{assayed} is a character vector,
    then \code{significant} must also be a character vector; likewise
    when \code{assayed} is a \code{list}.}

  \item{universe}{A character vector defining the universe of
    genes.}

  \item{representation}{Either \dQuote{over} or \dQuote{under}, to
    indicate testing for over- or under-representation, respectively, of
    differentially expressed genes.}

  \item{\dots}{Additional arguments, unused.}

}

\value{

  When invoked with a character vector of \code{assayed} genes, a named
  numeric vector providing the input values, P-value, odds ratio, and
  expected number of significantly expressed genes.

  When invoked with a list of character vectors of \code{assayed} genes,
  a data frame with columns of input values, P-value, odds ratio, and
  expected number of significantly expressed genes.

}

\author{
  Martin Morgan \url{mtmorgan@fhcrc.org} with contributions from Paul
  Shannon.
}

\seealso{

  \code{\link{hyperGTest}} for convenience functions using Bioconductor
  annotation resources such as GO.db.

}

\examples{
set.seed(123)

## artifical sets -- affy probes grouped by protein family
library(hgu95av2.db)
map <- select(hgu95av2.db, keys(hgu95av2.db), "PFAM")
sets <- Filter(function(x) length(x) >= 10, split(map$PROBEID, map$PFAM))

universe <- unlist(sets, use.names=FALSE)
siggenes <- sample(universe, length(universe) / 20)  ## simulate
sigsets <- Map(function(x, y) x[x \%in\% y], sets, MoreArgs=list(y=siggenes))

result <- hyperg(sets, sigsets, universe)
head(result)
}

\keyword{manip}

\name{LinearMResultBase-class}
\docType{class}
\alias{LinearMResultBase-class}
\alias{annotation,LinearMResultBase-method}
\alias{conditional,LinearMResultBase-method}
\alias{description,LinearMResultBase-method}
\alias{geneIdsByCategory,LinearMResultBase-method}
\alias{geneIds,LinearMResultBase-method}
\alias{geneIdUniverse,LinearMResultBase-method}
\alias{geneMappedCount,LinearMResultBase-method}
\alias{pvalueCutoff,LinearMResultBase-method}
\alias{show,LinearMResultBase-method}
\alias{sigCategories,LinearMResultBase-method}
\alias{summary,LinearMResultBase-method}
\alias{testDirection,LinearMResultBase-method}
\alias{testName,LinearMResultBase-method}
\alias{universeCounts,LinearMResultBase-method}
\alias{universeMappedCount,LinearMResultBase-method}

\title{Class "LinearMResultBase" }
\description{	 
  This VIRTUAL class represents common elements of the return values of
  generic functions like \code{linearMTest}. These elements are
  essentially those that are passed through from the input
  parameters. See \linkS4class{LinearMResult} for a concrete result
  class with the basic outputs.
}

\section{Objects from the Class}{
  A virtual Class: No objects may be created from it.
}

\section{Slots}{
  All of these slots correspond to slots in
  the \linkS4class{LinearMParams} class.
  \describe{
    \item{\code{annotation}:}{Object of class \code{"character"} ~~ }
    \item{\code{geneIds}:}{Object of class \code{"ANY"} ~~ }
    \item{\code{testName}:}{Object of class \code{"character"} ~~ }
    \item{\code{pvalueCutoff}:}{Object of class \code{"numeric"} ~~ }
    \item{\code{minSize}:}{Object of class \code{"integer"} ~~ }
    \item{\code{testDirection}:}{Object of class \code{"character"} ~~ }
    \item{\code{conditional}:}{Object of class \code{"logical"} ~~ }
    \item{\code{graph}:}{Object of class \code{"graph"} ~~ }
    \item{\code{gsc}:}{Object of class \code{"GeneSetCollection"} ~~ }
  }
}
\section{Methods}{
  \describe{
    \item{annotation}{\code{signature(object = "LinearMResultBase")}: ... }
    \item{conditional}{\code{signature(r = "LinearMResultBase")}: ... }
    \item{description}{\code{signature(object = "LinearMResultBase")}: ... }
    \item{geneIdsByCategory}{\code{signature(object = "LinearMResultBase")}: ... }
    \item{geneIds}{\code{signature(object = "LinearMResultBase")}: ... }
    \item{geneIdUniverse}{\code{signature(r = "LinearMResultBase")}: ... }
    \item{geneMappedCount}{\code{signature(r = "LinearMResultBase")}: ... }
    \item{pvalueCutoff}{\code{signature(r = "LinearMResultBase")}: ... }
    \item{show}{\code{signature(object = "LinearMResultBase")}: ... }
    \item{sigCategories}{\code{signature(r = "LinearMResultBase")}: ... }
    \item{summary}{\code{signature(object = "LinearMResultBase")}: ... }
    \item{testDirection}{\code{signature(r = "LinearMResultBase")}:
      ... }
    \item{conditional}{\code{signature(object = "LinearMResultBase")}: ... }
    \item{testName}{\code{signature(r = "LinearMResultBase")}: ... }
    \item{universeCounts}{\code{signature(r = "LinearMResultBase")}: ... }
    \item{universeMappedCount}{\code{signature(r = "LinearMResultBase")}: ... }
  }
}
\author{ Deepayan Sarkar, Michael Lawrence }

\seealso{
  \code{\linkS4class{LinearMResult}},
  \code{\linkS4class{LinearMParams}},
  \code{\link{linearMTest}}
}
\keyword{classes}

\name{HyperGParams-class}
\docType{class}

%% I don't know where these really belong :-(
\alias{conditional}
\alias{conditional<-}
\alias{geneIds<-}
\alias{ontology}
\alias{ontology<-}
\alias{pvalueCutoff<-}
\alias{testDirection<-}
\alias{universeGeneIds}
%% end :-(

\alias{HyperGParams-class}
\alias{initialize,HyperGParams-method}

\alias{geneIds,HyperGParams-method}
\alias{geneIds<-,HyperGParams,logical-method}
\alias{geneIds<-,HyperGParams,ANY-method}

\alias{annotation,HyperGParams-method}
\alias{annotation<-,HyperGParams,character-method}
\alias{conditional,HyperGParams-method}

\alias{ontology,HyperGParams-method}
\alias{pvalueCutoff,HyperGParams-method}
\alias{pvalueCutoff<-,HyperGParams-method}
\alias{testDirection,HyperGParams-method}
\alias{testDirection<-,HyperGParams-method}
\alias{universeGeneIds,HyperGParams-method}

\alias{categoryName}
\alias{categoryName,HyperGParams-method}

\alias{organism,HyperGParams-method}

\alias{makeValidParams,HyperGParams-method}
\alias{show,HyperGParams-method}

\title{Class "HyperGParams"}
\description{
  An abstract (VIRTUAL) parameter class for representing all parameters
  needed by a method specializing the \code{hyperGTest}
  generic.  You should only use subclasses of this class directly.
}
\section{Objects from the Class}{
Objects of this class cannot be instantiated directly.
}
\section{Slots}{
  \describe{
    \item{\code{geneIds}:}{Object of class \code{"ANY"}: A vector of
      gene identifiers.  Numeric and character vectors are probably the
      only things that make sense.  These are the gene ids for the
      selected gene set.}
    \item{\code{universeGeneIds}:}{Object of class \code{"ANY"}: A
      vector of gene ids in the same format as \code{geneIds} defining a
      subset of the gene ids on the chip that will be used as the
      universe for the hypergeometric calculation.  If this is
      \code{NULL} or has length zero, then all gene ids on the chip will
      be used.}
    \item{\code{annotation}:}{Object of class
      \code{"ANY"}. Functionally, this is either a string giving the name of the
      annotation data package for the chip used to generate the data, or
      the name of an annotation object downloaded using AnnotationHub.}
    \item{\code{categorySubsetIds}:}{Object of class \code{"ANY"}:
      If the test method supports it, can be used to specify a subset of
      category ids to include in the test instead of all possible
      category ids.}
    \item{\code{categoryName}:}{A string describing the category.
      Usually set automatically by subclasses.  For example "GO".}
    \item{\code{pvalueCutoff}:}{The p-value to use as a cutoff for
      significance for testing methods that require it.  This value
      will also be passed on to the result instance and used for
      display and counting of significant results.  The default is
      0.01.}
    \item{\code{testDirection}:}{A string indicating whether the test
      should be for overrepresentation (\code{"over"}) or
      underrepresentation (\code{"under"}).}
    \item{\code{datPkg}:}{Holds a DatPkg object which is of a
     particular type that in turn varies with the kind of annotation
     package this is.}

  }
}
\section{Methods}{
  \describe{
  \item{hyperGTest}{\code{signature(p =
        "HyperGParams")}: Perform hypergeometric tests to
      assess overrepresentation of category ids in the gene set.  See the
      documentation for the generic function for details.  This method
      must be called with a proper subclass of
      \code{HyperGParams}.}
    
  \item{\code{geneIds(object)}, \code{geneIds(object) <- value}}{Accessors for
      the gene identifiers that will be used as the selected gene
      list.}
    
  \item{code{annotation(object)}}{Accessor for annotation.  If you want
    to change the annotation for an existing instance, use the
    replacement form.}
    
  \item{\code{ontology(object)}}{Accessor for GO ontology.}

  \item{\code{organism(object)}}{Accessor for the organism character
    string used as an identifier in \code{DatPkg}.}
    
  \item{\code{pvalueCutoff(r)}, \code{pvalueCutoff(r) <-
        value}}{Accessor for the p-value cutoff.  When setting,
      \code{value} should be a numeric value between zero and one.}

  \item{\code{testDirection}}{Accessor for the test direction.  When setting,
        \code{value} must be either "over" or "under".}

    \item{\code{universeGeneIds(r)}}{accessor for vector of gene
        identifiers.}

  }
}

\author{S. Falcon}

\seealso{
  \code{\link{HyperGResult-class}}
  \code{\link{GOHyperGParams-class}}
  \code{\link{KEGGHyperGParams-class}}
  \code{\link{hyperGTest}}
}

\keyword{classes}

\name{applyByCategory}
\alias{applyByCategory}
\title{Apply a function to a vector of statistics, by category }
\description{
  For each category, apply the function \code{FUN} to the set of values
  of \code{stats} belonging to that category.
}

\usage{
applyByCategory(stats, Amat, FUN = mean, ...)
}

\arguments{
  \item{stats}{Numeric vector with test statistics of interest.}
  \item{Amat}{A logical or numeric matrix: the adjacency matrix of the
    bipartite genes - category graph.
    Its rows correspond to the categories, columns
    to the genes, and \code{TRUE} or a numeric value different from \code{0}
    indicates membership.
    The columns are assumed to be aligned with the elements of \code{stats}.}
  \item{FUN}{A function to apply to the subsets \code{stats} by
    categories.}
  \item{\dots}{Extra parameters passed to \code{FUN}.}
}

\details{
  For GO categories, the function \code{\link{cateGOry}} might be useful
  for the construction of \code{Amat}.
}

\value{
  The return value is a list or vector of length equal to
  the number of categories. Each element corresponds to the
  values obtained by applying \code{FUN} to the subset of values
  in \code{stats} according to the category defined for that
  row.
}

\author{R. Gentleman, contributions from W. Huber}


\seealso{\code{\link{apply}} }
\examples{
 set.seed(0xabcd)
 st = rnorm(20)
 names(st) = paste("gene", 1:20)

 a = matrix(sample(c(FALSE, TRUE), 60, replace=TRUE), nrow=3,
        dimnames = list(paste("category", LETTERS[1:3]), names(st)))

 applyByCategory(st, a, median)
}

\keyword{manip}

\name{probes2MAP}
\alias{probes2MAP}
\title{ Map probe IDs to MAP regions. }
\description{
 This function maps probe identifiers to MAP positions using
 the appropriate Bioconductor meta-data package.
}
\usage{
probes2MAP(pids, data = "hgu133plus2")
}
\arguments{
  \item{pids}{ A vector of probe IDs for the chip in use.}
  \item{data}{ The name of the chip, as a character string. }
}
\details{
 Probes are mapped to regions, no checking for duplicate Entrez gene IDs 
 is done.
}
\value{
 A vector, the same length as \code{pids}, with the MAP locations.
}
\author{ R. Gentleman }
\seealso{ \code{\link{probes2Path}} }
\examples{
  set.seed(123)
  library("hgu95av2.db")
  v1 = sample(names(as.list(hgu95av2MAP)), 100)
  pp = probes2MAP(v1, "hgu95av2.db")
}
\keyword{ manip }

\name{hyperGTest}
\docType{methods}

\alias{hyperGTest}
\alias{hyperGTest,HyperGParams-method}
\alias{hyperGTest,ChrMapHyperGParams-method}
\alias{hyperGTest,KEGGHyperGParams-method}
\alias{hyperGTest,PFAMHyperGParams-method}

\title{Hypergeometric Test for association of categories and genes}

\description{
  Given a subclass of \code{HyperGParams}, compute Hypergeomtric
  p-values for over or under-representation of each term in the
  specified category among the specified gene set.
}
\usage{
hyperGTest(p)
}

\arguments{
  \item{p}{An instance of a subclass of
  \code{HyperGParams}.  This parameter object
  determines the category of interest (e.g., GO or KEGG) as well as the
  gene set.}
}
\details{
  The gene identifiers in the \code{geneIds} slot of \code{p} define the
  selected set of genes.  The universe of gene ids is determined by the
  chip annotation found in the \code{annotation} slot of \code{p}.  Both
  the selected genes and the universe are reduced by removing
  identifiers that do not have any annotations in the specified
  category.
  
  For each term in the specified category that has at least one
  annotation in the selected gene set, we determine how many of its
  annotations are in the universe set and how many are in the selected
  set.  With these counts we perform a Hypergeometric test using
  \code{phyper}.  This is equivalent to using Fisher's exact test.
  
  It is important that the correct chip annotation data package be
  identified as it determines the universe of gene identifiers and is
  often used to determine the mapping between the category term and the
  gene identifiers.
  
  For S. cerevisiae if the \code{annotation} slot of \code{p} is set to
  '"org.Sc.sgd"' then comparisons and statistics are computed using common
  names and are with respect to all genes annotated in the S. cerevisiae
  genome not with respect to any microarray chip.  This will *not* be
  the right thing to do if you are working with a yeast microarray.
}
\value{
  A \code{HyperGResult} instance.
}

\section{Implementation Notes}{
  In most cases, the provided method with signature matching any
  subclass of \code{HyperGParams} is all that will be needed.  This
  method follows a template pattern.  To add support for a new FOO
  category type, a developer would need to create a
  \code{FooHyperGParams} subclass and then define two methods
  specialized to the new subclass that get called from inside
  \code{hyperGTest}: \code{universeBuilder} and
  \code{categoryToEntrezBuilder}.
}

\author{S. Falcon}
\seealso{
  \code{\link{HyperGResult-class}}
  \code{\link{HyperGParams-class}}
  \code{\link{GOHyperGParams-class}}
  \code{\link{KEGGHyperGParams-class}}
}
\keyword{htest}

\name{tree_visitor}
\alias{tree_visitor}
\alias{bottomup_tree_visitor}
\alias{topdown_tree_visitor}

\title{Tree Visitor Function}
\description{
  This function visits each node in a tree-like object in an order
  determined by the \code{relationOf} function.  The function given by
  \code{tfun} is called for each set of nodes and the function
  \code{nfun} determines which nodes to test next optionally making use
  of the result of the previous test.
}
\usage{
tree_visitor(g, start, tfun, nfun, relationOf)
topdown_tree_visitor(g, start, tfun, nfun)
bottomup_tree_visitor(g, start, tfun, nfun)
}

\arguments{

  \item{g}{A tree-like object that supports the method given by
  \code{relationOf}.}

  \item{start}{The set of nodes to start the computation (can be a list
    of siblings), but the nodes should all belong to the same level of
    the tree (same path length to root node).}

  \item{tfun}{The test function applied to each list of siblings at each
    level starting with \code{start}.  The signature of \code{tfun}
    should be \code{(start, g, prev_ans)}.}

  \item{nfun}{A function with signature \code{(ans, g)} that processes
    the result of \code{tfun} and returns a character vector of node
    names corresponding to nodes that were involved in an "interesting"
    test.  This is used to determine the next set of nodes to test (see
    details).}

  \item{relationOf}{The method used to traverse the tree.  For example
    \code{childrenOf} or \code{parentOf}.}
}

\details{
  The \code{tree_visitor} function is intended to allow developers to
  quickly prototype different statistical testing paradigms on trees.
  It may be possible to extend this to work for DAGs.

  The visit begins by calling \code{tfun} with the nodes provided by
  \code{start}.  The result of each call to \code{tfun} is stored in an
  environment.  The concept is visitation by tree level and so each
  result is stored using a key representing the level (this isn't quite
  right since the nodes in \code{start} need not be first level, but
  they will be assigned key "1".  After storing the result, \code{nfun}
  is used to obtain a vector of accepted node labels.  The idea is that
  the user should have a way of determining which nodes in the next
  level of the tree are worth testing.  The next \code{start} set is
  determined by \code{start <- relationOf(g, accepted)} where accepted
  is \code{unique(nfun(ans, g))}.
}

\value{
  A list.  See the return value of \code{cb_test} to get an idea.  Each
  element of the list represents a call to \code{tfun} at a given level
  of the tree.
}
\author{Seth Falcon}

%% FIXME: if/when this gets exported, remove the internal keyword
\keyword{internal}

\name{cb_parse_band_Hs}
\alias{cb_parse_band_Hs}


\title{Parse Homo Sapiens Chromosome Band Annotations}
\description{
  This function parses chromosome band annotations as found in the
  <chip>MAP map of Bioconductor annotation data packages.  The return
  value is a vector of parent bands up to the relevant chromosome.
}
\usage{
cb_parse_band_Hs(x)
}

\arguments{
  \item{x}{A chromosome band annotation given as a string.}
}

\value{
  A character vector giving the path to the relevant chromosome.
}
\details{
  The former function cb\_parse\_band\_hsa is now deprecated.
}
\author{Seth Falcon}

\examples{
cb_parse_band_Hs("12q32.12")
}

\name{HyperGResult-accessors}
\alias{HyperGResult-accessors}

\alias{pvalues}
\alias{pvalues,HyperGResult-method}
\alias{pvalues,ChrMapHyperGResult-method}

\alias{geneCounts}
\alias{geneCounts,HyperGResultBase-method}

\alias{universeCounts}
\alias{universeCounts,HyperGResultBase-method}

\alias{universeMappedCount}
\alias{universeMappedCount,HyperGResultBase-method}

\alias{geneMappedCount}

\alias{chrGraph}
\alias{chrGraph,ChrMapHyperGResult-method}

\alias{annotation}

\alias{description}
\alias{description,HyperGResultBase-method}

\alias{annotation,HyperGResultBase-method}

\alias{geneIds}
\alias{geneIds,HyperGResultBase-method}

\alias{geneIdUniverse}
\alias{geneIdUniverse,HyperGResult-method}
\alias{geneIdUniverse,ChrMapHyperGResult-method}


\alias{geneIdsByCategory}
\alias{geneIdsByCategory,HyperGResultBase-method}

\alias{sigCategories}
\alias{sigCategories,HyperGResultBase-method}

\alias{geneMappedCount}
\alias{geneMappedCount,HyperGResultBase-method}

\alias{testName}
\alias{testName,HyperGResultBase-method}

\alias{pvalueCutoff}
\alias{pvalueCutoff,HyperGResultBase-method}
\alias{testDirection}
\alias{testDirection,HyperGResultBase-method}

\alias{organism,HyperGResult-method}

\alias{oddsRatios}
\alias{oddsRatios,HyperGResult-method}
\alias{oddsRatios,ChrMapHyperGResult-method}

\alias{expectedCounts}
\alias{expectedCounts,HyperGResult-method}
\alias{expectedCounts,ChrMapHyperGResult-method}

\alias{summary,HyperGResultBase-method}
\alias{summary,KEGGHyperGResult-method}
\alias{summary,PFAMHyperGResult-method}

\alias{htmlReport}
\alias{htmlReport,HyperGResultBase-method}
\alias{htmlReport,KEGGHyperGResult-method}
\alias{htmlReport,PFAMHyperGResult-method}


\docType{methods}

\title{Accessors for HyperGResult Objects}

\description{
  This manual page documents generic functions for extracting data
  from the result object returned from a call to \code{hyperGTest}.
  The result object will be a subclass of \code{HyperGResultBase}.
  Methods apply to all result object classes unless otherwise noted.
}

\usage{

pvalues(r)
oddsRatios(r)
expectedCounts(r)

geneCounts(r)
universeCounts(r)
universeMappedCount(r)
geneMappedCount(r)

geneIds(object, ...)
geneIdUniverse(r, cond = TRUE)
geneIdsByCategory(r, catids = NULL)
sigCategories(r, p)

## R CMD check doesn't like these
## annotation(r)
## description(r)

testName(r)
pvalueCutoff(r)
testDirection(r)

chrGraph(r)


}

\arguments{
  \item{r, object}{An instance of a subclass of
    \code{HyperGResultBase}.}

  \item{catids}{A character vector of category identifiers.}
  \item{p}{Numeric p-value used as a cutoff for selecting a subset of
    the result.}
  \item{cond}{A logical value indicating whether to return conditional
  results for a conditional test.  The default is \code{TRUE}.  For
  non-conditional results, this argument is ignored.}
  \item{...}{Additional arguments that may be used by specializing methods.}
}

\section{Accessor Methods (Generic Functions)}{
  \describe{

   \item{organism}{returns a \code{"character"} vector describing the
     organism for which the results were calculated.}
   
   \item{geneCounts}{returns an \code{"integer"}
    vector: for each category term tested, the number of genes from
    the gene set that are annotated at the term.}

    \item{pvalues}{returns a \code{"numeric"}
      vector: the ordered p-values for each category term tested.}
    
    \item{universeCounts}{returns an \code{"integer"}
      vector: for each category term tested, the number of genes from
      the gene universe that are annotated at the term.}
    
    \item{universeMappedCount}{returns an \code{"integer"}
      vector of length one giving the size of the gene universe set.}
    
    \item{expectedCounts}{returns a \code{"numeric"} vector
      giving the expected number of genes in the selected gene list to
      be found at each tested category term.  These values may surprise
      you if you forget that your gene list and gene universe might have
      had to undergo further filtering to ensure that each gene has been
      labeled by at least one GO term.}

    \item{oddsRatios}{returns a \code{"numeric"} vector giving
      the odds ratio for each category term tested.}

    \item{annotation}{returns the name
      of the annotation data package used.  }

    \item{geneIds}{returns the input
      vector of gene identifiers intersected with the universe of
      gene identifiers used in the computation. }

    \item{geneIdUniverse}{returns a list named by the
      tested categories.  Each element of the list is a vector of gene
      identifiers (from the gene universe) annotated at the
      corresponding category term.}

    \item{geneIdsByCategory}{returns a list similar to
      \code{geneIdUniverse}, but each vector of gene IDs is
      intersected with the list of selected gene IDs from
      \code{geneIds}.  The result is the selected gene IDs annotated at
      each category.}

    \item{sigCategories}{returns a character vector of category
      identifiers with a significant p-value.  If argument \code{p} is
      missing, then the cutoff obtained from \code{pvalueCutoff(r)} will
      be used.}

    \item{geneMappedCount}{returns the size of
      the selected gene set used in the computation.  This is simply
      \code{length(geneIds(obj))}.}

    \item{pvalueCutoff}{accessor for the
      \code{pvalueCutoff} slot.}

    \item{testDirection}{accessor for the
      \code{testDirection} slot.  Contains a string indicating
      whether the test was for \code{"over"} or \code{"under"}
      representation of the categories.}

    \item{description}{returns a character
      string description of the test result. }

    \item{testName}{returns a string
      describing the testing method used.}

    \item{summary}{returns
      a \code{data.frame} summarizing the test result.  Optional
      arguments \code{pvalue} and \code{categorySize} allow
      specification of maximum p-value and minimum categorySize,
      respectively.}  The data frame contains the \code{GOID},
      \code{Pvalue}, \code{OddsRatio}, \code{ExpCount}, \code{Count},
      and \code{Size}.  \code{ExpCount} is the expected count and
      the \code{Count} is how many instances of that term were actually
      oberved in your gene list while the \code{Size} is the number that
      could have been found in your gene list if every instance had
      turned up.  Values like the \code{ExpCount} and the \code{Size}
      are going to be affected by what is included in the gene universe
      as well as by whether or not it was a conditional test.

    \item{htmlReport}{writes
      an HTML version of the table produced by the \code{summary}
      method.  The first argument should be a \code{HyperGResult}
      instance (or subclass). The path of a file to write the report to
      can be specified using the \code{file} argument.  The default is
      \code{file=""} which will cause the report to be printed to the
      screen.  If you wish to create a single report comprising multiple
      results you can set \code{append=TRUE}.  The default is
      \code{FALSE} (overwrite pre-existing report file).  You can
      specify a string to use as an identifier for each table by
      providing a value for the \code{label} argument.  The number of
      digits displayed in numerical columns can be controlled using
      \code{digits} (defaults to 3).  The \code{summary} method is
      called on the \code{HyperGResult} instance to generate a data
      frame that is transformed to HTML.  You can pass additional
      arguments to the \code{summary} method which is used to generate
      the data frame that is transformed to HTML by specifying a named
      list using \code{summary.args}.}

  }
}


\author{Seth Falcon}
\seealso{
  \code{\link{hyperGTest}}
  \code{\link{HyperGResult-class}}
  \code{\link{HyperGParams-class}}
  \code{\link{GOHyperGParams-class}}
  \code{\link{KEGGHyperGParams-class}}
}
\examples{
## Note that more in-depth examples can be found in the GOstats
## vignette (Hypergeometric tests using GOstats).
library("hgu95av2.db")
library("annotate")

## Retrieve 300 probeids that have PFAM ids
probids <- keys(hgu95av2.db,keytype="PROBEID",column="PFAM")[1:300]

## get unique Entrez Gene IDs
geneids <- select(hgu95av2.db, probids, 'ENTREZID', 'PROBEID')
geneids <- unique(geneids[['ENTREZID']])

## Now do the same for the universe
univ <- keys(hgu95av2.db,keytype="PROBEID",column="PFAM")
univ <- select(hgu95av2.db, univ, 'ENTREZID', 'PROBEID')
univ <- unique(univ[['ENTREZID']])

p <- new("PFAMHyperGParams", geneIds=geneids, universeGeneIds=univ,
         annotation="hgu95av2")
## this takes a while...
if(interactive()){
hypt <- hyperGTest(p)
summary(hypt)
htmlReport(hypt, file="temp.html", summary.args=list("htmlLinks"=TRUE))
}
}
\keyword{htest}

\name{NewChrBandTree}
\alias{NewChrBandTree}
\alias{ChrBandTreeFromGraph}

\title{Create a new ChrBandTree object}
\description{
  \code{NewChrBandTree} and \code{ChrBandTreeFromGraph} provide
  constructors for the \code{ChrBandTree} class.
}
\usage{
NewChrBandTree(chip, univ)
ChrBandTreeFromGraph(g)
}

\arguments{
  \item{chip}{The name of an annotation data package}
  \item{univ}{A vector of gene identifiers that defines the universe of
    genes.  Usually, this will be a vector of Entez Gene IDs.  If
    \code{univ} is \code{NULL}, then all genes probed on the specified
    chip will be in the universe.  We strongly recommend using the set
    of genes that remains after applying a non-specific filter as the
    universe.}
  \item{g}{A \code{graph} instance as returned by \code{makeChrBandGraph}}
}
\value{
  A new \code{ChrBandTree} instance.
}

\author{S. Falcon}
\seealso{\code{\link{ChrBandTree-class}}}

\name{LinearMResult-class}
\docType{class}
\alias{LinearMResult-class}
\alias{effectSize,LinearMResult-method}
\alias{summary,LinearMResult-method}
\alias{pvalues,LinearMResult-method}

\title{Class "LinearMResult" }
\description{

  This class represents the results of a test for systematic change in
  some gene-level statistic by gene sets.  The \code{linearMTest}
  generic function returns an instance of the \code{LinearMResult}
  class. 

}
\section{Objects from the Class}{

  Objects can be created by calls of the form \code{new("LinearMResult",
    ...)}, but is more commonly created using a call to
    \code{\link{linearMTest}}.
  
}
\section{Slots}{
  \describe{
    \item{\code{pvalues}:}{Object of class \code{"numeric"}, with the
      p-values for each term.
    }
    \item{\code{pvalue.order}:}{Object of class \code{"integer"}, the
      order vector (increasing) for the p-values.
    }
    \item{\code{effectSize}:}{Object of class \code{"numeric"}, with
      the effect size for each term.
    }
    \item{\code{annotation}:}{Object of class \code{"character"} ~~ }
    \item{\code{geneIds}:}{Object of class \code{"ANY"} ~~ }
    \item{\code{testName}:}{Object of class \code{"character"} ~~ }
    \item{\code{pvalueCutoff}:}{Object of class \code{"numeric"} ~~ }
    \item{\code{minSize}:}{Object of class \code{"integer"} ~~ }
    \item{\code{testDirection}:}{Object of class \code{"character"} ~~ }
    \item{\code{conditional}:}{Object of class \code{"logical"} ~~ }
    \item{\code{graph}:}{Object of class \code{"graph"} ~~ }
    \item{\code{gsc}:}{Object of class \code{"GeneSetCollection"} ~~ }
  }
}
\section{Extends}{
Class \code{"\linkS4class{LinearMResultBase}"}, directly.
}
\section{Methods}{
  \describe{
    \item{effectSize}{\code{signature(r = "LinearMResult")}: ... }
    \item{pvalues}{\code{signature(r = "LinearMResult")}: ... }
    \item{summary}{\code{signature(r = "LinearMResult")}: returns
      a \code{data.frame} with a row for each gene set tested the
      following columns: \code{ID}, \code{Pvalue}, \code{Effect} size, 
      \code{Size} (number of members), \code{Conditional} (whether the
      test used the conditional test), and \code{TestDirection} (for up
      or down).
    }
  }
}

\author{ Deepayan Sarkar, Michael Lawrence }

\seealso{
  \code{\link{linearMTest}}
}
\examples{
showClass("LinearMResult")
}
\keyword{classes}

\name{GSEAGOHyperGParams}
\alias{GSEAGOHyperGParams}
\alias{GSEAKEGGHyperGParams}

\title{Helper function for constructing a GOHyperGParams objects or
  KEGGHyperGParams objects from a GeneSetCollection}

\description{
  Helps to create A parameter class for representing all parameters
  needed for running the \code{hyperGTest} method.  If it is a
  GOHyperGParams object, being made, then with one of the GO ontologies
  (BP, CC, MF) as the category. This function will construct the
  parameter object from a GeneSetCollection object and if necessary will
  also try to check to make sure that the object is based on a GO2ALL
  mapping.
}

\usage{
  GSEAGOHyperGParams(name, geneSetCollection, geneIds, universeGeneIds,
  ontology, pvalueCutoff, conditional, testDirection, ...)

  GSEAKEGGHyperGParams(name, geneSetCollection, geneIds, universeGeneIds,
  pvalueCutoff, testDirection, ...)
}

\arguments{
    \item{name}{String specifying name of the GeneSetCollection.}
    \item{geneSetCollection}{A GeneSetCollection Object.  If a
      GOHyperGParams object is sought, then this GeneSetCollection
      should be based on a GO2ALLFrame object and so the idType of that
      GeneSetCollection should be GOAllFrameIdentifier. If a
      KEGGHyperGParams object is sought then a GeneSetCollection based
      on a KEGGFrame object should be used and the idType will be a
      KEGGFrameIdentifier.}
    \item{geneIds}{Object of class \code{"ANY"}: A vector of
      gene identifiers.  Numeric and character vectors are probably the
      only things that make sense.  These are the gene ids for the
      selected gene set.}
    \item{universeGeneIds}{Object of class \code{"ANY"}: A
      vector of gene ids in the same format as \code{geneIds} defining a
      subset of the gene ids on the chip that will be used as the
      universe for the hypergeometric calculation.  If this is
      \code{NULL} or has length zero, then all gene ids on the chip will
      be used.}
    \item{ontology}{A string specifying the GO ontology to use.
      Must be one of "BP", "CC", or "MF". (used with GO only)}
    \item{pvalueCutoff}{A numeric values between zero and one
      used as a p-value cutoff for p-values generated by the
      Hypergeometric test. When the test being performed is
      non-conditional, this is only used as a default value for
      printing and summarizing the results. For a conditional
      analysis, the cutoff is used during the computation to determine
      perform the conditioning: child terms with a p-value less than
      pvalueCutoff are conditioned out of the test for their parent
      term.}
    \item{conditional}{A logical indicating whether the
      calculation should condition on the GO structure. (GO only)}
    \item{testDirection}{A string which can be either "over"
      or "under". This determines whether the test performed detects
      over or under represented GO terms.}
    \item{...}{
      optional arguments to configure the GOHyperGParams object.
    }
    
}
  

\author{M. Carlson}

\seealso{
  \code{\link{HyperGResult-class}}
  \code{\link{GOHyperGParams-class}}
  \code{\link{hyperGTest}}
}

\keyword{classes}

\name{probes2Path}
\alias{probes2Path}
\title{A function to map probe identifiers to pathways. }
\description{
 Given a set of probe identifiers from a microarray this function looks
 up all KEGG pathways that the probe is documented to be involved in.
}
\usage{
probes2Path(pids, data = "hgu133plus2")
}

\arguments{
  \item{pids}{A vector of probe identifiers. }
  \item{data}{The character name of the chip. }
}
\details{
  This is a simple look up in the appropriate chip \code{PATH} data
  environment. 
}
\value{
 A list of pathway vectors. One element for each value of \code{pid}
 that is mapped to at least one pathway.
}
\author{R. Gentleman }

\seealso{ \code{\link{findAMstats}} }
\examples{
 library("hgu95av2.db")
 x = c("1001_at", "1000_at")
 probes2Path(x, "hgu95av2.db")

}
\keyword{manip}

\name{DatPkg-class}
\docType{class}
\alias{DatPkg-class}
\alias{AffyDatPkg-class}
\alias{YeastDatPkg-class}
\alias{ArabidopsisDatPkg-class}
\alias{Org.XX.egDatPkg-class}
\alias{GeneSetCollectionDatPkg}
\alias{OBOCollectionDatPkg-class}
\alias{OBOCollectionDatPkg}

\alias{DatPkgFactory}
\alias{DatPkgFactory,character-method}
\alias{DatPkgFactory,OrgDb-method}
\alias{DatPkgFactory,ChipDb-method}
\alias{DatPkgFactory,OBOCollection,GeneSetCollection-method}


\alias{ID2GO}
\alias{ID2KEGG}
\alias{ID2EntrezID}
\alias{GO2AllProbes}
\alias{KEGG2AllProbes}

\alias{ID2GO,DatPkg-method}
\alias{ID2GO,GeneSetCollectionDatPkg-method}
\alias{ID2KEGG,DatPkg-method}
\alias{ID2KEGG,GeneSetCollectionDatPkg-method}
\alias{ID2EntrezID,AffyDatPkg-method}
\alias{ID2EntrezID,YeastDatPkg-method}
\alias{ID2EntrezID,Org.XX.egDatPkg-method}
\alias{ID2EntrezID,ArabidopsisDatPkg-method}
\alias{ID2EntrezID,GeneSetCollectionDatPkg-method}
\alias{GO2AllProbes,DatPkg-method}
\alias{GO2AllProbes,YeastDatPkg-method}
\alias{GO2AllProbes,Org.XX.egDatPkg-method}
\alias{GO2AllProbes,GeneSetCollectionDatPkg-method}
\alias{KEGG2AllProbes,DatPkg-method}
\alias{KEGG2AllProbes,GeneSetCollectionDatPkg-method}
\alias{isDBDatPkg,DatPkg-method}
\alias{isDBDatPkg,GeneSetCollectionDatPkg-method}
\alias{organism,DatPkg-method}
\alias{organism,GeneSetCollectionDatPkg-method}


\title{Class "DatPkg"}
\description{
  \code{DatPkg} is a \code{VIRTUAL} class for representing annotation
  data packages.

  \code{AffyDatPkg} is a subclass of \code{DatPkg} used to represent
  standard annotation data packages that follow the format of Affymetrix
  expression array annotation.

  \code{YeastDatPkg} is a subclass of \code{DatPkg} used to represent
  the annotation data packages for yeast.  The yeast chip packages are
  based on sgd and are internally different from the \code{AffyDatPkg}
  conforming packages.

  \code{ArabidopsisDatPkg} is a subclass of \code{DatPkg} used to
  represent the annotation packages for Arabidopsis. These packages are
  internally slightly different from the \code{AffyDatPkg} conforming
  packages.

  \code{Org.XX.egDatPkg} is a subclass of \code{DatPkg} used to
  represent the \code{org.*.eg.db} organism-level Entez Gene based
  annotation data packages.
  
  \code{OBOCollectionDatPkg} is a subclass of \code{DatPkg} used to
  represent the \code{OBO} based annotation data packages.
  
  \code{GeneSetCollectionDatPkg} is a subclass of \code{DatPkg} used to
  represent annotations in the form of \code{GeneSetCollection} objects
  which are not based on any annotation packages but are instead derived
  from custom (user supplied) annotations.

  These methods have been extended to accommodate uninstalled annotation
  objects, primarily those available from the AnnotationHub
  package. See below for an example.
  
}
\section{Objects from the Class}{
  A virtual Class: No objects may be created from it.

  Given the name of an annotation data package, \code{DatPkgFactory} can
  be used to create an appropriate \code{DatPkg} subclass.
}
\section{Slots}{
  \describe{
    \item{name}{A string giving the name of the annotation data
      package.}
    \item{db}{The underlying AnnotationDbi database object.}
    \item{installed}{Boolean. Distinguishes between conventional
  installed annotation packages, and those from AnnotationHub.}
  }
}

\section{Methods}{
  See \code{showMethods(classes="DatPkg")}.
  %% FIXME: TODO: add doc for methods
  The set of methods, \code{ID2EntreizID} map between the standard
  IDs for an organism, or Chip and EntrezIDs, typically to give
  a way to get the GO terms.  Different organisms, such as S. cerevisae
  and A. thaliana have their own internal IDs, so we need specialized methods
  for them.
}

\author{Seth Falcon}
\examples{
DatPkgFactory("hgu95av2")
\dontrun{
DatPkgFactory("org.Sc.sgd")
DatPkgFactory("org.Hs.eg.db")
DatPkgFactory("ag")
library(AnnotationHub)
hub <- AnnotationHub()
## get an OrgDb for Atlantic salmon
query(hub, c("salmo salar","orgdb"))
salmodb <- hub[["AH58003"]]
DatPkgFactory(salmodb)
}
}
\keyword{classes}

\name{makeChrBandGraph}
\alias{makeChrBandGraph}

\title{Create a graph representing chromosome band annotation data}
\description{
  This function returns a \code{graph} object representing the nested
  structure of chromosome bands (also known as cytogenetic bands).
  The nodes of the graph are band identifiers.  Each node has a
  \code{geneIds} node attribute that lists the gene IDs that are
  annotated at the band (the gene IDs will be Entrez IDs in most
  cases).
}
\usage{
makeChrBandGraph(chip, univ = NULL)
}

\arguments{
  \item{chip}{A string giving the annotation source.  For example, \code{"hgu133plus2"}}
  \item{univ}{A vector of gene IDs (these should be Entrez IDs for
  most annotation sources).  The annotations
  attached to the graph will be limited to those specified by \code{univ}.
  If \code{univ} is \code{NULL} (default), then the gene IDs are those
  found in the annotation data source.}
}
\details{
  This function parses the data stored in the
  \code{<chip>MAP} map from the appropriate annotation data package.
  Although cytogenetic bands are observed in all organisms, currently,
  only human and mouse band nomenclatures are supported.
}
\value{
  A \code{\link[graph]{graph-class}} instance.  The graph will be a
  tree and the root node is labeled for the organism.
}

\author{Seth Falcon}

\examples{
chrGraph <- makeChrBandGraph("hgu95av2.db")
chrGraph
}

\keyword{manip}

\name{getPathNames}
\alias{getPathNames}
\title{A function to print pathway names given their numeric ID.}
\description{
 Given a KEGG pathway ID this function returns the character name of the
 pathway. 
}
\usage{
getPathNames(iPW, organism = "hsa")
}
\arguments{
  \item{iPW}{A vector of KEGG pathway IDs. }
  \item{organism}{A single character vector of the organism identifier, e.g., "hsa"}
}
\details{
  This function simply does a look up in \code{KEGGPATHID2NAME} and
  returns a list of the pathway names.

  Possible extensions would be to extend it to work with the cMAP
  library as well.
}
\value{
  A list of pathway names.
}

\author{R. Gentleman }

\seealso{\code{\link[KEGG.db]{KEGGPATHID2NAME}} }
\examples{

 nms = "00031"
 getPathNames(nms) 
}
\keyword{manip}

\name{LinearMParams-class}
\docType{class}
\alias{LinearMParams-class}
\alias{annotation<-,LinearMParams,character-method}
\alias{annotation,LinearMParams-method}
\alias{categoryName,LinearMParams-method}
\alias{conditional,LinearMParams-method}
\alias{conditional<-,LinearMParams,logical-method}
\alias{geneIds<-,LinearMParams,ANY-method}
\alias{geneIds,LinearMParams-method}
\alias{pvalueCutoff<-,LinearMParams-method}
\alias{pvalueCutoff,LinearMParams-method}
\alias{show,LinearMParams-method}
\alias{testDirection<-,LinearMParams-method}
\alias{testDirection,LinearMParams-method}
\alias{universeGeneIds,LinearMParams-method}

\title{Class "LinearMParams" }
\description{
  A parameter class for representing all parameters
  needed by a method specializing the \code{\link{linearMTest}} generic.
}

\section{Objects from the Class}{
  Objects can be created by calls of the form
  \code{new("LinearMParams", ...)}.}

\section{Slots}{
  \describe{
    \item{\code{geneStats}:}{Named vector of class \code{"numeric"},
      giving the gene-level statistics to be used in the tests. The
      names should correspond to the gene identifiers in \code{gsc}. }

    \item{\code{universeGeneIds}:}{Object of class \code{"ANY"}: A
      vector of gene ids defining a subset of the gene ids on the chip
      that will be used as the universe for the hypergeometric
      calculation.  If this is \code{NULL} or has length zero, then all
      gene ids on the chip will be used. Currently this parameter is
      ignored by the base \code{linearMTest} method.}

    \item{\code{annotation}:}{A string giving the name of the
      annotation data package for the chip used to generate the data.}
    
    \item{\code{datPkg}:}{Object of class \code{"DatPkg"} used to assist
      with dispatch based on type of annotation data
      available. Currently this parameter is ignored by the
      base \code{linearMTest} method.}
    
    \item{\code{categorySubsetIds}:}{Object of class \code{"ANY"}:
      If the test method supports it, can be used to specify a subset of
      category ids to include in the test instead of all possible
      category ids. Currently this parameter is ignored by the
      base \code{linearMTest} method.}

    \item{\code{categoryName}:}{A string describing the category.
      Usually set automatically by subclasses.  For example
      \code{"ChrMap"}. }

    \item{\code{pvalueCutoff}:}{The p-value to use as a cutoff for
      significance for testing methods that require it.  This value will
      also be passed on to the result instance and used for display and
      counting of significant results.  The default is 0.01. }

    \item{\code{minSize}:}{An integer giving a minimum size for a gene
      set for it to be tested.  The default is 5.  }

    \item{\code{testDirection}:}{A string indicating whether the test
      should test for systematic increase (\code{"up"}) or decrease
      (\code{"down"}) in the \code{geneStats} values within a gene set
      relative to the remaining genes.  }

    \item{\code{graph}:}{The \code{\link[graph:graph-class]{graph}}
      object indicating the hierarchical relationship among terms of the
      ontology or other grouping.
    }

    \item{\code{conditional}:}{A \code{logical} indicating whether
      conditional tests should be performed. This tests whether a term
      is still significant even when including its sub-terms in the model.
    }

    \item{\code{gsc}:}{The
      \code{\link[GSEABase:GeneSetCollection-class]{GeneSetCollection}}
      object grouping the gene ids into sets.
    }
  }
}
\section{Methods}{
  These are accessor methods for the various parameter slots:
  \describe{
    \item{annotation<-}{\code{signature(object = "LinearMParams", value = "character")}: ... }
    \item{annotation}{\code{signature(object = "LinearMParams")}: ... }
    \item{categoryName}{\code{signature(r = "LinearMParams")}: ... }
    \item{conditional}{\code{signature(r = "LinearMParams")}: ... }
    \item{geneIds<-}{\code{signature(object = "LinearMParams")}: ... }
    \item{geneIds}{\code{signature(object = "LinearMParams")}: ... }
    \item{pvalueCutoff<-}{\code{signature(r = "LinearMParams")}: ... }
    \item{pvalueCutoff}{\code{signature(r = "LinearMParams")}: ... }
    \item{show}{\code{signature(object = "LinearMParams")}: ... }
    \item{testDirection<-}{\code{signature(r = "LinearMParams")}: ... }
    \item{testDirection}{\code{signature(r = "LinearMParams")}: ... }
    \item{conditional<-}{\code{signature(r = "LinearMParams")}: ... }
    \item{conditional}{\code{signature(r = "LinearMParams")}: ... }
    \item{universeGeneIds}{\code{signature(r = "LinearMParams")}: ... }
  }
}
\author{ Deepayan Sarkar, Michael Lawrence }

\seealso{
  See \code{\link{linearMTest}} for
  examples. \code{\linkS4class{ChrMapLinearMParams}} is a specialization
  of this class for chromosome maps.
}

\keyword{classes}

\name{linearMTest}
\docType{methods}

\alias{linearMTest}
\alias{linearMTest,LinearMParams-method}
% \alias{linearMTest,ChrMapLinearMParams-method}

\title{A linear model-based test to detect enrichment of unusual genes
  in categories}

\description{

  Given a subclass of \code{LinearMParams}, compute p-values for
  detecting systematic up or downregulation of the specified gene set in
  the specified category.

}

\usage{
  linearMTest(p)
}

\arguments{

  \item{p}{An instance of a subclass of \code{LinearMParams}.  This
    parameter object determines the category of interest (currently,
    only chromosome bands) as well as the gene set.
  }
}

\details{

  The per-gene statistics in the \code{geneStats} slot of \code{p} give
  a measure of up or down regulation of the individual genes in the
  universe.
%   The list of genes is reduced by removing identifiers that do not have
%   any annotations in the specified category.

%% FIXME: more details needed

%   It is important that the correct chip annotation data package be
%   identified as it determines the universe of gene identifiers and is
%   often used to determine the mapping between the category term and the
%   gene identifiers.
  
%   For S. cerevisiae if the \code{annotation} slot of \code{p} is set to
%   '"YEAST"' then comparisons and statistics are computed using common
%   names and are with respect to all genes annotated in the S. cerevisiae
%   genome not with respect to any microarray chip.  This will *not* be
%   the right thing to do if you are working with a yeast microarray.
}
\value{
  A \code{LinearMResult} instance.
}


\author{D. Sarkar}
\seealso{
  \code{\link{LinearMResult-class}}
  \code{\link{LinearMParams-class}}
}
\keyword{htest}

\name{cateGOry}
\alias{cateGOry}
\title{Construct a category membership matrix from a list of gene
  identifiers and their annotated GO categories.}
\description{
  The function constructs a category membership matrix, such as used by
  \code{\link{applyByCategory}},
  from a list of gene identifiers and their annotated GO categories.
  For each of the GO categories stated in \code{categ},
  all less specific terms (ancestors) are also included, thus one need
  only obtain the most specific set of GO term mappings, which
  can be obtained from Bioconductor annotation packages or via \pkg{biomaRt}.
  The ancestor relationships are obtained from the \pkg{GO.db} package.   
}
\usage{
cateGOry(x, categ, sparse=FALSE)
}
\arguments{
  \item{x}{Character vector with (arbitrary) gene identifiers. They will
  be used for the column names of the resulting matrix.}
\item{categ}{A character vector of the same length as \code{x}
    with GO annotations for the genes in \code{x}. If a
    gene has multiple GO annotations, it is expected to occur multiple
    times in \code{x}, once for each different annotation.}
  \item{sparse}{Logical. If \code{TRUE}, the resulting matrix is constructed using \code{\link[Matrix:Matrix]{Matrix}}, 
   otherwise, R's base \code{matrix} is used.}
}
\details{
  The function requires the \code{\link[GO.db:GOBASE]{GO}} package.

  For subsequent analyses, it is often useful to remove categories that
  have only a small number of members. Use the normal matrix subsetting
  syntax for this, see example.

  If a GO category in \code{categ} is not found in the GO annotation
  package, a warning will be generated, and no ancestors
  for that GO category are added (but that category itself will be part
  of the returned adjacency matrix).
}
\value{
  The adjacency matrix of the bipartite category membership graph,
  rows are categories and columns genes.
}

\author{Wolfgang Huber}

\seealso{\code{\link{applyByCategory}}}
\examples{
  g = cateGOry(c("CG2671", "CG2671", "CG2950"),
               c("GO:0090079", "GO:0001738", "GO:0003676"), sparse=TRUE)
  g

  rowSums(g)   ## number of genes in each category

  ## Filter out categories with less than minMem and more than maxMem members.
  ## This is toy data, in real applications, a choice of minMem higher
  ## than 2 will be more appropriate.
  filter = function(x, minMemb = 2, maxMemb = 35) ((x>=minMemb) & (x<=maxMemb))
  g[filter(rowSums(g)),,drop=FALSE ]
}

\keyword{manip}

\name{ttperm}
\alias{ttperm}
\title{A simple function to compute a permutation t-test. }
\description{
  The data matrix, \code{x}, with two-level factor, \code{fac}, is used
  to compute t-tests. The values of \code{fac} are permuted \code{B}
  times and the complete set of t-tests is performed for each
  permutation. 
}
\usage{
ttperm(x, fac, B = 100, tsO = TRUE)
}
\arguments{
  \item{x}{ A data matrix. The number of columns should be the same as
    the length of \code{fac}. }
  \item{fac}{A factor with two levels. }
  \item{B}{An integer specifying the number of permutations. }
  \item{tsO}{A logical indicating whether to compute only the t-test
    statistic for each permuation. If \code{FALSE} then p-values are
    also computed - but this can be very slow. }
}
\details{
 Not much more to say. Probably there is a generic function somewhere,
 but I could not find it.
}
\value{
 A list, the first element is named \code{obs} and contains the true,
 observed, values of the t-statistic. The second element is named
 \code{ans}  and contains a list of length \code{B} containing the
 different permuations. 
}

\author{R. Gentleman }

\seealso{\code{\link[genefilter:rowFtests]{rowttests}} }
\examples{
   x=matrix(rnorm(100), nc=10)
   y = factor(rep(c("A","B"), c(5,5)))
   ttperm(x, y, 10)

}
\keyword{manip}

\name{ChrMapLinearMResult-class}
\docType{class}
\alias{ChrMapLinearMResult-class}

% deprecated/hidden
\alias{chrGraph,ChrMapLinearMResult-method}

\title{Class "ChrMapLinearMResult" }
\description{

  This class represents the results of a linear model-based test for
  systematic changes in a per-gene statistic by chromosome band
  annotation.  The \code{\link{linearMTest}} function returns an
  instance of \code{ChrMapLinearMResult} when given a parameter object
  of class \code{ChrMapLinearMParams}.  Most slots can be queried using
  accessors.  }

\section{Objects from the Class}{

  Objects can be created by calls of the form
  \code{new("ChrMapLinearMResult", ...)}, but is more commonly created
  by callinf \code{\link{linearMTest}}

}

\section{Slots}{
  \describe{
   \item{\code{pvalues}:}{Object of class \code{"numeric"}, with the
      p-values for each term.
    }
    \item{\code{pvalue.order}:}{Object of class \code{"integer"}, the
      order vector (increasing) for the p-values.
    }
    \item{\code{effectSize}:}{Object of class \code{"numeric"}, with
      the effect size for each term.
    }
    \item{\code{annotation}:}{Object of class \code{"character"} ~~ }
    \item{\code{geneIds}:}{Object of class \code{"ANY"} ~~ }
    \item{\code{testName}:}{Object of class \code{"character"} ~~ }
    \item{\code{pvalueCutoff}:}{Object of class \code{"numeric"} ~~ }
    \item{\code{minSize}:}{Object of class \code{"integer"} ~~ }
    \item{\code{testDirection}:}{Object of class \code{"character"} ~~ }
    \item{\code{conditional}:}{Object of class \code{"logical"} ~~ }
    \item{\code{graph}:}{Object of class \code{"graph"} ~~ }
    \item{\code{gsc}:}{Object of class \code{"GeneSetCollection"} ~~ }
  }
}
\section{Extends}{
  Class \code{"\linkS4class{LinearMResult}"}, directly.

  Class \code{"\linkS4class{LinearMResultBase}"}, by class
  "LinearMResult", distance 2.
}
\section{Methods}{
  None
}
\author{ Deepayan Sarkar, Michael Lawrence }
\seealso{
  \code{\link{linearMTest}}, \code{\linkS4class{ChrMapLinearMParams}},
  \code{\linkS4class{LinearMResult}},
  \code{\linkS4class{LinearMResultBase}},
}
\examples{
showClass("ChrMapLinearMResult")
}
\keyword{classes}

\name{exampleLevels}
\alias{exampleLevels}

\title{Display a sample node from each level of a ChrBandTree object}
\description{
  The "levels" of a chromosome band tree represented by a \code{ChrBandTree} object
  are the sets of nodes with a given path length to the root node.  This
  function displays the available levels along with an example node from
  each level.
}
\usage{
exampleLevels(g)
}

\arguments{
  \item{g}{A \code{ChrBandTree} object}
}
\value{
  A list with an element for each level.  The names of the list are the
  levels.  Each element is an example of a node from the given level.
}

\author{S. Falcon}
% R documentation directory.

\name{makeEBcontr}
\alias{makeEBcontr}
\title{ A function to make the contrast vectors needed for EBarrays }
\description{
 Using EBarrays to detect differential expression requires the
 construction of a set of contrasts. This little helper function
 computes these contrasts for a two level factor.
}
\usage{
makeEBcontr(f1, hival)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{f1}{The factor that will define the contrasts.}
  \item{hival}{The \code{level} of the factor to treat as the high level.}
}
\details{
  Not much more to add, see EBarrays for more details. This is used in
  the Category package to let users compute the posterior probability of
  differential expression, and hence to compute expected numbers of
  differentially expressed genes, per category.
}
\value{
An object of class  ``ebarraysPatterns''.
}

\author{R. Gentleman }

\seealso{ \code{\link[EBarrays:utilities]{ebPatterns}}}
\examples{
 if( require("EBarrays") ) {
  myfac = factor(rep(c("A", "B"), c(12, 24)))
  makeEBcontr(myfac, "B")
 }
}
\keyword{manip}

\name{cb_parse_band_Mm}
\alias{cb_parse_band_Mm}

\title{Parse Mus Musculus Chromosome Band Annotations}
\description{
  This function parses chromosome band annotations as found in the
  <chip>MAP map of Bioconductor annotation data packages.  The return
  value is a vector of parent bands up to the relevant chromosome.
}
\usage{
cb_parse_band_Mm(x)
}

\arguments{
  \item{x}{A chromosome band annotation given as a string.}
}

\value{
  A character vector giving the path to the relevant chromosome.
}

\author{Seth Falcon \& Nolwenn Le Meur}

\examples{
cb_parse_band_Mm("10 B3")
}

\name{ChrMapLinearMParams-class}
\docType{class}
\alias{ChrMapLinearMParams-class}

\title{Class "ChrMapLinearMParams"}
\description{

  This class encapsulates parameters needed for testing systematic
  variations in some gene-level statistic by chromosome bands using
  \code{\link{linearMTest}}.

}
\section{Objects from the Class}{
  Objects can be created by calls of the form
  \code{new("ChrMapLinearMParams", ...)}.
}
\section{Slots}{
  \describe{

    \item{\code{graph}:}{Object of class \code{"graph"}.  The nodes
      are the chromosome bands and the edges describe the tree structure
      of the bands.  Each node has a "geneIds" node attributes (see
      \code{nodeData}) which contains a vector of gene IDs annotated at
      the given band.}

    \item{\code{conditional}:}{Object of class \code{"logical"},
      indicating whether the test performed should be a conditional
      test.  }

    \item{\code{gsc}:}{The
      \code{\link[GSEABase:GeneSetCollection-class]{GeneSetCollection}}
      object grouping the gene ids into sets.
    }
    
    \item{\code{geneStats}:}{Named vector of class \code{"numeric"},
    giving the gene-level statistics to be used in the tests.  }
    
    \item{\code{universeGeneIds}:}{Object of class \code{"ANY"}: A
      vector of gene ids defining a subset of the gene ids on the chip
      that will be used as the universe for the hypergeometric
      calculation.  If this is \code{NULL} or has length zero, then all
      gene ids on the chip will be used.}

    \item{\code{annotation}:}{A string giving the name of the
      annotation data package for the chip used to generate the data.}

    \item{\code{datPkg}:}{Object of class \code{"DatPkg"} used to assist
      with dispatch based on type of annotation data available. }

    \item{\code{categorySubsetIds}:}{Object of class \code{"ANY"}:
      If the test method supports it, can be used to specify a subset of
      category ids to include in the test instead of all possible
      category ids.}

    \item{\code{categoryName}:}{A string describing the category.
      Usually set automatically by subclasses.  For example
      \code{"GO"}.}

    \item{\code{pvalueCutoff}:}{The p-value to use as a cutoff for
      significance for testing methods that require it.  This value will
      also be passed on to the result instance and used for display and
      counting of significant results.  The default is 0.01.}

    \item{\code{minSize}:}{An integer giving a minimum size for a gene
      set for it to be tested.  The default is 5. }

    \item{\code{testDirection}:}{A string indicating whether the test
      should test for systematic increase (\code{"up"}) or decrease
      (\code{"down"}) in the \code{geneStats} values within a gene set
      relative to the remaining genes.  }

  }
}
\section{Extends}{
  Class \code{"\linkS4class{LinearMParams}"}, directly.
}
       
\author{ Deepayan Sarkar }

\seealso{
  \code{\link{linearMTest}}
}

\examples{
showClass("ChrMapLinearMParams")
}

\keyword{classes}

\name{findAMstats}
\alias{findAMstats}
\title{Compute per category summary statistics }
\description{
 For a given incidence matrix, \code{Amat}, compute some per category
 statistics. 
}
\usage{
findAMstats(Amat, tstats)
}
\arguments{
  \item{Amat}{An incidence matrix, with categories as the rows and
    probes as the columns. }
  \item{tstats}{A vector of per probe test statistics (should be the
    same length as \code{ncol(Amat)}. }
}
\details{
  Simple summary statistics are computed, such as the row sums and the
  vector of per category sums of the test statistics, \code{tstats}.
}
\value{
A list with components,
  \item{eDE }{per category sums of the test statistics}
  \item{lens }{row sums of \code{Amat}}
}

\author{R. Gentleman }

\seealso{\code{\link{applyByCategory}}}
\examples{

 ts = rnorm(100)
 Am = matrix(sample(c(0,1), 1000, replace=TRUE), ncol=100)
 findAMstats(Am, ts)
}

\keyword{manip}

\name{ChrBandTree-class}
\docType{class}
\alias{ChrBandTree-class}

\alias{allGeneIds}
\alias{childrenOf}
\alias{parentOf}
\alias{lgeneIds}
\alias{treeLevels}
\alias{level2nodes}

\alias{allGeneIds,ChrBandTree-method}
\alias{childrenOf,ChrBandTree,character-method}
\alias{geneIds,ChrBandTree-method}
\alias{lgeneIds,ChrBandTree-method}
\alias{parentOf,ChrBandTree,character-method}
\alias{treeLevels,ChrBandTree-method}
\alias{level2nodes,ChrBandTree,numeric-method}
\alias{level2nodes,ChrBandTree,character-method}

\alias{show,ChrBandTree-method}

\title{Class "ChrBandTree"}
\description{
  This class represents chromosome band annotation data for a
given experiment.  The class is responsible for storing the mapping of
band to set of gene IDs located within that band as well as for
representing the tree structured relationship among the bands.

}

\section{Objects from the Class}{
Objects should be created using \code{NewChrBandTree} or
\code{ChrBandTreeFromGraph}.
}
\section{Slots}{
	 \describe{
    \item{\code{toParentGraph}:}{Object of class \code{"graph"}
    representing the tree of chromosome bands.  Edges in this directed
    graph go from child to parent.}
    \item{\code{toChildGraph}:}{Object of class \code{"graph"}.  This is
    the same as \code{toParentGraph}, but with the edge directons
    reversed.  This is not an ideal implementation due to the
    duplication of data, but it provides quick access to parents or
    children of a given node.}
    \item{\code{root}:}{Object of class \code{"character"} giving the
    name of the root node.  The convention is to use "ORGANISM:<organism>".}
    \item{\code{level2nodes}:}{Object of class \code{"list"} providing a
    mapping of levels in the tree to the set of nodes at that level.
    Levels \code{X} is defined as the set of nodes with a path length of
    \code{X} from the root node.}
  }
}
\section{Methods}{
  \describe{

    \item{allGeneIds}{Return a vector of gene IDs representing the gene
    universe for this \code{ChrBandTree}}

  \item{childrenOf}{Return a list with an element for each the
    character vector \code{n}.  Each element is a character vector of
    node names of the children of the named element.}

  \item{geneIds}{Return a vector of gene IDs for a single band.}
  
  \item{lgeneIds}{Return a list of vectors of gene IDs when given more
    than one band.  The "l" prefix is for list.}
  
  \item{parentOf}{Return the parents of the specified bands.  See
    \code{childrenOf} for a description of the structure of the return
    value.}

  \item{treeLevels}{Return an integer vector identifying the levels of
    the tree.}

  \item{level2nodes(g, level)}{Return the nodes in the tree that are at
    the level specified by \code{level}.  The \code{level} argument can
    be either numeric or character, but should match a level returned by
    \code{treeLevels}.}
}
}
\author{S. Falcon}
\note{
  Not all known chromosome bands will be represented in a given
  instance.  The set of bands that will be present is determined by the
  available annotation data and the specified gene universe.  The
  annotation source maps genes to their most specific band.  Such bands
  and all bands on the path to the root will be represented in the
  resulting tree.

  Currently there is only support for human and mouse data.
}

\examples{
library("hgu95av2.db")
set.seed(0xfeee)
univ = NULL ## use all Entrez Gene IDs on the chip (not recommended)
ct = NewChrBandTree("hgu95av2.db", univ)

length(allGeneIds(ct))

exampleLevels(ct)

geneIds(ct, "10p11")
lgeneIds(ct, "10p11")
lgeneIds(ct, c("10p11", "Yq11.22"))

pp = parentOf(ct, c("10p11", "Yq11.22"))
childrenOf(ct, unlist(pp))

treeLevels(ct)

level2nodes(ct, 0)
level2nodes(ct, 0L)
level2nodes(ct, "0")

level2nodes(ct, 1)

}
\keyword{classes}

\name{cb_contingency}
\alias{cb_contingency}
\alias{cb_sigBands}
\alias{cb_children}

\title{Create and Test Contingency Tables of Chromosome Band Annotations}
\description{
  For each chromosome band identifier in \code{chrVect},
  \code{cb_contingency} builds and performs a test on a 2 x k
  contingency table for the genes from \code{selids} found in the child
  bands of the given \code{chrVect} element.

  \code{cb_sigBands} extracts the chromosome band identifiers that were
  in a contingency table that tested significant given the specified
  p-value cutoff.

  \code{cb_children} returns the child bands of a given band in the
  chromosome band graph.  The argument must have length equal to one.
}
\usage{
cb_contingency(selids, chrVect, chrGraph, testFun = chisq.test,
               min.expected = 5L, min.k = 1L)

cb_sigBands(b, p.value = 0.01)

cb_children(n, chrGraph)
}

\arguments{
  \item{selids}{A vector of the selected gene identifiers (usual Entrez IDs).}
  \item{chrVect}{A character vector of chromosome band identifiers}
  \item{chrGraph}{A \code{graph} object as returned by
    \code{makeChrBandGraph}.  The nodes should be chromosome band IDs
    and the edges should represent the tree structure of the bands.
    Furthermore, the graph is expected to have a \code{"geneIds"} node
    attribute providing a vector of gene IDs annotated at each band.}
  \item{testFun}{The function to use for testing the 2 x k contingency
  tables.  The default is \code{chisq.test}.  It will be called with a
  single argument, a 2 x k matrix representing the contingency table.}
  \item{min.expected}{A numeric value specifying the minimum expected
    count for columns to be included in the contingency table.  The
    expected count is \code{(rowSum * colSum) / n}.  Chromosome bands
    with a select cell count less than \code{min.expected} are dropped
    from the table before testing occurs.  If \code{NULL}, then no bands
    will be dropped.}
  \item{min.k}{An integer giving the minimum number of chromosome bands
    that must be present in a contingency table in order to proceed with
    testing.}
  \item{b}{A list as returned by \code{cb_contingency}}
  \item{p.value}{A p-value cutoff to use in selecting significant
    contingency tables.}
  \item{n}{A length one character vector specifying a chromosome band
    annotation.  Bands not found in \code{chrGraph} will return
    \code{character(0)} when passed to \code{cb_children}.}

}

\details{
  \code{cb_sigBands} assumes that the p-value associated with a result
  of \code{testFun} can by accessed as \code{testFun(t)$p.value}.  We
  should improve this to be a method call which can then be specialized
  based on the class of the object returned by \code{testFun}.
}

\value{

  \code{cb_contingency} returns a list with an element for each test
  performed.  This will most often be shorter than
  \code{length(chrVect)} due to skipped tests based on \code{min.found}
  and \code{min.k}.  Each element of the returned list is itself a list
  with components:
  \item{table}{A 2 x k contingency table}
  \item{result }{The output of \code{testFun} applied to the \code{table}.}

  \code{cb_sigBands} returns a character vector of chromosome band
  identifiers that are in one of the contingency tables that had a
  p-value less than the cutoff specified by \code{p.value}.
}

\author{Seth Falcon}

\name{HyperGResultBase-class}
\docType{class}
\alias{HyperGResultBase-class}

\alias{conditional,HyperGResultBase-method}

\alias{show,HyperGResultBase-method}


\title{Class "HyperGResultBase"}
\description{	 
  This VIRTUAL class represents common elements of the return values
  of generic functions like \code{hyperGTest}.  All subclasses are
  intended to implement the accessor functions documented at
  \link{HyperGResult-accessors}.
}
\section{Objects from the Class}{A virtual Class: No objects may be
  created from it.} 
\section{Slots}{
	 \describe{

         \item{\code{annotation}:}{Object of class \code{"character"}
             giving the name of the annotation data package used. }

         \item{\code{geneIds}:}{Object of class \code{"ANY"} (usually
             a character vector, but sometimes an integer vector).
             The input vector of gene identifiers intersected with the
             universe of gene identifiers used in the computation.  }

         \item{\code{testName}:}{Object of class \code{"character"}
             identifying the testing method used.
           }

         \item{\code{pvalueCutoff}:}{Numeric value used by the
             testing method as a p-value cutoff.  Not all testing
             methods use this.  Also used by the \code{show} method to
             count number of significant terms.  }

         \item{\code{testDirection}:}{A string indicating whether the test
             performed was for overrepresentation (\code{"over"}) or
             underrepresentation(\code{"under"}).}

  }
}
\section{Methods}{
  See \link{HyperGResult-accessors}.
}

\author{Seth Falcon}


\seealso{
  \code{\link{HyperGResult-class}}
  \code{\link[GOstats]{GOHyperGResult-class}}
  \link{HyperGResult-accessors}
}

\keyword{classes}

\name{gseattperm}
\alias{gseattperm}

\title{Permutation p-values for GSEA}
\description{
  This function performs GSEA computations and returns p-values
  for each gene set based on repeated permutation of the phenotype
  labels.
}
\usage{
gseattperm(eset, fac, mat, nperm)
}

\arguments{
  \item{eset}{An \code{ExpressionSet} object}
  \item{fac}{A \code{factor} identifying the phenotypes in
    \code{eset}.  Usually, this will be one of the columns in the
    phenotype data associated with \code{eset}.}
  \item{mat}{A 0/1 incidence matrix with each row representing a gene
    set and each column representing a gene.  A 1 indicates membership
    of a gene in a gene set.}
  \item{nperm}{Number of permutations to test to build the reference
    distribution.}
}
\details{
  The t-statistic is used (via \code{rowttests}) to test for a
  difference in means between the phenotypes determined by \code{fac}
  within each gene set (given as a row of \code{mat}).

  A reference distribution for these statistics is established by
  permuting \code{fac} and repeating the test \code{B} times.
}
\value{
  A matrix with the same number of rows as \code{mat} and two columns,
  \code{"Lower"} and \code{"Upper"}.  The \code{"Lower"}
  (\code{"Upper"}) column gives the probability of seeing a t-statistic
  smaller (larger) than the observed.
}

\author{Seth Falcon}

\examples{
## This example uses a random sample of probesets and a randomly
## generated category matrix.  The results, therefore, are not
## meaningful, but the code demonstrates how to use gseattperm without
## requiring any expensive computations.

## Obtain an ExpressionSet with two types of samples (mol.biol)
haveALL <- require("ALL")
if (haveALL) {
data(ALL)
set.seed(0xabcd)
rndIdx <- sample(1:nrow(ALL), 500)
Bcell <- grep("^B", as.character(ALL$BT))
typeNames <- c("NEG", "BCR/ABL")
bcrAblOrNegIdx <- which(as.character(ALL$mol.biol) \%in\% typeNames)
s <- ALL[rndIdx, intersect(Bcell, bcrAblOrNegIdx)]
s$mol.biol <- factor(s$mol.biol)

## Generate a random category matrix
nCats <- 100
set.seed(0xdcba)
rndCatMat <- matrix(sample(c(0L, 1L), replace=TRUE),
                    nrow=nCats, ncol=nrow(s),
                    dimnames=list(
                      paste("c", 1:nCats, sep=""),
                      featureNames(s)))

## Demonstrate use of gseattperm
N <- 10
pvals <- gseattperm(s, s$mol.biol, rndCatMat, N)
pvals[1:5, ]
}
}

\name{HyperGResult-class}
\docType{class}
\alias{HyperGResult-class}
\alias{KEGGHyperGResult-class}
\alias{PFAMHyperGResult-class}

\title{Class "HyperGResult"}
\description{
  This class represents the results of a test for over-representation of
  categories among genes in a selected gene set based upon the
  Hypergeometric distribution.  The \code{hyperGTest}
  generic function returns an instance of the
  \code{HyperGResult} class. For details on accessing
  the results, see \link{HyperGResult-accessors}.
}

\section{Objects from the Class}{
Objects can be created by calls of the form \code{new("HyperGResult", ...)}.
}
\section{Slots}{
	 \describe{
    \item{\code{pvalues}:}{\code{"numeric"} vector: the ordered
      p-values for each category term tested.}
    \item{\code{catToGeneId}:}{Object of class \code{"list"}.  The
        names of the list are category IDs.  Each element is a vector
        of gene IDs annotated at the given category ID and in the
        specified gene universe.}
    \item{\code{annotation}:}{A string giving the name of the chip
      annotation data package used.}
    \item{\code{geneIds}:}{Object of class \code{"ANY"}: the input
      vector of gene identifiers intersected with the universe of gene
      identifiers used in the computation.  The class of this slot is
      specified as \code{"ANY"} because gene IDs may be integer or
      character vectors depending on the annotation package.}
    \item{\code{testName}:}{A string identifying the testing method
      used.}
    \item{\code{pvalueCutoff}:}{Numeric value used a a p-value
        cutoff.  Used by the \code{show} method to count number of
        significant terms.
      }

      \item{\code{testDirection}:}{A string indicating whether the
          test should be for overrepresentation (\code{"over"}) or
          underrepresentation (\code{"under"}).}

  \item{\code{oddsRatios}}{a \code{"numeric"} vector giving
      the odds ratio for each category term tested.}

  \item{expectedCounts}{a \code{"numeric"} vector
      giving the expected number of genes in the selected gene list to
      be found at each tested category term.}


    }
  }

\section{Extends}{
Class \code{"HyperGResultBase"}, directly.
}
\section{Methods}{
See \link{HyperGResult-accessors}.
}

\author{Seth Falcon}

\seealso{
  \code{\link{HyperGResultBase-class}}
  \code{\link[GOstats]{GOHyperGResult-class}}
  \link{HyperGResult-accessors}
}

\keyword{classes}
